<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 02.01.2019
 */
//declare(strict_types=1);
namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\Controls\Options;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\App;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Mzdy\NastaveniOdpoctu;
use RonPortal\Personalistika\ZdravotniStavOsoby;
use RonPortal\System\Report;
use RonPortal\Zamestnanci\AdresaOsoby;
use function Ron\ronText;
use stdClass;


require_once('mzdy/dane_dokumenty.php');


/**
 * Class DaneController
 * @package Ron\RonPortal
 */
class DaneController extends JsonController
{

    /** @var Database */
    public $database;

    /** @var PracovniPomer $pracovniPomer */
    public $pracovniPomer;


    /** @var PravaNaOsoby $pravo_vyplnit */
    public $pravo_prohlaseni;

    /** @var PravaNaOsoby $pravo_vyuctovani */
    public $pravo_vyuctovani;


    /**
     * DaneController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        $this->pravo_prohlaseni = Prava::vratPrava('dane_prohlaseni');
        $this->pravo_vyuctovani = Prava::vratPrava('dane_vyuctovani');
    }

    /**
     * @param int $rok Rok vyuctovani dane
     * @return bool
     * @throws Exception
     */
    public static function lzeEditovatVyuctovani($rok)
    {
        if (RonPortalSettings::getIntValue('zadostizuctovanidanepoterminu')) {
            return true;
        }

        //editace vyuctovani za zvoleny rok je mozna od 1.1. nasledujiciho roku
        $editovatOd = DateUtils::incMonth($rok, 12);
        //do zvoleneho terminu v danem (nasledujicim) roce (napr. 28.2.) - pokud neni zadano bere se 31.3.
        $editovatDo = DateUtils::dateStringToTimestamp(RonPortalSettings::getValue('daneterminvyplneni', '31.3.') . date('Y', $editovatOd));

        if ((DateUtils::today() >= $editovatOd) && (DateUtils::today() <= $editovatDo)) {
            return true;
        }

        return false;
    }

    public static function vratTypDokumentuProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case 'predchozizamestnavatel' :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_typ');
            case 'slevanapartnera' :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ');
            case 'skolkovne' :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ');
            case 'odecitatelnepolozky' :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ');
            //prohlaseni
            case 'deti' :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
            case 'partner' :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_typ');
            case 'odecet' :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
        }
    }

    /**
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return int|mixed
     */
    public static function povinnyDokumentProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case 'predchozizamestnavatel' :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne');
            case 'slevanapartnera' :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne');
            case 'skolkovne' :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne');
            case 'odecitatelnepolozky' :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_povinne');
            //prohlaseni
            case 'deti' :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne');
            case 'partner' :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne');
            case 'odecet' :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne');
        }
    }

    /**
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return string
     */
    public static function vratTypFormulareDleSekce($sekce)
    {
        switch ($sekce) {
            case 'predchozizamestnavatel' :
            case 'slevanapartnera' :
            case 'skolkovne' :
            case 'odecitatelnepolozky' :
                return 'formularzadosti';
            case 'deti' :
            case 'partner' :
            case 'odecet' :
            default :
                return 'formularprohlaseni';
        }
    }

    /**
     * @param Database $database
     * @param $rok
     * @param $pracovniPomer
     * @param string $sekceFormulare ke ktere casti jsou dokumenty mozne hodnoty: deti, partner, odecet
     * @param $povolitEditaci
     * @param $kontrolovatPovinnost
     * @throws DBException
     */
    public static function zobrazDokumenty($database, $rok, $pracovniPomer, $sekceFormulare, $povolitEditaci, $kontrolovatPovinnost = true, $castiSekce = null)
    {
        \RonPortal\Mzdy\DaneController::zobrazDokumenty($database, $rok, $pracovniPomer, $sekceFormulare, $povolitEditaci, $kontrolovatPovinnost, $castiSekce);
    }

    /**
     * @param Database $database
     * @param int $rok
     * @param \RonPortal\PracovniPomer $pracovniPomer
     * @param string $castProhlaseni
     * @param string $popisDokumentu
     * @throws DBException
     */
    public static function zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, $castProhlaseni, $popisDokumentu)
    {
        \RonPortal\Mzdy\DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, $castProhlaseni, $popisDokumentu);
        return;

        $typDokumentu = self::vratTypDokumentuProSekci($castProhlaseni);

        if ($typDokumentu) {
            $fields = '';
            $popisTypu = '';

            if (RonPortalSettings::getIntValue('dane_dokumenty_kopirovat_popis')) {
                $fields = ';popis';
                $params = $database->createParameters();
                $popisTypu = DBUtils::getSqlValue($database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            }

            $params = $database->createParameters();
            $sql =
                "SELECT Popis FROM Dokument JOIN DokumentyOsoby ON Dokument.id = DokumentyOsoby.id 
                WHERE DokumentyOsoby.VyuctovaniDane_RC = " . $params->addParameter($pracovniPomer->rc) .
                "  AND DokumentyOsoby.VyuctovaniDane_Rok = " . $params->addDateParameter(DateUtils::dateStringToTimestamp("1.1.$rok")) .
                "  AND Dokument.typ = " . $params->addParameter($typDokumentu) .
                "  AND Dokument.Popis LIKE " . $params->addParameter("%{$popisDokumentu}%") .
                " ORDER BY Dokument.id DESC";
            $posledniPopis = DBUtils::getSqlValue($database, $sql, $params);

            $posledniCislo = substr($posledniPopis, -1);
            $noveCislo = (int)$posledniCislo + 1;

            echo
                "<a href='#' title='" . TextManager::_('Vlozit_novy_dokument') . "'>" .
                    "<i class='inline-glyph fa glyph-upload show-dokument-dialog' data-fields='dokumentprohlasenityp{$fields}' data-field-dokumentprohlasenityp='{$castProhlaseni}'" .
                        "data-field-popis='{$popisTypu} - {$popisDokumentu} {$noveCislo}'></i> " .
                "</a>";
        }
    }

    /**
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @param array $historieOdpoctuNaDeti
     * @return bool
     */
    public function jePovinnyDokument($hodnotyProhlaseni, $historieOdpoctuNaDeti)
    {
        $povinnyDokumentDeti = false;
        if (self::povinnyDokumentProSekci('deti')) {
            $povinnyDokumentDeti = true;

            $odpocetNaDetiZmeny = $historieOdpoctuNaDeti['zmeny'];

            if (empty($odpocetNaDetiZmeny) || count($odpocetNaDetiZmeny) == 0)
                $povinnyDokumentDeti = false;

            $uplatnujeOdpocetNaDite = false;
            foreach ($odpocetNaDetiZmeny as $dite) {
                if (($dite['datumOdNavrh'] != '' && $dite['poradiDitete_navrh'] != '') || ($dite['datumOd'] != '' && $dite['poradiDitete'] != '')) {
                    $uplatnujeOdpocetNaDite = true;
                    break;
                }
            }
            if (!$uplatnujeOdpocetNaDite)
                $povinnyDokumentDeti = false;
        }

        if ($povinnyDokumentDeti
            || self::povinnyDokumentProSekci('partner')
            || (self::povinnyDokumentProSekci('odecet') && (($hodnotyProhlaseni->naZdravotniStav != $hodnotyProhlaseni->naZdravotniStavNavrh)
                 || ($hodnotyProhlaseni->odpocetNaStudenta != $hodnotyProhlaseni->naZdravotniStavNavrh))))
            return true;

        return false;
    }

    public function zkontrolujPovinneDokumentyProhlaseni($rok, $hodnoty)
    {
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne') && ($hodnoty['pocetdetipoplatnika'] > 0)) {
            $diteStr = isset($hodnoty['pozaduji_upravu_poradi_ditete']) ? 'poradi_ditete_navrh_' : 'poradi_ditete_old_';
            $uplatnujeOdecet = false;
            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {
                if ($hodnoty["$diteStr{$i}"] > 0)
                    $uplatnujeOdecet = true;
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeOdecet) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        //TODO: kontrolovat povinnost jen pokud je zadana zmena na invaliditu nebo studenta
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne') && $this->jeZmenaSlev($hodnoty))
            $this->jePripojenyDokument($typ, $rok);

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_partner_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne') && $hodnoty['vyzivujeJinyPoplatnik'] == 1)
            $this->jePripojenyDokument($typ, $rok);
    }

    public function zkontrolujPovinneDokumentyZmenyProhlaseni($rok, $hodnoty)
    {
        //$hodnoty je objekt!!!

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne') &&  ((int)$hodnoty->pocetDetiZmenyPoradi > 0) && ($hodnoty->zmenova_cast_pozadujiUpravuPoradiDeti)) {

            $uplatnujeOdecet = false;
            foreach ($hodnoty->zmenova_cast_poradiDanovehoZvyhodneniNaDeti as $rcDitete => $novePoradi) {
                if ($novePoradi > 0)
                    $uplatnujeOdecet = true;
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeOdecet) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        //TODO: kontrolovat povinnost jen pokud je zadana zmena na invaliditu nebo studenta
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne') && ($hodnoty->nastalaZmenaOdpoctu && $hodnoty->zmenova_cast_pozadujiUpravuSlev == 1))
            $this->jePripojenyDokument($typ, $rok);
    }

    public function zkontrolujPovinneDokumentyVyuctovani($rok, $hodnoty)
    {
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne') && ($hodnoty['pocetdetipoplatnika'] > 0)) {
            $uplatnujeSkolkovne = false;
            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {
                $rcDitete = $hodnoty['ditepoplatnika_' . $i];
                $skolkovneNaDite = $hodnoty['skolkovnenadite_' . $rcDitete];
                if ($skolkovneNaDite > 0)
                    $uplatnujeSkolkovne = true;
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeSkolkovne) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

       $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ'))
            $this->jePripojenyDokument($typ, $rok);

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne'))
            $this->jePripojenyDokument($typ, $rok);
    }

    public function jePripojenyDokument($typDokumentu, $rok)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentyOsoby.Id ' .
            '  FROM DokumentyOsoby JOIN Dokument ON (Dokument.Id = DokumentyOsoby.Id) ' .
            ' WHERE DokumentyOsoby.RC =  ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND DokumentyOsoby.VyuctovaniDane_Rok = ' . $params->addParameter($this->database->dateToParam($rok)) .
            '   AND DokumentyOsoby.VyuctovaniDane_RC = ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND Dokument.typ = ' . $params->addParameter($typDokumentu);

        if (!$cursor = $this->database->select($sql, $params)) {
            throw new DBException($this->database, $sql);
        }

        $row = $cursor ->fetch();
        if (!$row) {
            $params = $this->database->createParameters();
            $typDokumentuPopis = DBUtils::getSqlValue($this->database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            throw new Exception(sprintf(ronText('Prohlaseni_neexistuje_povinny_dokument'), $typDokumentuPopis));
        }
    }

    public function jeZmenaSlev($hodnoty)
    {
        if (isset($hodnoty['pozaduji_upravu_slev']) && ((($hodnoty['sleva_na_zdravotni_stav1'] == 1) && ($hodnoty['sleva_na_zdravotni_stav1'] != $hodnoty['sleva_na_zdravotni_stav1_old']))
            || (($hodnoty['sleva_na_zdravotni_stav2'] == 1) && ($hodnoty['sleva_na_zdravotni_stav2'] != $hodnoty['sleva_na_zdravotni_stav2_old']))
            || (($hodnoty['sleva_ztpp'] == 1) && ($hodnoty['sleva_ztpp'] != $hodnoty['ztpp']))
            || (($hodnoty['sleva_na_studenta_nova'] == 1) && ($hodnoty['sleva_na_studenta_nova'] != $hodnoty['odpocetNaStudenta_old']))))
           return true;

        return false;
    }

    /**
     * @param PracovniPomer $pp
     * @param $obdobi
     * @param string $logDescription
     * @param bool $pouzeOdpocet
     * @throws DBException
     */
    public function ukliditNavrhyRocnihoVyuctovaniDane($pp, $obdobi, $logDescription='', $pouzeOdpocet=false)
    {
        $zacatekRoku = DateUtils::firstDayOfYear(time());
        $konecRoku = (mktime(0,0,0,12,31,(int)date("Y", (int)time())));

        require_once('adsapp/ronportaldbrecord.php');
        //Odstraneni navrhu odpoctu - pred vytvorenim zaznamu rocniho vyuctovani dane, nebo "odvolani" u zmeny odpoctu
        $rec = new RonPortalDBRecord($this->database, 'OdpocetNavrh');
        $params = $this->database->createParameters();

        $rec->logDescription = $logDescription;
        $rec->addField('DatumVlozeniNavrhu', 'timestamp', false);
        $rec->addField('RC', '', false);
        $rec->addField('OsobniCislo', '', false);
        $rec->addField('DatumNastupu', 'timestamp', false);
        $rec->addField('DatumOd', 'timestamp', false);
        $rec->addField('ZakladniOdpocet', '', false);
        $rec->addField('OdpocetNaStudenta', '', false);
        $rec->addField('NaZdravotniStav', '', false);
        $rec->addField('Stav', '', false);

        $rec->setWhere( " DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam($zacatekRoku)) .
            " AND DatumVlozeniNavrhu < " . $params->addParameter($this->database->timestampToParam($konecRoku)) .
            ' AND RC  = ' . $params->addParameter($pp->rc)
            , $params);

        $rec->checkMultiple = false;
        if ($rec->loadRecord()) {
            if (!$rec->deleteRecords()) {
                MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri odstraneni navrhu odpoctu.'));
            }
        }

        if ($pouzeOdpocet == false)
        {
            //Odstraneni navrhu odpoctu na dite - pred vytvorenim zaznamu rocniho vyuctovani dane
            //nacti pouze deti poplatnika
            $detiPoplatnika = $this->nactiRCDitetePoplatnika($pp->rc);
            foreach ($detiPoplatnika as $dite)
            {
                $params = $this->database->createParameters();
                $rec = new RonPortalDBRecord($this->database, 'OdpocetNaDiteNavrh');
                $rec->logDescription = $logDescription;
                $rec->addField('DatumVlozeniNavrhu', 'timestamp', false);
                $rec->addField('RC', '', false);
                $rec->addField('OsobniCislo', '', false);
                $rec->addField('DatumNastupu', 'timestamp', false);
                $rec->addField('DatumOd', 'timestamp', false);
                $rec->addField('RC_Ditete', '', false);
                $rec->addField('PoradiDitete', '', false);
                $rec->addField('Stav', '', false);

                $rec->setWhere(" DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam($zacatekRoku)) .
                    " AND DatumVlozeniNavrhu < " . $params->addParameter($this->database->timestampToParam($konecRoku)) .
                    ' AND RC  = ' . $params->addParameter($pp->rc) .
                    ' AND RC_Ditete = ' . $params->addParameter($dite['rc_rodinnehoPrislusnika'])
                    , $params);

                $rec->checkMultiple = false;
                if ($rec->loadRecord()) {
                    if (!$rec->deleteRecords()) {
                        MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri odstraneni navrhu odpoctu na dite.'));
                    }
                }
            }
        }
    }

    /**
     * @param $rc
     * @return array
     * @throws DBException
     */
    private function nactiRCDitetePoplatnika($rc)
    {
        $params = $this->database->createParameters();

        $sql = "Select RP.RC_RodinnehoPrislusnika 
                From RodinnyPrislusnik RP LEFT JOIN ClenoveRodiny CR ON (RP.RC_RodinnehoPrislusnika = CR.RC_RodinnehoPrislusnika)
                Where RC = " . $params->addParameter($rc) ." AND CR.RodinnyVztah IN (4,5,6,7) 
                AND (PartnerDatumDo IS NULL OR PartnerDatumDo > " . $params->addParameter($this->database->timestampToParam(time())) . ") ";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $deti = [];
        while ($row = $cursor->fetch()) {
            $potomek['rc_rodinnehoPrislusnika'] = $row[0];
            $deti[] = $potomek;
        }
        $cursor->close();

        return $deti;
    }


    /**
     * @param $osoba
     * @param $obdobi
     * @throws DBException
     * @throws Exception
     */
    public function kontrolaZaznamuRocnihoVyuctovaniDane($osoba, $obdobi)
    {
        //Tato fc muze byt zavolana az po za 1. je nalezeno IDPP v RVD nebo za 2. je uzivatelem vybrany PP
        require_once ("adsapp/ppomer.php");
        require_once "mzdy/prohlasenipoplatnikadane.php";

        $this->pracovniPomer = new PracovniPomer();
        $this->pracovniPomer->nactiPPPodleId($this->database, $osoba);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if ( !($this->pravo_prohlaseni->lzeZobrazitRC($this->pracovniPomer->rc))){
            RonPortalUtils::neniPravoException();
        }

        //Existuje rocni vyuctovani dane na tento rok? Pokud ano zobrazit browser. Pokud ne pokusit se nacist predchozi (lonsky) zaznam
        //pokud je, pak jej pouzit jako sablonu, pokud neni predchozi(lonsky) pak zalozit novy
        //Overit zda existuje Prohlaseni na dany rok - zjistuji pouze existenci RC, ktere je povinny udaj
        $existujeRVD = $this->nactiPracovniPomerZRocnihoVyuctovaniDane($obdobi, true);

        if ($existujeRVD == false) {
            //Neexistuje Prohlaseni na dany rok - nutno vytvorit novy
            //Nacist predchozi. Je predchozi?
            $obdobiPredchoziRok = DateUtils::incMonth(DateUtils::firstDayOfYear(time()), -12);
            $hodnotyProhlaseniLonskehoRoku = ProhlaseniPoplatnikaDane::nactiProhlaseni($this->database, $this->pracovniPomer, $obdobiPredchoziRok, true);

            //uklidit OdpocetNavrh / OdpocetNaDiteNavrh - pokud mzd.prac. odstrani zaznam po kterem zustane nejaky zaznam
            //nahlasenych navrhu pak to delalo problemy pri vytvoreni noveho - cisteho pres portal pri vstupu na browser Prohlaseni
            $this->ukliditNavrhyRocnihoVyuctovaniDane($this->pracovniPomer, $obdobi, 'Odstraneni navrhu odpoctu/odpoctu na dite - pred vytvorenim zaznamu rocniho vyuctovani dane  Portal');

            if ($hodnotyProhlaseniLonskehoRoku == null) {
                //Neni predchozi, vytvori se cisty zaznam
                $this->vytvorRocniVyuctovaniDane($obdobi, null);

            } else {
                //Existuje predchozi, pouzije se jako predloha pro nove na tento rok
                $this->vytvorRocniVyuctovaniDane($obdobi, $hodnotyProhlaseniLonskehoRoku);
            }
        }
    }


    /** Slouzi k nalezeni PP id,lze ale zavolat jako kontrolu zda vubec RVD existuje s true jako 2. parametr
     * @param $obdobi
     * @param bool $testExistenceRVD        Probehne pokazde jako 1. kontrola
     * @return array|bool
     * @throws DBException
     */
    public function nactiPracovniPomerZRocnihoVyuctovaniDane($obdobi, $testExistenceRVD=false)
    {
        $params = $this->database->createParameters();

        //Hleda dle RC prihlasene osoby
        $sql = "SELECT PP.ID, RVD.RC
                FROM PracovniPomer PP 
                    INNER JOIN RocniVyuctovaniDane RVD ON (PP.RC = RVD.ProhlaseniPortalRC AND PP.OsobniCislo = RVD.ProhlaseniPortalOsc AND PP.DatumNastupu = RVD.ProhlaseniPortalDatNastup ) 
                WHERE RVD.RC = " . $params->addParameter(User::getRC()) . " AND RVD.Rok = " . $params->addParameter($this->database->timestampToParam($obdobi)) .
                " AND PP.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(time())) .
                " AND (PP.DatumUkonceni >= " . $params->addParameter($this->database ->dateToParam(time())) . " OR PP.DatumUkonceni IS NULL) ";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($testExistenceRVD == true){
            MessageManager::addDebugLog('RVD_testovani_OK');
            return ($row[1] != "");
        }

        $cursor->close();

        MessageManager::addDebugLog('RVD_hledani_PP');
        return $row[0];
    }


    /**
     * @return Redirect|BrowserProhlaseniPoplatnikaDane
     * @throws DBException
     * @deprecated Predelano
     */
    public function browserProhlaseniPoplatnikaDane()
    {
        $osoba = isset($_GET['osoba']) ? $_GET['osoba'] : '';
        //Pokud se jedna o dny kdy jeste nebylo vytvoreno obdobi pak nemusi jeste existovat zaznam o rocnim vyuctovani dane, proto zkontrolovat
        $obdobi = DateUtils::firstDayOfYear(time());         //1.1.aktu.roku

        //Pokud je v URL osoba pak jiz nekontrolovat
        if ($osoba != "") {
            //Kontroly zda je vytvoreno, zda ma nastaveno IDPP, zda existuje vice PP a zaroven neni nastaveno IDPP v RVD
            //Pred zobrazenim browseru overi zda existuje zaznam na aktualni rok, pokud neni pak ho doplni a pouzije hodnoty z lonska pokud jsou k dispozici
            $this->kontrolaZaznamuRocnihoVyuctovaniDane($osoba, $obdobi);
        } else {
            $osoba = $this->nactiPracovniPomerZRocnihoVyuctovaniDane($obdobi);
            if (!is_null($osoba)) {
                require_once 'adsapp/ppomer.php';
                $pp = PracovniPomer::nactiPP($this->database, $osoba);
                $pp->vyberNactenyPP();
            }
        }

        //Kontrola na PP se zdanenim, musí existovat alespon jeden
        $ppSeZdanenim = $this->kontrolaNaVicePracovnichPomeruSeZdanenim($this->database, $obdobi);
        if (empty($ppSeZdanenim)){
            //Pokud existuje pak neni jiste zda je tento PP prirazen k RVD, tedy pokusi se najit PP v RVD, pokud tam nebude pak se pravdepodobne jedna o 1. pristup
            //a nutno vyzvat uzivatele k vyberu PP, tedy v pripade ze se nejedná o zamestnance s prave jednim PP
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }


        //Pokud je vic PP a neni v RVD urcen konkretni pak se vypise hlaska a uzivatel se vyzve aby jeden zvolil.
        require_once ('mzdy/b_prohlasenipoplatnikadane.php');
        $b = new BrowserProhlaseniPoplatnikaDane($this->database, $ppSeZdanenim, $obdobi);
        $b->load();

        return $b;
    }

    /**
     * @param $obdobi
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @throws DBException
     */
    public function vytvorRocniVyuctovaniDane($obdobi, $hodnotyProhlaseni)
    {
        require_once ("adsapp/ronportaldbrecord.php");

        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane poplatnika - vytvoreni zaznamu, portal';

        $povolitProhlaseniNaPortalu = RonPortalSettings::getIntValue('povolitprohlaseninaportale');
        $prohlaseniElektronicky = ($hodnotyProhlaseni->prohlaseniElektronicky ?: ($povolitProhlaseniNaPortalu ?: 0));
        $zadostElektronicky = ($hodnotyProhlaseni->zadostElektronicky ?: ($povolitProhlaseniNaPortalu ?: 0));
        //$prohlaseniPortalStav = ($hodnotyProhlaseni->prohlaseniPortalStav ? 1 : 0);
        //Pokud existuje predchozi zaznam tak nastavi na 1, jinak lze pres aplikaci nastaveni zrusit
        $provestRocniVyuctovani = ($hodnotyProhlaseni->provestRocniVyuctovani ?: 1);

        $rec->addField('Rok', 'timestamp', false, 0, $obdobi);
        $rec->addField('RC', '', false, 15, $this->pracovniPomer->rc);
        $rec->addField('ProhlaseniElektronicky', 'int', false, 0, $prohlaseniElektronicky);
        $rec->addField('ZadostElektronicky', 'int', false, 0, $zadostElektronicky);
//        $rec->addField('ProhlaseniPortalStav', 'int', false, 0, $prohlaseniPortalStav);
        $rec->addField('ZadostPortalStav', 'int', false, 0, 0);
        $rec->addField('ProvestRocniVyuctovani', 'int', false, 0, $provestRocniVyuctovani);
        $rec->addField('MesicVyuctovani', 'timestamp', false, 0, DateUtils::incMonth($obdobi, 14)); //1.3.nasledujici rok
        $rec->addField('ProhlaseniPortal', 'int', false, 0, 1);
        $rec->addField('ElektronickyPodpis', 'memo', false, 0, null);

        //nastavi se aktualni pracovni pomer, loni mohlo byt vytvoreno pro jiny PP, ktery jiz nemusi byt platny
        $rec->addField('ProhlaseniPortalRC', '', false, 15, $this->pracovniPomer->rc);
        $rec->addField('ProhlaseniPortalOsc', '', false, 15, $this->pracovniPomer->osobniCislo);
        $rec->addField('ProhlaseniPortalDatNastup', 'timestamp', false, 0, $this->pracovniPomer->datumNastupu);

        //nastavi udaje danoveho nerezidenta z predchoziho prohlaseni
        $rec->addField('CisloDanoveIdentifikace', '', true, 20, $hodnotyProhlaseni->cisloDanoveIdentifikace ?? null);
        $rec->addField('TypDanovehoIdentifikatoru', 'int', true, 0, $hodnotyProhlaseni->typDanovehoIdentifikatoru ?? null);
        $rec->addField('DanovyNerezident', 'int', true, 0, $hodnotyProhlaseni->danovyNerezident ?? 0);
        $rec->addField('StatDanoveRezidence', '', true, 10, $hodnotyProhlaseni->statDanoveRezidence ?? null);

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($this->pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($obdobi));

        $rec->setWhere($sqlWhere, $params);

        //10/2020 provadi se jen insert (puvodne zde byl i update)
        if ($rec->loadRecord() == false) {
            try {
                $rec->insertRecord();
            } catch (Exception $exception) {
                throw new Exception(ronText("chyba_vytvoreni_prohlaseni"));
            }
        }
    }



    /**
     * @return BrowserZadostORocniZuctovaniZaloh
     * @throws DBException
     */
    public function browserZadostiORocniZuctovaniZaloh()
    {
        require_once "mzdy/b_zadostozuctovanizaloh.php";
        $b = new BrowserZadostORocniZuctovaniZaloh($this->database);
        $b->load();

        return $b;
    }


    /**
     * @param int $datumProhlaseni
     * @return AdresaOsoby
     * @throws DBException
     */
    public function aktualniAdresaPoplatnika($datumProhlaseni)
    {
        $adresa = new AdresaOsoby();

        $adresy = AdresaOsoby::nactiAdresuOsoby($this->database, $this->pracovniPomer->rc, $datumProhlaseni ?: DateUtils::today(), 1);

        foreach ($adresy as $adresaOsoby) {
            if ($adresaOsoby->platnostOd > $adresa->platnostOd)
                $adresa = $adresaOsoby;
        }

        return $adresa;
    }


    /**
     * @param null | bool $dodatecne
     * @return Options
     */
    public function volbyComboBoxuZadostiAProhlaseni($dodatecne = false)
    {
        $volbyXaXD = new Options();
        $volbyXaXD ->addOption(new Option(0, ""));
        $volbyXaXD ->addOption(new Option(1, "X"));

        //Momentalne se neresi MaKr (16.6.20)
        if ($dodatecne) {
            $volbyXaXD->addOption(new Option(2, "XD"));
        }

        return $volbyXaXD;
    }

    /**
     * @param $prohlaseni
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @return ComboBox
     */
    public function comboDodatecneProhlaseni($prohlaseni, $hodnotyProhlaseni)
    {
        //Prohlaseni cinim, dle predchoziho roku se nastavi na X pokud byl tak i predchozi rok
        $volby = $this->volbyComboBoxuZadostiAProhlaseni();
        $combo = new ComboBox('prohlasenikdanicinim', $volby->data);

        //Pokud je navrh schvalen, pak neumoznit volbu nepodepsat viz Prohlášení k dani činím / dodatečně činím nelze zmenit
        $combo->htmlParams = "id='prohlasenikdanicinim'" . ($hodnotyProhlaseni->prohlaseniPortalStav != 1 ? "" : "disabled='disabled'");
//        $combo->className = 'adsfilterforminput_kod';
        if ($hodnotyProhlaseni->datumProhlaseniPortal != "" && $hodnotyProhlaseni->provestRocniVyuctovani == 1 && $prohlaseni == 1){
            $combo->selectedOption = $combo->findOption(1);
        } else {
            $combo->selectedOption = $combo->findOption((int)$prohlaseni);
        }

        return $combo;
    }


    /**
     * @param $name
     * @param $class
     * @param int $vybraneObdobi //Pokud neni predchozi zaznam, pak pred-vybrat aktualni mesic
     * @return ComboBox
     */
    public function comboKalendarnichMesicu($name, $class, $vybraneObdobi=0)
    {
        $mesice = new Options();

        $pocatecniMesic = 1;
        for ($i = $pocatecniMesic; $i <=12; $i++){
            $mesice->addOption(new Option($i, ronText("mesic$i")));
        }

        $combo = new ComboBox($name, $mesice->data);
        $combo->className = "$class adsfilterforminput_kod";
        $combo->htmlParams = "id='$name'" . " disabled='disabled'";

        //Prednastavit aktualni mesic pokud se jedna o novou zmenu
        if ($vybraneObdobi == 0) {
            $vybraneObdobi = date("n", time());
        }
        $combo->selectedOption = $combo->findOption((int)$vybraneObdobi);

        return $combo;
    }


    /**
     * @param Database $database
     * @param $obdobi
     * @return array
     * @throws DBException
     */
    public function kontrolaNaVicePracovnichPomeruSeZdanenim($database, $obdobi, $idPP = null)
    {
        //Zjisteni zda ma osoba pouze jeden PP se zdanenim

        $datum1 = (mktime(0,0,0,1,1,(int)date("Y", (int)$obdobi)));
        $datum2 = (mktime(0,0,0,12,31,(int)date("Y", (int)$obdobi)));

        $params = $database->createParameters();
        $sql = "select PracovniPomer.ID from VlastnostTypuPP, PracovniPomer
                where " . (!is_null($idPP) ? "(PracovniPomer.ID = " . $params->addParameter($idPP) . ")" : "(PracovniPomer.RC = " . $params->addParameter(User::getRC()) .  " )") . "
                  and (PracovniPomer.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ") 
                  and ((PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . ") or (PracovniPomer.DatumUkonceni is null))
                  and (VlastnostTypuPP.PlatnostOd <= " . $params->addParameter($database->timestampToParam($datum1)) . ")
                  and ((VlastnostTypuPP.PlatnostDo is null) or (VlastnostTypuPP.PlatnostDo >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                  and (VlastnostTypuPP.Kod = PracovniPomer.TypPracovnihoPomeru)
                  and (((VlastnostTypuPP.ZpusobZdaneni = 0) or (VlastnostTypuPP.ZpusobZdaneni = 1)) 
                    or ((VlastnostTypuPP.ZpusobZdaneni = 3) 
                    and ((VlastnostTypuPP.KodPP_ProZdaneni = '') or (not Exists(select P1.RC from PracovniPomer P1
                                                                                  where (P1.RC = PracovniPomer.RC) and ((P1.OsobniCislo <> PracovniPomer.OsobniCislo) or (P1.DatumNastupu <> PracovniPomer.DatumNastupu))
                                                                                    and (P1.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ")
                                                                                    and ((P1.DatumUkonceni is null) or (P1.datumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                                                                                    and (P1.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni))))
                    and ((VlastnostTypuPP.KodPP_ProZdaneni2 = '') or (not Exists(select P2.RC from PracovniPomer P2
                                                                                   where (P2.RC = PracovniPomer.RC) and ((P2.OsobniCislo <> PracovniPomer.OsobniCislo) or (P2.DatumNastupu <> PracovniPomer.DatumNastupu))
                                                                                     and (P2.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ") 
                                                                                     and ((P2.DatumUkonceni is null) or (P2.datumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                                                                                     and ((P2.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni) or (P2.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni2)))))
                    and ((VlastnostTypuPP.KodPP_ProZdaneni3 = '') or (not Exists(select P3.RC from PracovniPomer P3
                                                                                   where (P3.RC = PracovniPomer.RC) and ((P3.OsobniCislo <> PracovniPomer.OsobniCislo) or (P3.DatumNastupu <> PracovniPomer.DatumNastupu))
                                                                                     and (P3.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ") 
                                                                                     and ((P3.DatumUkonceni is null) or (P3.datumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                                                                                     and ((P3.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni) or (P3.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni2)))))
                    and ((VlastnostTypuPP.KodPP_ProZdaneni4 = '') or (not Exists(select P4.RC from PracovniPomer P4
                                                                                   where (P4.RC = PracovniPomer.RC) and ((P4.OsobniCislo <> PracovniPomer.OsobniCislo) or (P4.DatumNastupu <> PracovniPomer.DatumNastupu))
                                                                                     and (P4.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ") 
                                                                                     and ((P4.DatumUkonceni is null) or (P4.datumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                                                                                     and ((P4.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni) or (P4.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni2) or (P4.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni3)))))
                    and ((VlastnostTypuPP.KodPP_ProZdaneni5 = '') or (not Exists(select P5.RC from PracovniPomer P5
                                                                                   where (P5.RC = PracovniPomer.RC) and ((P5.OsobniCislo <> PracovniPomer.OsobniCislo) or (P5.DatumNastupu <> PracovniPomer.DatumNastupu))
                                                                                     and (P5.DatumNastupu <= " . $params->addParameter($database->timestampToParam($datum2)) . ") 
                                                                                     and ((P5.DatumUkonceni is null) or (P5.datumUkonceni >= " . $params->addParameter($database->timestampToParam($datum1)) . "))
                                                                                     and ((P5.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni) or (P5.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni2)
                                                                                       or (P5.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni3) or (P5.TypPracovnihoPomeru = VlastnostTypuPP.KodPP_ProZdaneni4)))))
                
                    and (VlastnostTypuPP.ZpusobZdaneniBezHPP < 2)))";

//TODO problem vznika kdyz tady se vyhledava PP ke zdaneni na cely aktualni rok a ve filtru pak lze vybrat jen aktivni PP k aktualnimu dni (zmeneno na aktualni rok) = DORESIT ZDA JE SPRAVNE
        $cursor = $database->getSqlResult($sql, $params);

        $ppSeZdanenim = [];
        while ($row = $cursor->fetch()) {
            $ppSeZdanenim[] = $row[0];
        }
        $cursor->close();

        if (!is_array($ppSeZdanenim)){
            //Formulář nelze vyplnit, dokud nebude opraveno mzdovým pracovníkem
            //Chyba - neni pp se zdanenim !! Nove je chybou pouze pokud nema ani jeden PP se zdanenim
            $this->zapisChybyRocnihoVyuctovaniDane($this->database, $this->pracovniPomer);
        }

        return $ppSeZdanenim;
    }


    /**
     * @param $osoba
     * @param $obdobi
     * @param bool $zmenaProhlaseni
     * @return BaseViewObject
     * @throws DBException
     */
    public function zobrazitFormularProhlaseni($osoba, $obdobi, $zmenaProhlaseni)
    {
        //Načte PP
        require_once ("adsapp/ppomer.php");

        if ($osoba == "") {
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }

        $this->pracovniPomer = PracovniPomer::nactiPP($this ->database, $osoba);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if ( !($this->pravo_prohlaseni->lzeZobrazitRC($this->pracovniPomer->rc))) {
            RonPortalUtils::neniPravoException();
        }

        require_once 'mzdy/prohlasenipoplatnikadane.php';
        $hodnotyProhlaseni = ProhlaseniPoplatnikaDane::nactiProhlaseni($this->database, $this->pracovniPomer, $obdobi);

        //Hodnota $hodnotaPPSeZdanenim = 1 (je jeden - idealni stav), $hodnotaPPSeZdanenim = 2 (je jich více, musi si zvolit), $hodnotaPPSeZdanenim = 0 (není ani jeden!)
        //Pokud je vice PP pak nabidnout vyber jednoho pres dialog -
        // Chyba nastane pokud nebude ani jeden - nesmi byt 0
        $chybaPP = false;


        if ($hodnotyProhlaseni->prohlaseniPortalStav == 3) {
            //Formulář nelze vyplnit, dokud nebude opraveno mzdovým pracovníkem, prohlaseniPortalStav znamena ze neni ani jeden PP se zdanenim
            $chybaPP = true;
        }

        $prohlaseniSchvaleno = (bool)($hodnotyProhlaseni->prohlaseniPortalStav == 1);

        //Jedna se o podepsani/upravu nebo zmenovou cast
        if ($zmenaProhlaseni && $prohlaseniSchvaleno) {
            $url = "?menuid=dane&action=zmenovacast";

            //Tab. OdpocetNavrh
            //TODO upravit funkce nactiHistoriiZmenyOdpoctu a nactiNavrhyOdpoctu a prepracovat do jedine s parametrem zda se jedna o nacteni historie viz 3 - Schválená změna nebo 2 - Změna (návrh změny v již v uvedených skutečnostech - v průběhu roku)
            $historieZmenOdpoctu = $this->nactiOdpocetNavrh($this->pracovniPomer, $hodnotyProhlaseni->rok);
            Model::addData('historieZmenOdpoctu', $historieZmenOdpoctu);

            //Tab. OdpocetNavrh
            $zmenyOdpoctu = $this->nactiOdpocetNavrh($this->pracovniPomer, $hodnotyProhlaseni->rok, false);
            Model::addData('zmenyOdpoctu', $zmenyOdpoctu);
            Model::addData('comboMesicNastaleZmeny', $this->comboKalendarnichMesicu("zmenova_cast_kalendarni_mesice_nastale_zmeny", "zmenova_cast_uprava_slevy_poplatnika", $zmenyOdpoctu->mesicZmeny));

            //TODO upravit funkce nactiHistoriiZmenOdpoctuNaDeti a nactiNavrhyZmenyOdpoctuNaDeti a prepracovat do jedine s parametrem zda se jedna o nacteni historie viz 3 - Schválená změna nebo 2 - Změna (návrh změny v již v uvedených skutečnostech - v průběhu roku)
            //Tab. OdpocetNaDiteNavrh
            $historieZmenOdpoctuNaDeti = $this->nactiNavrhOdpoctuNaDite($this->pracovniPomer, $hodnotyProhlaseni->rok);
            Model::addData('historieZmenOdpoctuNaDeti', $historieZmenOdpoctuNaDeti);

            //Tab. OdpocetNaDiteNavrh
            $navrhZmenyOdpoctuNaDite = $this->nactiNavrhOdpoctuNaDite($this->pracovniPomer, $hodnotyProhlaseni->rok, false);
            Model::addData('zmenyOdpoctuNaDite', $navrhZmenyOdpoctuNaDite);
            Model::addData('comboMesicZmenyPoradiDeti', $this->comboKalendarnichMesicu("zmenova_cast_kalendarniMesic_zmenyPoradiDeti", "zmenova-cast-uprava-poradi-slev-na-deti", (int)$navrhZmenyOdpoctuNaDite['mesicZmeny']));

        } elseif (!$prohlaseniSchvaleno ) {
            //Ikdyz nebude chtit podepsat tak musi jit prohlaseni ulozit
            $url = "?menuid=dane&action=ulozitprohlaseni";
        } else {
            $url = "?menuid=dane";
        }

        //pokud je definovan text souhlasu (prohlaseni) danoveho nerezidenta, musi ho pred vyplnenim potvrdit
        if ((RonPortalSettings::getMultilineValue('dane_nerezident_text_souhlasu') != '') && $hodnotyProhlaseni->danovyNerezident) {
            if (!$hodnotyProhlaseni->elektronickyPodpisCestneProhlaseni) {
                Model::addData('prohlaseni_nerezident', RonPortalSettings::getMultilineValue('dane_nerezident_text_souhlasu'));
                $url = "?menuid=dane&action=prohlaseninerezidenta";
            }
            if ($hodnotyProhlaseni->nerezidentMaPrijemZahranici) {
                throw new Exception(ronText('Prohlaseni_nerezident_ma_jine_prijmy'));
            }
        }

        //Tab. Odpocet  - Nacte odpocet za aktualni rok nebo ten ktery zapocal v v predchozim obdobi a stale trva
        $historieOdpoctu = $this->nactiOdpocet($hodnotyProhlaseni, $this->pracovniPomer);
        Model::addData('historieOdpoctu', $historieOdpoctu);

        //Tab. OdpocetNaDite
//        $historieOdpoctuNaDeti = $this->nactiHistoriiOdpoctuNaDeti($hodnotyProhlaseni, $this->pracovniPomer);           //DEPRECATED nactiHistoriiOdpoctuNaDeti
        $historieOdpoctuNaDeti = $this->nactiOdpocetNaDeti($hodnotyProhlaseni, $this->pracovniPomer, true);
        Model::addData('historieOdpoctuNaDeti', $historieOdpoctuNaDeti);

        Model::addData('pracovniPomer', $this->pracovniPomer);
        Model::addData('url', $url);
        Model::addData('chybaPP', $chybaPP);
        Model::addData('datumObdobi_timestamp', $obdobi);
        Model::addData('chciPodepsatProhlaseni', $hodnotyProhlaseni->prohlaseniKDaniCinim);
        Model::addData('zmenaProhlaseni', $zmenaProhlaseni);
        Model::addData('hodnotyProhlaseni', $hodnotyProhlaseni);

        Model::addData('comboProhlaseni', $this->comboDodatecneProhlaseni($hodnotyProhlaseni->prohlaseniKDaniCinim, $hodnotyProhlaseni));
//        Model::addData('comboCastZdanovacihoObdobi', $this->comboKalendarnichMesicu());

        Model::addData('partnerPoplatnika', $this->nactiPartneraPoplatnika($this->database, $this->pracovniPomer, $obdobi));
        Model::addData('adresa', $this->aktualniAdresaPoplatnika($hodnotyProhlaseni->datumProhlaseni));

//        $detiPoplatnika = $this->nactiDetiPoplatnikaAOdpocetNaDite($this->database, $obdobi, $this->pracovniPomer);       //DEPRECATED
        $detiPoplatnika = $this->nactiOdpocetNaDeti($hodnotyProhlaseni, $this->pracovniPomer);
        Model::addData('detiPoplatnika', $detiPoplatnika);
        Model::addData('database', $this->database);

        Model::addData('zdravotniStav', $this->nactiZdravotniStavOsoby($hodnotyProhlaseni, $this->pracovniPomer->rc));

        Model::addData('povinnedokumenty', $this->jePovinnyDokument($hodnotyProhlaseni, $historieOdpoctuNaDeti));
        Model::addData('reportlist', Report::loadFormReports($this->database, 'DaneProhlaseni'));

        return new BaseViewObject("mzdy/view_prohlasenipoplatnikadane.php");
    }

    /**
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @param string $rc
     * @return ZdravotniStavOsoby
     */
    public function nactiZdravotniStavOsoby($hodnotyProhlaseni, $rc)
    {
        $datumOd = $hodnotyProhlaseni->rok;
        $datumDo = $hodnotyProhlaseni->datumProhlaseni ?: DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);

        $stav = ZdravotniStavOsoby::nactiStav($this->database, $rc, $datumOd, $datumDo);

        if (is_array($stav) && count($stav) > 1) {
            $stav = end($stav);
        }

        return $stav;
    }


    /** TODO DEPRECATED pokud bude fc nactiOdpocetNavrh good!, pak tuto fc nactiNavrhyOdpoctu odstranit - je v nactiOdpocetNavrh zahrnuta
     * @param $rok
     * @param PracovniPomer $pp
     * @return stdClass
     * @throws DBException
     * @deprecated
     */
    public function nactiNavrhyOdpoctu($rok, $pp)
    {
        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;

        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);

        $params = $this->database->createParameters();
        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }


        $sql = "SELECT DatumOd, ZakladniOdpocet, OdpocetNaStudenta, NaZdravotniStav, Stav, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, DatumVlozeniNavrhu 
                FROM OdpocetNavrh 
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  
            
                 AND (DatumVlozeniNavrhu IS NULL OR DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time()))) . ")" .
                " AND (DatumVlozeniNavrhu < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) . ")" .
                " AND Stav = 2 ";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        $hodnotyDB = [];
        if (isset($row)) {
            $hodnotyDB['datumOd'] = $this->database->stringToTimestamp($row[0]);
            $hodnotyDB['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[6]);
            $hodnotyDB['zakladniOdpocet'] = $row[1];
            $hodnotyDB['odpocetNaStudenta'] = $row[2];
            $hodnotyDB['naZdravotniStav'] = $row[3];
            $hodnotyDB['stav'] = $row[4];
            $hodnotyDB['elektronickyPodpis'] = $row[5];
            if ($row[0] != "") {
                $hodnotyDB['mesicZmeny'] = date("n", (int)$this->database->stringToTimestamp($row[0]));
            } else {
                $hodnotyDB['mesicZmeny'] = 0;
            }
        } else {
            $hodnotyDB['datumOd'] = null;
            $hodnotyDB['datumVlozeniNavrhu'] = null;
            $hodnotyDB['zakladniOdpocet'] = null;
            $hodnotyDB['odpocetNaStudenta'] = null;
            $hodnotyDB['naZdravotniStav'] = null;
            $hodnotyDB['stav'] = null;
            $hodnotyDB['elektronickyPodpis'] = null;
            $hodnotyDB['mesicZmeny'] = 0;
        }

        $zmenyOdpoctu = new stdClass();
        foreach ($hodnotyDB as $key => $value) {
            $zmenyOdpoctu->$key = $value;
        }

        $cursor->close();

        return $zmenyOdpoctu;
    }

    /** Nacte odpocet za aktualni rok nebo ten ktery zapocal v predchozim obdobi a stale trva
     * @param ProhlaseniPoplatnikaDane $prohlaseni
     * @param $pp
     * @return array
     * @throws DBException
     */
    public function nactiOdpocet($prohlaseni, $pp)
    {
        $params = $this->database->createParameters();

        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($prohlaseni->rok, 12);

        if ($prohlaseni->datumProhlaseni != ''){
            $datumProhlaseni = $prohlaseni->datumProhlaseni;
            $datumPlatnosti = $prohlaseni->datumProhlaseni;
        } else {
            $datumProhlaseni = $nasledujiciRokRocnihoVyuctovani;
            $datumPlatnosti = DateUtils::today();
        }

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $prohlaseni->rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }

        $sql = "SELECT DatumOd, DatumDo, ZakladniOdpocet, OdpocetNaStudenta, NaZdravotniStav, ZTPP, PoradiPrvnihoDitete
                FROM Odpocet
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .

                " AND (DatumOd <=  " . $params->addParameter($this->database->timestampToParam($datumPlatnosti)) . " AND (DatumDo >= " . $params->addParameter($this->database->timestampToParam($datumPlatnosti)) . " OR DatumDo IS NULL ) )
                 ORDER BY DatumOd";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $historieOdpoctu = [];

        while ($row = $cursor->fetch()) {

            $hodnotyDB = [];
            $datumOd = $this->database->stringToTimestamp($row[0]);
            $datumDo = $this->database->stringToTimestamp($row[1]);
            $hodnotyDB['datumOd'] = $datumOd;
            $hodnotyDB['datumDo'] = $datumDo;
            $hodnotyDB['zakladniOdpocet'] = $row[2];
            $hodnotyDB['odpocetNaStudenta'] = $row[3];
            $hodnotyDB['naZdravotniStav'] = $row[4];
            $hodnotyDB['ZTPP'] = $row[5];

            if ($row[0] != "") {
                $hodnotyDB['mesicZmeny'] = date("n", (int)$this->database->stringToTimestamp($row[0]));
            } else {
                $hodnotyDB['mesicZmeny'] = 0;
            }

//            $datumood = "";
//            $datumoprohl = "";
//            $datumddo = "";
//            $sledovaneddatum = "";
//
//
//            $datumood = StringUtils::format("date", $datumOd);
//            $datumoprohl = StringUtils::format("date", $datumProhlaseni);
//            $datumddo = StringUtils::format("date", $datumDo);
//            $sledovaneddatum = StringUtils::format("date", $sledovaneDatum);



            if ($datumOd <= $datumProhlaseni && ($datumDo >= $sledovaneDatum || $datumDo == '')){
                $historieOdpoctu['prohlaseni'] = $hodnotyDB;
            } else {
                $historieOdpoctu['zmeny'] = $hodnotyDB;
            }
        }

        if (empty($historieOdpoctu['zmeny']) && !empty($historieOdpoctu['prohlaseni'])) {
            //Pri testovani jsem narazil na pripad kdy zmena neni, pak je potreba zobrazit alespon udaje z obdobi prohlaseni
            //Tedy zacatek roku cca do 15.2 by se melo podepsat prohlaseni zmeny mezi zacatkem roku a podepsanim jsou v poli $historieOdpoctu['prohlaseni'] a zmeny provedene po podepsani jsou v poli $historieOdpoctu['zmeny']
            $historieOdpoctu['zmeny'] = $historieOdpoctu['prohlaseni'];
        }

        $cursor->close();

        return $historieOdpoctu;
    }



    /** TODO DEPRECATED - fc zahrnuta v nactiNavrhOdpoctuNaDite
     * @param $rok
     * @param $pp
     * @return array
     * @throws DBException
     * @deprecated
     */
    public function nactiNavrhyZmenyOdpoctuNaDeti($rok, $pp)
    {
        $params = $this->database->createParameters();

        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;

        $rokRocnihoVyuctovani = $rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }

        $sql = "SELECT DatumVlozeniNavrhu, RC_Ditete, DatumOd, PoradiDitete, Stav, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, RP.Jmeno, RP.Prijmeni, RP.RodneCisloRP 
                FROM OdpocetNaDiteNavrh INNER JOIN RodinnyPrislusnik RP ON (OdpocetNaDiteNavrh.RC_Ditete = RP.RC_RodinnehoPrislusnika)
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .
                " AND (DatumVlozeniNavrhu IS NULL OR DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time()))) . ")" .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  
            
                 AND Stav IN(0, 2) ";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $odpocetNaDiteNavrhZmeny = [];

        while ($row = $cursor->fetch()) {
            $datumOd = $this->database->stringToTimestamp($row[2]);
            $navrhZmeny['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[0]);
            $navrhZmeny['datumOd'] = $datumOd;

            $navrhZmeny['poradiDitete'] = $row[3];
            $navrhZmeny['RC_Ditete'] = $row[1];
            $navrhZmeny['RodneCisloRP'] = $row[8];
            $navrhZmeny['stav'] = $row[4];
            $navrhZmeny['elektronickyPodpis'] = $row[5];

            $navrhZmeny['jmenoDitete'] = $row[6];
            $navrhZmeny['prijmeniDitete'] = $row[7];


            if ($row[2] != ""){
                $odpocetNaDiteNavrhZmeny['mesicZmeny'] = date("n", (int)$this->database->stringToTimestamp($row[2]));
            }else{
                $odpocetNaDiteNavrhZmeny['mesicZmeny'] = 0;
            }


            if ($datumOd >= $rokRocnihoVyuctovani) {
                //Zapsat pro zobrazeni pokud je datum Odpoctu z tohoto roku
                $odpocetNaDiteNavrhZmeny[$row[1]] = $navrhZmeny;
            }
        }

        $cursor->close();

        return $odpocetNaDiteNavrhZmeny;
    }




    /**
     * @param $prvniZaznam
     * @param $zakladniOdpocet
     * @param $odpocetNaStudenta
     * @param $naZdravotniStav
     * @param $predchoziZakladniOdpocet
     * @param $predchoziOdpocetNaStudenta
     * @param $predchoziNaZdravotniStav
     * @param string $stylZmeny
     * @param string $stylTextu
     * @return string
     */
    private function druhyZmenyTextove($prvniZaznam,
                                       $zakladniOdpocet,
                                       $odpocetNaStudenta,
                                       $naZdravotniStav,
                                       $predchoziZakladniOdpocet,
                                       $predchoziOdpocetNaStudenta,
                                       $predchoziNaZdravotniStav,
                                       $stylZmeny="",
                                       $stylTextu="")
    {
        $result = "";
        //Substituce - Přidána/Zrušena
        $prohlaseniTextPridani = "<span class='$stylZmeny'>" . "&nbsp;" . ronText('Prohlaseni_zmena_pridani'). "</span>" ;
        $prohlaseniTextZruseni = "<span class='$stylZmeny'>" . ronText('Prohlaseni_zmena_zruseni'). "</span>" ;

        if ($prvniZaznam){
            // pokud se jedna o prvni zaznam pak 1 považovat za přidání slevy

            if ($zakladniOdpocet == 1){
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani .  "<span class='$stylTextu'>" . ronText('Prohlaseni_zmena_zakladni_slevy') . "</span>" . "<br>";
            }

            if ($odpocetNaStudenta == 1){
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani . "</span>" . "<span class='$stylTextu'>" . ronText('Prohlaseni_zmena_sleva_studenta') . "</span>" . "<br>";
            }

            if ($naZdravotniStav > 0){
                // Pridany slevy - jelikož se jedna o prvni zaznam neni vetev else
                $result .= $prohlaseniTextPridani . "</span>" . "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            }

        } else {
            // zde jiz zjistovat zda doslo ke zmene - jaka byla predchozi hodnota Základní slevu na poplatníka ?

            $prohlaseniTextSlevaNaPoplatnika = "<span class='$stylTextu'>" . ronText('Prohlaseni_zmena_zakladni_slevy'). "</span>";
            if ($zakladniOdpocet > $predchoziZakladniOdpocet){
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  $prohlaseniTextSlevaNaPoplatnika . "<br>";
            } elseif ($zakladniOdpocet < $predchoziZakladniOdpocet){
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  $prohlaseniTextSlevaNaPoplatnika . "<br>";
            }


            if ($naZdravotniStav > $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 0){
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 2 && $predchoziNaZdravotniStav == 0){
                // 0 -> 2 (Pridano)
                $result .= $prohlaseniTextPridani .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav > $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 1){
                // 1 -> 2 (Pridano)
                $result .= $prohlaseniTextZruseni .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
                $result .= $prohlaseniTextPridani .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 1 && $naZdravotniStav < $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 2){
                // 2 -> 1 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
                $result .= $prohlaseniTextPridani .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$naZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav < $predchoziNaZdravotniStav && $predchoziNaZdravotniStav == 1){
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
            } elseif ($naZdravotniStav == 0 && $predchoziNaZdravotniStav == 2){
                // 2 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  "<span class='$stylTextu'>" . ronText("Prohlaseni_zmena_slevy_zdravi_$predchoziNaZdravotniStav") . "</span>" . "<br>";
            }


            $prohlaseniTextSlevaStudent = "<span class='$stylTextu'>" . ronText('Prohlaseni_zmena_sleva_studenta'). "</span>" ;
            if ($odpocetNaStudenta > $predchoziOdpocetNaStudenta){
                // 0 -> 1 (Pridano)
                $result .= $prohlaseniTextPridani .  $prohlaseniTextSlevaStudent . "<br>";
            } elseif ($odpocetNaStudenta < $predchoziOdpocetNaStudenta){
                // 1 -> 0 (Zruseno)
                $result .= $prohlaseniTextZruseni .  $prohlaseniTextSlevaStudent . "<br>";
            }
        }

        return $result;
    }


    /**
     * @param $pp
     * @param $rok
     * @param bool $zobrazitHistorii //true - pokud je pozadovano nacist pouze zaznamy ve stavu 3, viz: 3 - Schválená změna
     * @return array|stdClass
     * @throws DBException
     */
    public function nactiOdpocetNavrh($pp, $rok, $zobrazitHistorii=true)
    {
        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;

        //Pokud zamestnanec nasoupil v prubehu roku pak hledat zaznamy az po tomto datu, jinak hledat vůči začátku roku
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }

        $rokRocnihoVyuctovani = $rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);


        //Pokud je Stav ($stavOdpoctu) = 2 pak se jedna o jeden zaznam (aktualne nahlasena změna u mzdovky), pokud je Stav = 3 pak se jedná o radky historie zmen
        $stavOdpoctu = ($zobrazitHistorii == true) ? 3 : 2;

        $params = $this->database->createParameters();
        $sql = "SELECT DatumVlozeniNavrhu, RC, OsobniCislo, DatumNastupu,                                                                               --0..3
                        DatumOd, ZakladniOdpocet, OdpocetNaStudenta, NaZdravotniStav,                                                           --4..7
                        Stav, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}, ZTPP                                  --8..11

                FROM OdpocetNavrh
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  

                  AND (DatumVlozeniNavrhu IS NULL OR DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time()))) . ")" .
                " AND (DatumVlozeniNavrhu < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) . ")" .
                " AND Stav = " . $stavOdpoctu;


        if($zobrazitHistorii == true){
            //Pokud je pozadovano nacteni historickych zaznamu
            MessageManager::addDebugLog($sql);
            $cursor = $this->database->getSqlResult($sql, $params);

            $zaznamyOdpoctu = [];
            $prvniZaznam = true;                //pomocna promenna pro spravny vypis textove formulace zmeny napr. (Přidána: základní sleva na poplatníka, Přidána: sleva na studenta apod.)
            $predchoziZakladniOdpocet = "";
            $predchoziOdpocetNaStudenta = "";
            $predchoziNaZdravotniStav = "";

            while ($row = $cursor->fetch()) {
                $datumVlozeniNavrhu = $this->database->stringToTimestamp($row[0]);
                $datumOd = $this->database->stringToTimestamp($row[4]);

                $odpocet['datumVlozeniNavrhu'] = $datumVlozeniNavrhu;
                $odpocet['datumOd'] = $datumOd;

                $zakladniOdpocet = $row[5];
                $odpocetNaStudenta = $row[6];
                $naZdravotniStav = $row[7];

                //Overit predchozi stav, pokud zadny nebyl pak 1 u zpracovavaneho je "pridani", 0 pak "zruseni"
                $text = $this->druhyZmenyTextove($prvniZaznam,
                                                 $zakladniOdpocet,
                                                 $odpocetNaStudenta,
                                                 $naZdravotniStav,
                                                 $predchoziZakladniOdpocet,
                                                 $predchoziOdpocetNaStudenta,
                                                 $predchoziNaZdravotniStav,
                                                 "font-weight-bold");

                $odpocet['zmenyTextove'] = $text;

                $odpocet['elektronickyPodpis'] = $row[9];
                $odpocet['elektronickyPodpisUcetni'] = $row[10];

                if ($datumVlozeniNavrhu > $rokRocnihoVyuctovani && $datumVlozeniNavrhu < $nasledujiciRokRocnihoVyuctovani || $datumVlozeniNavrhu > $rokRocnihoVyuctovani && $datumVlozeniNavrhu == "") {
                    //Pokud zaznam spada do aktualniho roku pak jej pridat do pole pro zobrazeni na formulari (pokud je datum Odpoctu z tohoto roku)
                    $zaznamyOdpoctu[] = $odpocet;
                }

                //Pomocne promenne pro spravne zobrazeni druhu zmen textove - proto se plni az na konci cyklu
                $predchoziZakladniOdpocet = $zakladniOdpocet;
                $predchoziOdpocetNaStudenta = $odpocetNaStudenta;
                $predchoziNaZdravotniStav = $naZdravotniStav;
                $prvniZaznam = false;
            }

            $cursor->close();

            return $zaznamyOdpoctu;

        } else {
            //Pokud je pozadovano nacteni aktualne pozadovane zmeny odpoctu, ktera jeste nebyla schvalena mzdovnkou
            MessageManager::addDebugLog($sql);
            $cursor = $this->database->getSqlResult($sql, $params);
            $row = $cursor->fetch();

            $hodnotyDB = [];
            if (isset($row)) {
                $hodnotyDB['datumOd'] = $this->database->stringToTimestamp($row[4]);
                $hodnotyDB['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[0]);
                $hodnotyDB['zakladniOdpocet'] = $row[5];
                $hodnotyDB['odpocetNaStudenta'] = $row[6];
                $hodnotyDB['naZdravotniStav'] = $row[7];
                $hodnotyDB['ZTPP'] = $row[10];
                $hodnotyDB['stav'] = $row[8];
                $hodnotyDB['elektronickyPodpis'] = $row[9];
                if ($row[4] != "") {
                    $hodnotyDB['mesicZmeny'] = date("n", (int)$this->database->stringToTimestamp($row[4]));
                } else {
                    $hodnotyDB['mesicZmeny'] = 0;
                }
            } else {
                $hodnotyDB['datumOd'] = null;
                $hodnotyDB['datumVlozeniNavrhu'] = null;
                $hodnotyDB['zakladniOdpocet'] = null;
                $hodnotyDB['odpocetNaStudenta'] = null;
                $hodnotyDB['naZdravotniStav'] = null;
                $hodnotyDB['ZTPP'] = null;
                $hodnotyDB['stav'] = null;
                $hodnotyDB['elektronickyPodpis'] = null;
                $hodnotyDB['mesicZmeny'] = 0;
            }

            $zmenyOdpoctu = new stdClass();
            foreach ($hodnotyDB as $key => $value) {
                $zmenyOdpoctu->$key = $value;
            }

            $cursor->close();

            return $zmenyOdpoctu;
        }
    }


    /** TODO DEPRECATED  pokud bude fc nactiOdpocetNavrh good!, pak tuto fc nactiHistoriiZmenyOdpoctu odstranit - je v nactiOdpocetNavrh zahrnuta
     * @param $pp
     * @param $rok
     * @return array
     * @throws DBException
     * @deprecated
     */
    public function nactiHistoriiZmenyOdpoctu($pp, $rok)
    {
        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;
        $params = $this->database->createParameters();
        $rokRocnihoVyuctovani = $rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }


        $sql = "SELECT DatumVlozeniNavrhu, RC, OsobniCislo, DatumNastupu, 
                        DatumOd, ZakladniOdpocet, OdpocetNaStudenta, NaZdravotniStav, 
                        Stav, {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}
                FROM OdpocetNavrh
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  

                 AND (DatumVlozeniNavrhu IS NULL OR DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time()))) . ")" .
                " AND (DatumVlozeniNavrhu < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) . ")" .
                " AND Stav = 3 ";



        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $zaznamyOdpoctu = [];
        $prvniZaznam = true;                //pomocna promenna pro spravny vypis textove formulace zmeny napr. (Přidána: základní sleva na poplatníka,  Přidána: sleva na studenta apod.)
        $predchoziZakladniOdpocet = "";
        $predchoziOdpocetNaStudenta = "";
        $predchoziNaZdravotniStav = "";

        while ($row = $cursor->fetch()) {
            $datumVlozeniNavrhu = $this->database->stringToTimestamp($row[0]);
            $datumOd = $this->database->stringToTimestamp($row[4]);

            $odpocet['datumVlozeniNavrhu'] = $datumVlozeniNavrhu;
            $odpocet['datumOd'] = $datumOd;

            $zakladniOdpocet = $row[5];
            $odpocetNaStudenta = $row[6];
            $naZdravotniStav = $row[7];

            //Overit predchozi stav, pokud zadny nebyl pak 1 u zpracovavaneho je "pridani", 0 pak "zruseni"
            $text = $this->druhyZmenyTextove($prvniZaznam,
                                             $zakladniOdpocet,
                                             $odpocetNaStudenta,
                                             $naZdravotniStav,
                                             $predchoziZakladniOdpocet,
                                             $predchoziOdpocetNaStudenta,
                                             $predchoziNaZdravotniStav,
                                             "font-weight-bold");

            $odpocet['zmenyTextove'] = $text;

            $odpocet['elektronickyPodpis'] = $row[9];
            $odpocet['elektronickyPodpisUcetni'] = $row[10];

            if ($datumVlozeniNavrhu > $rokRocnihoVyuctovani && $datumVlozeniNavrhu < $nasledujiciRokRocnihoVyuctovani || $datumVlozeniNavrhu > $rokRocnihoVyuctovani && $datumVlozeniNavrhu == "") {
                //Pokud zaznam spada do aktualniho roku pak jej pridat do pole pro zobrazeni na formulari (pokud je datum Odpoctu z tohoto roku)
                $zaznamyOdpoctu[] = $odpocet;
            }

            //Pomocne promenne pro spravne zobrazeni druhu zmen textove - proto se plni az na konci cyklu
            $predchoziZakladniOdpocet = $zakladniOdpocet;
            $predchoziOdpocetNaStudenta = $odpocetNaStudenta;
            $predchoziNaZdravotniStav = $naZdravotniStav;
            $prvniZaznam = false;
        }

        $cursor->close();
        return $zaznamyOdpoctu;
    }


    /** Nacte hodnoty tab. OdpocetNaDiteNavrh dle stavu(3) již schvalene - historie , nebo připravene pro mzdovku na shvaleni (0,2)
     * @param $pp
     * @param $rok
     * @param bool $zobrazitHistorii
     * @return array
     * @throws DBException
     */
    public function nactiNavrhOdpoctuNaDite($pp, $rok, $zobrazitHistorii=true)
    {
        $params = $this->database->createParameters();

        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;

        $rokRocnihoVyuctovani = $rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }


        //Pokud je $zobrazitHistorii = true pak se jedna o jeden zaznam (aktualne nahlasena změna u mzdovky), pokud je Stav = 3 pak se jedná o radky historie zmen

        $sql = "SELECT DatumVlozeniNavrhu, RC_Ditete, DatumOd, PoradiDitete, Stav, RP.Jmeno, RP.Prijmeni, RP.RodneCisloRP,      --0..7
                        {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}                      --8..9
                
                FROM OdpocetNaDiteNavrh INNER JOIN RodinnyPrislusnik RP ON (OdpocetNaDiteNavrh.RC_Ditete = RP.RC_RodinnehoPrislusnika)

                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .

                " AND (DatumVlozeniNavrhu IS NULL OR DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time()))) . ")" .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  ";

        if($zobrazitHistorii == true){
            $sql .= " AND Stav IN(3) " . " ORDER BY DatumVlozeniNavrhu, PoradiDitete";
        } else {
            $sql .= " AND Stav IN(0, 2) " . " ORDER BY DatumVlozeniNavrhu, PoradiDitete";
        }


        MessageManager::addDebugLog($sql);

        if($zobrazitHistorii == true){
            $cursor = $this->database->getSqlResult($sql, $params);

            $seznamHistorieOdpoctuNaDeti = [];
            while ($row = $cursor->fetch()) {
                $datumVlozeniNavrhu = $this->database->stringToTimestamp($row[0]);  //pom promenna pro ladeni
                $datumOd = $this->database->stringToTimestamp($row[2]);
                $odpocetNaDite['datumVlozeniNavrhu'] = $datumVlozeniNavrhu;
                $odpocetNaDite['datumOd'] = $datumOd;

                $odpocetNaDite['RC_Ditete'] = $row[1];
                $odpocetNaDite['poradiDitete'] = $row[3];

                $odpocetNaDite['jmenoDitete'] = $row[5];
                $odpocetNaDite['prijmeniDitete'] = $row[6];

                $odpocetNaDite['elektronickyPodpis'] = $row[8];
                $odpocetNaDite['elektronickyPodpisUcetni'] = $row[9];


                $seznamHistorieOdpoctuNaDeti[] = $odpocetNaDite;
            }

            $cursor->close();

            return $seznamHistorieOdpoctuNaDeti;

        } else {
            $cursor = $this->database->getSqlResult($sql, $params);

            $odpocetNaDiteNavrhZmeny = [];

            while ($row = $cursor->fetch()) {
                $datumOd = $this->database->stringToTimestamp($row[2]);
                $navrhZmeny['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[0]);
                $navrhZmeny['datumOd'] = $datumOd;

                $navrhZmeny['poradiDitete'] = $row[3];
                $navrhZmeny['RC_Ditete'] = $row[1];
                $navrhZmeny['RodneCisloRP'] = $row[7];
                $navrhZmeny['stav'] = $row[4];
                $navrhZmeny['elektronickyPodpis'] = $row[8];

                $navrhZmeny['jmenoDitete'] = $row[5];
                $navrhZmeny['prijmeniDitete'] = $row[6];


                if ($row[2] != ""){
                    $odpocetNaDiteNavrhZmeny['mesicZmeny'] = date("n", (int)$this->database->stringToTimestamp($row[2]));
                }else{
                    $odpocetNaDiteNavrhZmeny['mesicZmeny'] = 0;
                }


                if ($datumOd >= $rokRocnihoVyuctovani) {
                    //Zapsat pro zobrazeni pokud je datum Odpoctu z tohoto roku
                    $odpocetNaDiteNavrhZmeny[$row[1]] = $navrhZmeny;
                }
            }

            $cursor->close();

            return $odpocetNaDiteNavrhZmeny;
        }
    }



    /** TODO DEPRECATED - fc zahrnuta v nactiNavrhOdpoctuNaDite
     * @param $pp
     * @param $rok
     * @return array
     * @throws DBException
     * @deprecated
     */
    public function nactiHistoriiZmenOdpoctuNaDeti($pp, $rok)
    {
        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rok, 12);

        $nastup = $pp->datumNastupu;
        $pocatekRoku = $rok;
        if ($nastup < $pocatekRoku){
            $sledovaneDatum = $pocatekRoku;
        } else {
            $sledovaneDatum = $nastup;
        }


        $params = $this->database->createParameters();
        $sql = "SELECT DatumVlozeniNavrhu, RC, OsobniCislo, DatumNastupu, RC_Ditete, 
                        DatumOd, PoradiDitete, Stav, RP.Jmeno, RP.Prijmeni, 
                        {fn_CASTWIDEMEMO(ElektronickyPodpis)}, {fn_CASTWIDEMEMO(ElektronickyPodpisUcetni)}
                FROM OdpocetNaDiteNavrh LEFT JOIN RodinnyPrislusnik RP ON (RP.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete)
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .


                " AND DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND (DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR DatumOd IS NULL )  


                 AND Stav = 3 " .
              " ORDER BY DatumVlozeniNavrhu, PoradiDitete";

        $cursor = $this->database->getSqlResult($sql, $params);

        $seznamHistorieOdpoctuNaDeti = [];
        while ($row = $cursor->fetch()) {
            $datumVlozeniNavrhu = $this->database->stringToTimestamp($row[0]);
            $datumDo = $this->database->stringToTimestamp($row[5]);
            $odpocetNaDite['datumVlozeniNavrhu'] = $datumVlozeniNavrhu;
            $odpocetNaDite['datumOd'] = $datumDo;

            $odpocetNaDite['RC_Ditete'] = $row[4];
            $odpocetNaDite['poradiDitete'] = $row[6];

            $odpocetNaDite['jmenoDitete'] = $row[8];
            $odpocetNaDite['prijmeniDitete'] = $row[9];

            $odpocetNaDite['elektronickyPodpis'] = $row[10];
            $odpocetNaDite['elektronickyPodpisUcetni'] = $row[11];


            $seznamHistorieOdpoctuNaDeti[] = $odpocetNaDite;
        }

        $cursor->close();

        return $seznamHistorieOdpoctuNaDeti;
    }


    /** Nová lepsí fc, nahrazujici nactiHistoriiOdpoctuNaDeti a nactiDetiPoplatnikaAOdpocetNaDite
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @param $pp
     * @param bool $zobrazitHistorii
     * @return array
     * @throws DBException
     */
    public function nactiOdpocetNaDeti($hodnotyProhlaseni, $pp, $zobrazitHistorii=false)
    {
        $rokRocnihoVyuctovani = $hodnotyProhlaseni->rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rokRocnihoVyuctovani, 12);

        if ($hodnotyProhlaseni->prohlaseniElektronicky) {
            $datumProhlaseni = $hodnotyProhlaseni->datumProhlaseniPortal ?? DateUtils::today();
        } else {
            $datumProhlaseni = $hodnotyProhlaseni->datumProhlaseni ?? DateUtils::today();
        }

        if ($pp->datumNastupu < $rokRocnihoVyuctovani) {
            $sledovaneDatum = $rokRocnihoVyuctovani;
        } else {
            $sledovaneDatum = $pp->datumNastupu;
        }




        $params = $this->database->createParameters();
        $sql =
            "SELECT RP.RC_RodinnehoPrislusnika, OdpocetNaDite.DatumOd, OdpocetNaDite.DatumDo, OdpocetNaDite.PoradiDitete, RP.Jmeno, RP.Prijmeni, RP.DatumNarozeni, RP.rodneCisloRP, RP.drzitelPrukazuZTTP, " . // 0 - 8
                   "RP.RC_RodinnehoPrislusnika, OdpocetNaDiteNavrh.PoradiDitete, OdpocetNaDiteNavrh.DatumVlozeniNavrhu, OdpocetNaDiteNavrh.DatumOd, ClenoveRodiny.VyzivovanaOsobaOd, ClenoveRodiny.VyzivovanaOsobaDo, " . // 9 - 14
                   "RP.ZdravotniStav, OdpocetNaDiteNavrh.PoradiDitete, " . // 15 - 16
                    " (Select Skolkovne From Nast_Odpocty Where PlatnostOd = " . $params->addParameter($this->database->timestampToParam($rokRocnihoVyuctovani)) . "), OdpocetNaDiteNavrh.Stav, " .  //17 - 18
                    "SkolkovneNaDite.Skolkovne " . // 19

            " FROM RodinnyPrislusnik RP 
                    JOIN ClenoveRodiny ON (RP.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) 
                    LEFT OUTER JOIN OdpocetNaDite ON (RP.RC_RodinnehoPrislusnika = OdpocetNaDite.RC_Ditete 
                          AND OdpocetNaDite.RC = " . $params->addParameter($pp->rc) .
                        " AND OdpocetNaDite.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                        " AND OdpocetNaDite.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pp->datumNastupu)) .
                        " AND (OdpocetNaDite.DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                        " AND (OdpocetNaDite.DatumDo >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR OdpocetNaDite.DatumDo IS NULL ) ) " .
                        ")
                    LEFT OUTER JOIN OdpocetNaDiteNavrh ON (RP.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete " .

                        " AND (OdpocetNaDiteNavrh.DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                        " AND (OdpocetNaDiteNavrh.DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR OdpocetNaDiteNavrh.DatumOd IS NULL ) ) " .

                        " AND Stav IN (0, 2) AND OdpocetNaDiteNavrh.RC = " . $params->addParameter($pp->rc) .
                        " AND OdpocetNaDiteNavrh.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                        " AND OdpocetNaDiteNavrh.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pp->datumNastupu)) . ") " .
                        " LEFT JOIN SkolkovneNaDite ON (RP.RC_RodinnehoPrislusnika = SkolkovneNaDite.RC_RodinnehoPrislusnika AND ClenoveRodiny.RC = SkolkovneNaDite.RC AND SkolkovneNaDite.Rok = " . $params->addParameter($this->database->timestampToParam($rokRocnihoVyuctovani)) . ") " .
                        
                " WHERE ClenoveRodiny.RC = " . $params->addParameter($pp->rc) .
                " AND ClenoveRodiny.RodinnyVztah IN (4,5,6,7) AND RP.DatumNarozeni < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND ClenoveRodiny.StejnaAdresa = 1 AND ClenoveRodiny.VyzivovanaOsoba = 1 " .
                " AND (ClenoveRodiny.VyzivovanaOsobaOd IS NULL OR ClenoveRodiny.VyzivovanaOsobaOd < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) . ")" .
                " AND (ClenoveRodiny.VyzivovanaOsobaDo IS NULL OR ClenoveRodiny.VyzivovanaOsobaDo >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . ")" .

                " ORDER BY RP.DatumNarozeni, OdpocetNaDite.DatumOd";



        MessageManager::addDebugLog($sql);

        if ($zobrazitHistorii == true) {

            $cursor = $this->database->getSqlResult($sql, $params);

            $zaznamyOdpoctuNaDeti = [];
            while ($row = $cursor->fetch()) {
                $rc_Ditete = $row[0];
                $odpocetNaDite['rc_Ditete'] = $rc_Ditete;

                $datumNarozeni = $this->database->stringToTimestamp($row[6]);

                $odpocetOd = $this->database->stringToTimestamp($row[1]);

                $odpocetDo = $this->database->stringToTimestamp($row[2]);

                $odpocetNaDite['datumOd'] = $odpocetOd;
                $odpocetNaDite['datumDo'] = $odpocetDo;

                $poradi = $row[3];
                $odpocetNaDite['poradiDitete'] = $poradi;

                $poradiNavrh = $row[10];
                $odpocetNaDite['poradiDitete_navrh'] = $poradiNavrh;

                $odpocetNaDite['jmenoDitete'] = $row[4];
                $odpocetNaDite['prijmeniDitete'] = $row[5];
                $odpocetNaDite['datumNarozeni'] = $datumNarozeni;
                $odpocetNaDite['rodneCisloRP'] = $row[7];
                $odpocetNaDite['drzitelPrukazuZTTP'] = $row[8];
                $odpocetNaDite['rc_rodinnehoPrislusnika'] = $row[9];
                $odpocetNaDite['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[11]);
                $odpocetNaDite['datumOdNavrh'] = $this->database->stringToTimestamp($row[12]);

                $vekDitete = DateUtils::daysBetween($this->database->stringToTimestamp($row[6]), $hodnotyProhlaseni->datumProhlaseni ?: DateUtils::today()) / 365.25;

                $zacatekRokuDT = new \DateTime();
                $zacatekRokuDT->setTimestamp($rokRocnihoVyuctovani);
                $datumNarozeniDT = new \DateTime();
                $datumNarozeniDT->setTimestamp($this->database->stringToTimestamp($row[6]));
                $vek = $zacatekRokuDT->diff($datumNarozeniDT);

                $odpocetNaDite['zletile'] = ($vekDitete) < 18 ? false : true;
                $odpocetNaDite['diteDoSestiLet'] = ($vekDitete) > 7 ? false : true; //24.1.2023 zmena z 6 na 7 (aby se do formulare nacetli i deti, ktere meli cast roku 5 let)
                $odpocetNaDite['diteNad26'] = ($vekDitete) < 26 ? false : true;
                $odpocetNaDite['maxVyseSkolkovneho'] = $row[17];

                //dite, ktere ma vice nez 26 na zacatku roku se nenacita - neni narok na slevu
                if ($vek->y < 26) {

                    //Zapsat pro zobrazeni posledni aktualni zaznam dle podminky
                    if ((!empty($odpocetOd) && ($odpocetOd <= $datumProhlaseni)  && (($odpocetDo >= $rokRocnihoVyuctovani) || empty($odpocetDo)))
                        || (empty($odpocetOd) && ((!empty($datumProhlaseni) && ($datumNarozeni <= $datumProhlaseni)) || (empty($datumProhlaseni) && ($datumNarozeni <= DateUtils::today()))))) {
                        $zaznamyOdpoctuNaDeti['prohlaseni'][$rc_Ditete] = $odpocetNaDite;
                    }

                    $zaznamyOdpoctuNaDeti['zmeny'][$rc_Ditete] = $odpocetNaDite;
                }
            }

            $cursor->close();

            return $zaznamyOdpoctuNaDeti;

       } else {

            $cursor = $this->database->getSqlResult($sql, $params);

            $deti = [];

            while ($row = $cursor->fetch()) {
                $potomek['jmeno'] = $row[4];
                $potomek['prijmeni'] = $row[5];
                $potomek['rc_rodinnehoPrislusnika'] = $row[9];
                $potomek['drzitelPrukazuZTTP'] = $row[8];
                $potomek['poradiDitete'] = $row[3];
                $potomek['poradiDitete_navrh'] = $row[10];
                $potomek['rodneCisloRP'] = $row[7];
                $potomek['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[11]);
                $potomek['datumOdNavrh'] = $this->database->stringToTimestamp($row[12]);

                $potomek['zdravotniStav'] = $row[15];
                $potomek['stav'] = (int)($row[18]);
                $potomek['skolkovne'] = $row[19];


                //TODO výpočet věku provadět jinak??
                $vekDitete = DateUtils::daysBetween($this->database->stringToTimestamp($row[6]), $hodnotyProhlaseni->datumProhlaseni ?: DateUtils::today()) / 365.25;
//                $vekDitete = DateUtils::daysBetween($this->database->stringToTimestamp($row[11]), time()) / 365.25;
                $potomek['zletile'] = ($vekDitete) < 18 ? false : true;
                $potomek['diteDoSestiLet'] = ($vekDitete) > 7 ? false : true; //24.1.2023 zmena z 6 na 7 (aby se do formulare nacetli i deti, ktere meli cast roku 5 let)
                $potomek['diteNad26'] = ($vekDitete) < 26 ? false : true;
                $potomek['maxVyseSkolkovneho'] = $row[17];

                $deti[$row[9]] = $potomek;
            }

            $cursor->close();

            return $deti;
        }
    }




    /** DEPRECATED
     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
     * @param PracovniPomer $pp
     * @return array
     * @throws DBException
     * @deprecated
     */
    public function nactiHistoriiOdpoctuNaDeti($hodnotyProhlaseni, $pp)
    {
        $rokRocnihoVyuctovani = $hodnotyProhlaseni->rok;
        $nasledujiciRokRocnihoVyuctovani = DateUtils::incMonth($rokRocnihoVyuctovani, 12);

        if ($hodnotyProhlaseni->datumProhlaseni != ''){
            $datumProhlaseni = $hodnotyProhlaseni->datumProhlaseni;
        } else {
            $datumProhlaseni = $nasledujiciRokRocnihoVyuctovani;
        }

        if ($pp->datumNastupu < $rokRocnihoVyuctovani){
            $sledovaneDatum = $rokRocnihoVyuctovani;
        } else {
            $sledovaneDatum = $pp->datumNastupu;
        }

        $params = $this->database->createParameters();
        $sql =
            "SELECT RP.RC_RodinnehoPrislusnika, OdpocetNaDite.DatumOd, OdpocetNaDite.DatumDo, OdpocetNaDite.PoradiDitete, RP.Jmeno, RP.Prijmeni, RP.DatumNarozeni, RP.rodneCisloRP, RP.drzitelPrukazuZTTP, " . // 0 - 8
                   "RP.RC_RodinnehoPrislusnika, OdpocetNaDiteNavrh.PoradiDitete, OdpocetNaDiteNavrh.DatumVlozeniNavrhu, OdpocetNaDiteNavrh.DatumOd, ClenoveRodiny.VyzivovanaOsobaOd, ClenoveRodiny.VyzivovanaOsobaDo " . // 9 - 14
       
            " FROM RodinnyPrislusnik RP 
                    JOIN ClenoveRodiny ON (RP.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) 
                    LEFT OUTER JOIN OdpocetNaDite ON (RP.RC_RodinnehoPrislusnika = OdpocetNaDite.RC_Ditete 
                          AND OdpocetNaDite.RC = " . $params->addParameter($pp->rc) .
                        " AND OdpocetNaDite.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                        " AND OdpocetNaDite.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pp->datumNastupu)) .
                        " AND (OdpocetNaDite.DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                        " AND (OdpocetNaDite.DatumDo >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR OdpocetNaDite.DatumDo IS NULL ) ) " .
                        ")
                    LEFT OUTER JOIN OdpocetNaDiteNavrh ON (RP.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete " .

                        " AND (OdpocetNaDiteNavrh.DatumOd <  " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                        " AND (OdpocetNaDiteNavrh.DatumOd >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . " OR OdpocetNaDiteNavrh.DatumOd IS NULL ) ) " .

                        " AND Stav IN (0, 2) AND OdpocetNaDiteNavrh.RC = " . $params->addParameter($pp->rc) .
                        " AND OdpocetNaDiteNavrh.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                        " AND OdpocetNaDiteNavrh.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pp->datumNastupu)) . ")
                WHERE ClenoveRodiny.RC = " . $params->addParameter($pp->rc) .
                " AND ClenoveRodiny.RodinnyVztah IN (4,5,6,7) AND RP.DatumNarozeni < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) .
                " AND ClenoveRodiny.StejnaAdresa = 1 AND ClenoveRodiny.VyzivovanaOsoba = 1 " .
                " AND (ClenoveRodiny.VyzivovanaOsobaOd IS NULL OR ClenoveRodiny.VyzivovanaOsobaOd < " . $params->addParameter($this->database->timestampToParam($nasledujiciRokRocnihoVyuctovani)) . ")" .
                " AND (ClenoveRodiny.VyzivovanaOsobaDo IS NULL OR ClenoveRodiny.VyzivovanaOsobaDo >= " . $params->addParameter($this->database->timestampToParam($sledovaneDatum)) . ")" .

                " ORDER BY RP.DatumNarozeni, OdpocetNaDite.DatumOd";

        $cursor = $this->database->getSqlResult($sql, $params);

        $zaznamyOdpoctuNaDeti = [];
        while ($row = $cursor->fetch()) {
            $rc_Ditete = $row[0];
            $odpocetNaDite['rc_Ditete'] = $rc_Ditete;

            $datumNarozeni = $this->database->stringToTimestamp($row[6]);

            $odpocetOd = $this->database->stringToTimestamp($row[1]);

            $odpocetDo = $this->database->stringToTimestamp($row[2]);

            $odpocetNaDite['datumOd'] = $odpocetOd;
            $odpocetNaDite['datumDo'] = $odpocetDo;

            $poradi = $row[3];
            $odpocetNaDite['poradiDitete'] = $poradi;

            $poradiNavrh = $row[10];
            $odpocetNaDite['poradiDitete_navrh'] = $poradiNavrh;

            $odpocetNaDite['jmenoDitete'] = $row[4];
            $odpocetNaDite['prijmeniDitete'] = $row[5];
            $odpocetNaDite['datumNarozeni'] = $datumNarozeni;
            $odpocetNaDite['rodneCisloRP'] = $row[7];
            $odpocetNaDite['drzitelPrukazuZTTP'] = $row[8];
            $odpocetNaDite['rc_rodinnehoPrislusnika'] = $row[9];
            $odpocetNaDite['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[11]);
            $odpocetNaDite['datumOdNavrh'] = $this->database->stringToTimestamp($row[12]);

            $vekDitete = DateUtils::daysBetween($this->database->stringToTimestamp($row[6]), $hodnotyProhlaseni->datumProhlaseni ?: DateUtils::today()) / 365.25;

            $zacatekRokuDT = new \DateTime();
            $zacatekRokuDT->setTimestamp($rokRocnihoVyuctovani);
            $datumNarozeniDT = new \DateTime();
            $datumNarozeniDT->setTimestamp($this->database->stringToTimestamp($row[6]));
            $vek = $zacatekRokuDT->diff($datumNarozeniDT);

            $odpocetNaDite['zletile'] = ($vekDitete) < 18 ? false : true;
            $odpocetNaDite['diteDoSestiLet'] = ($vekDitete) > 6 ? false : true;
            $odpocetNaDite['diteNad26'] = ($vekDitete) < 26 ? false : true;
            $odpocetNaDite['maxVyseSkolkovneho'] = $row[12];

            //dite, ktere ma vice nez 26 na zacatku roku se nenacita - neni narok na slevu
            if ($vek->y < 26) {

                //Zapsat pro zobrazeni posledni aktualni zaznam dle podminky
                if ((!empty($odpocetOd) && ($odpocetOd <= $datumProhlaseni)  && (($odpocetDo >= $rokRocnihoVyuctovani) || empty($odpocetDo)))
                    || (empty($odpocetOd) && ((!empty($datumProhlaseni) && ($datumNarozeni <= $datumProhlaseni)) || (empty($datumProhlaseni) && ($datumNarozeni <= DateUtils::today()))))) {
                    $zaznamyOdpoctuNaDeti['prohlaseni'][$rc_Ditete] = $odpocetNaDite;
                }

                $zaznamyOdpoctuNaDeti['zmeny'][$rc_Ditete] = $odpocetNaDite;
            }
        }

        $cursor->close();

        return $zaznamyOdpoctuNaDeti;
    }




    /**
     * @param $obdobi
     * @return BaseViewObject
     * @throws DBException
     * @throws Exception
     * @deprecated Predelano, nove se nekontroluje PP se zdanenim
     */
    public function zobrazitFormularZadostiOZuctovaniDane($obdobi, $idPP)
    {
        //Kontrola na jediny PP se zdanenim
        $ppSeZdanenim = $this->kontrolaNaVicePracovnichPomeruSeZdanenim($this->database, $obdobi, $idPP);
        if (empty($ppSeZdanenim)){
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }

        //Načte PP
        require_once ("adsapp/ppomer.php");
        $this->pracovniPomer = new PracovniPomer();
        $this->pracovniPomer->nactiPPPodleId($this->database, $ppSeZdanenim[0]);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if ( !($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))){
            RonPortalUtils::neniPravoException();
        }

        require_once 'mzdy/zadostozuctovanizaloh.php';
        $hodnotyZadosti = ZadostOZuctovaniZaloh::nactiZadost($this->database, $this->pracovniPomer, $obdobi);

        $chybaPP = false;
        if ($hodnotyZadosti->zadostPortalStav == 3)
        {
            //Formulář nelze vyplnit, dokud nebude opraveno mzdovým pracovníkem
            $chybaPP = true;

        } elseif ($hodnotyZadosti->zadostPortalStav == 0) {
            if (is_array($ppSeZdanenim)){
                if (count($ppSeZdanenim) > 1){
                    $chybaPP = true;
                }
            } else {
                $chybaPP = true;
            }
        }



        Model::addData('database', $this->database);

        //TODO datumObdobiRok a datumObdobi_timestamp sjednotit !
        Model::addData('datumObdobiRok', $obdobi);
        Model::addData('datumObdobi_timestamp', $obdobi);
        Model::addData('chybaPP', $chybaPP);
        Model::addData('hodnotyZadosti', $hodnotyZadosti);
        Model::addData('volby_comboboxu', $this->volbyComboBoxuZadostiAProhlaseni());
        Model::addData('adresa', $this->aktualniAdresaPoplatnika($obdobi));
        Model::addData('pracovniPomer', $this->pracovniPomer);
        Model::addData('partnerPoplatnika', $this->nactiPartneraPoplatnika($this->database, $this->pracovniPomer, $obdobi));
//        Model::addData('detiPoplatnika', $this->nactiDetiPoplatnikaAOdpocetNaDite($this->database, $obdobi, $this->pracovniPomer));   //TODO DEPRECATED - misto nactiDetiPoplatnikaAOdpocetNaDite pouzit nactiOdpocetNaDeti s parametrem zobrazitHistorii=false
        //Pro potreby fc. nactiOdpocetNaDeti, kde je predavano $hodnotyProhlaseni

        require_once 'mzdy/prohlasenipoplatnikadane.php';
        $hodnotyProhlaseni = ProhlaseniPoplatnikaDane::nactiProhlaseni($this->database, $this->pracovniPomer, $obdobi);
        Model::addData('detiPoplatnika', $this->nactiOdpocetNaDeti($hodnotyProhlaseni, $this->pracovniPomer));
        Model::addData('nastaveniOdpoctu', NastaveniOdpoctu::nactiNastaveni($this->database, $obdobi));

        Model::addData('reportlist', Report::loadFormReports($this->database, 'DaneVyuctovani'));

        return new BaseViewObject("mzdy/view_zadostozuctovanizaloh.php");
    }



    /** DEPRECATED
     * @param Database $database
     * @param $obdobi
     * @param PracovniPomer $pp
     * @return array
     * @throws DBException
     * @deprecated
     */
    public function nactiDetiPoplatnikaAOdpocetNaDite($database, $obdobi, $pp)
    {
        $params = $this->database->createParameters();
        $datum1 = (mktime(0,0,0,1,1,(int)date("Y", (int)$obdobi)));
        $datum2 = (mktime(0,0,0,12,31,(int)date("Y", (int)$obdobi)));

        $sql =
            "SELECT RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.RC_RodinnehoPrislusnika, ZdravotniStav, DrzitelPrukazuZTTP, " .  //0..4
            "OdpocetNaDite.PoradiDitete, OdpocetNaDiteNavrh.PoradiDitete, RodinnyPrislusnik.RodneCisloRP, OdpocetNaDiteNavrh.DatumOd, " .                   //5..8

            " (SELECT Stav From OdpocetNaDiteNavrh Where ClenoveRodiny.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete AND DatumOd >= " . $params->addParameter($this->database->timestampToParam($datum1)) .
                " AND DatumOd <= " . $params->addParameter($this->database->timestampToParam($datum2)) .
                " AND Stav IN (0, 2) AND OdpocetNaDiteNavrh.RC = " . $params->addParameter($pp->rc) .
                " AND OdpocetNaDiteNavrh.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                " AND OdpocetNaDiteNavrh.DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu)) . ")," .           //9

            " SkolkovneNaDite.Skolkovne, RodinnyPrislusnik.DatumNarozeni, " .                  //10..11

            " (Select Skolkovne From Nast_Odpocty Where PlatnostOd = " . $params->addParameter($this->database->timestampToParam($datum1)) . "), OdpocetNaDiteNavrh.DatumVlozeniNavrhu" .  //12-13

            " FROM RodinnyPrislusnik 
                    LEFT JOIN ClenoveRodiny ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) 
                    
                    LEFT JOIN Osoba ON (Osoba.RC = ClenoveRodiny.RC) 
                    
                    LEFT JOIN OdpocetNaDite ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = OdpocetNaDite.RC_Ditete AND OdpocetNaDite.RC = " . $params->addParameter($pp->rc) .
                        " AND OdpocetNaDite.OsobniCislo = " . $params->addParameter($pp->osobniCislo) .
                        " AND OdpocetNaDite.DatumNastupu = " . $params->addParameter($database->timestampToParam($pp->datumNastupu)) .
                        " AND OdpocetNaDite.DatumOd <= " . $params->addParameter($this->database->timestampToParam($datum2)) .
//                        " AND (OdpocetNaDite.DatumDo >= " . $params->addParameter($this->database->timestampToParam($datum1)) .
                        " AND OdpocetNaDite.DatumDo IS NULL) 
                        
                    LEFT JOIN OdpocetNaDiteNavrh ON (ClenoveRodiny.RC_RodinnehoPrislusnika = OdpocetNaDiteNavrh.RC_Ditete AND OdpocetNaDiteNavrh.Stav = 0 
                          AND OdpocetNaDiteNavrh.DatumOd >= " . $params->addParameter($this->database->timestampToParam($datum1)) .
                        " AND OdpocetNaDiteNavrh.DatumOd <= " . $params->addParameter($this->database->timestampToParam($datum2)) .
                        " AND OdpocetNaDiteNavrh.DatumVlozeniNavrhu >= " . $params->addParameter($this->database->timestampToParam($datum1)) .
                        " AND OdpocetNaDiteNavrh.DatumVlozeniNavrhu <= " . $params->addParameter($this->database->timestampToParam($datum2)) . ")" .


                  " LEFT JOIN SkolkovneNaDite ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = SkolkovneNaDite.RC_RodinnehoPrislusnika AND SkolkovneNaDite.Rok = " . $params->addParameter($this->database->timestampToParam($datum1)) . ") " .

            " WHERE Osoba.RC = " . $params->addParameter($pp->rc) .
            " AND RodinnyVztah IN (4,5,6,7) AND RodinnyPrislusnik.DatumNarozeni <= " . $params->addParameter($this->database->timestampToParam($datum2)) .
            " AND (PartnerDatumDo IS NULL OR PartnerDatumDo > " . $params->addParameter($database->timestampToParam(time())) . ")";

        $sql .=
            " ORDER BY RodinnyPrislusnik.RC_RodinnehoPrislusnika DESC";

        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        $deti = [];

        while ($row = $cursor->fetch()) {
            $potomek['jmeno'] = $row[0];
            $potomek['prijmeni'] = $row[1];
            $potomek['rc_rodinnehoPrislusnika'] = $row[2];
            $potomek['zdravotniStav'] = $row[3];
            $potomek['drzitelPrukazuZTTP'] = $row[4];
            $potomek['poradiDitete'] = $row[5];
            $potomek['poradiDitete_navrh'] = $row[6];
            $potomek['rodneCisloRP'] = $row[7];
            $potomek['datumOdNavrh'] = $this->database->stringToTimestamp($row[8]);
            $potomek['stav'] = (int)($row[9]);
            $potomek['skolkovne'] = $row[10];
            $potomek['datumVlozeniNavrhu'] = $this->database->stringToTimestamp($row[13]);


            //TODO výpočet věku provadět jinak??
            $vekDitete = DateUtils::daysBetween($this->database->stringToTimestamp($row[11]), time()) / 365.25;
            $potomek['zletile'] = ($vekDitete) < 18 ? false : true;
            $potomek['diteDoSestiLet'] = ($vekDitete) > 6 ? false : true;
            $potomek['diteNad26'] = ($vekDitete) < 26 ? false : true;
            $potomek['maxVyseSkolkovneho'] = $row[12];

            $deti[] = $potomek;
        }

        $cursor->close();

        return $deti;
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @return mixed
     * @throws DBException
     * @deprecated @use nactiPartneraPoplatnika
     */
    public function nactiPartneraPoplatnikaProZadost($database, $pp)
    {
        //nalézt posledního partnera pokud jich je v historii více, a pokud je pouze jeden pak nemusí mít datum platnosti vůbec vyplněn
        $params = $database->createParameters();
        $sql = 'SELECT ClenoveRodiny.RC, ClenoveRodiny.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Jmeno, 
                        RodinnyPrislusnik.Prijmeni, ClenoveRodiny.RodinnyVztah, ClenoveRodiny.PartnerDatumOd, ClenoveRodiny.PartnerDatumDo, 
                        RodinnyPrislusnik.DrzitelPrukazuZTTP, S_AnoNe.popis, RodinnyPrislusnik.RodneCisloRP

                FROM ClenoveRodiny 
                  LEFT JOIN RodinnyPrislusnik ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika)
                    LEFT JOIN S_AnoNe ON (S_AnoNe.kod = RodinnyPrislusnik.DrzitelPrukazuZTTP)

                WHERE ClenoveRodiny.RodinnyVztah IN (0,1,2,3,8,9) AND ClenoveRodiny.RC = ' . $params ->addParameter($pp->rc) .
                ' AND (PartnerDatumOd IS NULL OR PartnerDatumOd = (SELECT MAX (PartnerDatumOd) FROM ClenoveRodiny AS Partner WHERE Partner.RodinnyVztah IN (0,1,2,3,8,9) AND Partner.RC = ' .
            $params ->addParameter($pp->rc) . ') )';

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        $udajePartnera = $row;
        $cursor->close();

        return $udajePartnera;
    }
    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @return mixed
     * @throws DBException
     */
    public function nactiPartneraPoplatnika($database, $pp, $rok, $partnerOdecetDane = false)
    {
        if ($partnerOdecetDane) {
            $rodinnyVztah = '0,1,8,9';
        } else {
            $rodinnyVztah = '0,1,2,3,8,9';
        }

        //nalézt posledního partnera pokud jich je v historii více, a pokud je pouze jeden pak nemusí mít datum platnosti vůbec vyplněn
        $params = $database->createParameters();
        $sql = 'SELECT ClenoveRodiny.RC, ClenoveRodiny.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Jmeno, 
                        RodinnyPrislusnik.Prijmeni, ClenoveRodiny.RodinnyVztah, ClenoveRodiny.PartnerDatumOd, ClenoveRodiny.PartnerDatumDo, 
                        RodinnyPrislusnik.DrzitelPrukazuZTTP, S_AnoNe.popis, RodinnyPrislusnik.RodneCisloRP

                FROM ClenoveRodiny 
                  INNER JOIN RodinnyPrislusnik ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika)
                  INNER JOIN S_AnoNe ON (S_AnoNe.kod = RodinnyPrislusnik.DrzitelPrukazuZTTP)

                WHERE ClenoveRodiny.RodinnyVztah IN (' . $rodinnyVztah . ') 
                  AND ClenoveRodiny.RC = ' . $params ->addParameter($pp->rc) .
                ' AND ClenoveRodiny.StejnaAdresa = 1 ' .
                ' AND (ClenoveRodiny.PartnerDatumOd IS NULL OR ClenoveRodiny.PartnerDatumOd = (SELECT MAX (PartnerDatumOd) FROM ClenoveRodiny AS Partner WHERE Partner.RodinnyVztah IN (' . $rodinnyVztah . ') AND Partner.StejnaAdresa = 1' .
                '            AND (Partner.PartnerDatumDo IS NULL OR Partner.PartnerDatumDo >= '. $params->addDateParameter($rok) . ') AND Partner.RC = ' . $params ->addParameter($pp->rc) . ') )' .
                ' AND (ClenoveRodiny.PartnerDatumDo IS NULL OR ClenoveRodiny.PartnerDatumDo >= '. $params->addDateParameter($rok) . ')';

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        $udajePartnera = $row;
        $cursor->close();

        return $udajePartnera;
    }


    /**
     * @return mixed
     */
    public function nactiPOSTProhlaseni()
    {
        $hodnoty["provedenaZmenaSlevy"] = 0;
        foreach ($_POST as $key => $value) {
            $hodnoty[$key] = $value;
        }

        if ($hodnoty["zakladniOdpocet_old"] != $hodnoty["zakladniOdpocet"]){
            $hodnoty["provedenaZmenaSlevy"] = 1;
        }

        if ($hodnoty["naZdravotniStav_old"] == 1 && $hodnoty["naZdravotniStav1"] != 1){
            $hodnoty["provedenaZmenaSlevy"] = 1;
        }
        if ($hodnoty["naZdravotniStav_old"] == 2 && $hodnoty["naZdravotniStav2"] != 2){
            $hodnoty["provedenaZmenaSlevy"] = 1;
        }

        if ($hodnoty["odpocetNaStudenta_old"] != $hodnoty["odpocetNaStudenta"]){
            $hodnoty["provedenaZmenaSlevy"] = 1;
        }

        ksort($hodnoty);

        return $hodnoty;
    }

    /**
     * @return mixed
     */
    public function nactiPOSTZadosti()
    {
        foreach ($_POST as $key => $value) {
            //TODO doplnit kontrolu na kladna čísla $_POST['zadostdaryvyse']...
            //hodnoty mohou být pouze kladné

            //TODO FILTER_SANITIZE_SPECIAL_CHARS - dat do pryc a nahradit necim jinym ??
            /*
            if (strpos($key, 'jinaosobauplatnujeodpocetadresa_') === 0){
                $hodnoty[$key] = filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
            }elseif (strpos($key, 'zadostbytovepotrebyuveruadresa') === 0){
                $hodnoty[$key] =  filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
            }elseif (strpos($key, 'jinaosobauplatnujeodpocetjmeno_') === 0){
                $hodnoty[$key] =  filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
            }elseif (strpos($key, 'jinaosobauplatnujeodpocetprijmeni_') === 0){
                $hodnoty[$key] =  filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
            }elseif (strpos($key, 'predchoziplatcedaneadresa_') === 0){
                $hodnoty[$key] = filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
            } else { */
                $hodnoty[$key] = $value;
//            }
        }

        ksort($hodnoty);
        return $hodnoty;
    }




    /**
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function kontrolaJizSchvalenehoProhlaseni($rok)
    {
        $params = $this->database->createParameters();

        $sql = "SELECT ProhlaseniPortalStav FROM RocniVyuctovaniDane Where RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear($rok)));

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if ($row[0] == 1 ){
            throw new Exception(StringUtils::fromUTF8("Prohlášení již bylo schváleno mzdovým pracovníkem!"));
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function kontrolaZdaLzePodepsatElektronicky($rok, $zadost = false)
    {
        //Pokud by v prubehu vyplnovani formulare doslo ke zruseni elektronickeho prohlaseni
        $params = $this->database->createParameters();

        $sql = "SELECT ProhlaseniElektronicky, ZadostElektronicky, Schvaleno FROM RocniVyuctovaniDane Where RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear($rok)));

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if ($zadost) {
            if ($row[1] == 0 || $row[2] == 1) {
                throw new Exception(ronText('Mzdy_elektronicky_podpis_zrusen'));
            }
            return;
        }

        if ($row[0] == 0) {
            throw new Exception(ronText('Mzdy_elektronicky_podpis_zrusen'));
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function kontrolaJizSchvaleneZadosti()
    {
        $params = $this->database->createParameters();

        $sql = "SELECT ZadostPortalStav FROM RocniVyuctovaniDane Where RC = " . $params->addParameter(User::getRC()) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam(DateUtils::firstDayOfYear(time())));

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if ($row[0] == 1 || $row[0] == 4 ) { //schvaleno, nebo schvaleno ucetni
            throw new Exception(StringUtils::fromUTF8("Žádost již byla schválena mzdovým pracovníkem!"));
        }
    }


    /**
     * Pokud dojde k chybě - uživatel nemá pp pro zdanění pak dojde k zápisu stavu "ProhlaseniPortalStav" = 3 a výpisu informace uzivateli
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @throws DBException
     * @throws Exception
     */
    public function zapisChybyRocnihoVyuctovaniDane($database, $pracovniPomer)
    {
        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane poplatnika, zadny platny pp se zdanenim - portal';

        $rok = DateUtils::firstDayOfYear(time());
        $rec->addField('ProhlaseniPortalStav', 'int', false, 0, 3);

        $params = $database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($rok));

        $rec->setWhere($sqlWhere, $params);

        $rec->loadRecord();
        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_prohlaseni_portal_stav"));
        }
    }


    /**
     * @return string
     */
    public function signaturaUzivatele()
    {
        $ip = $_SERVER['REMOTE_ADDR'];
        $ipAdresa = "IP: " . $ip;
        $login = " Login: " . User::getLoginName() . "(" . User::getUserName() . ")";
        $datumCas = "Datum: " . StringUtils::format("datetime", time());

        return $datumCas . ", " . $ipAdresa. ", " . $login;
    }


    /**
     * @param $pracovniPomer
     * @param $datumOdOdpocetPuvodni
     * @param $datumOdOdpocetNavrh
     * @param int $datumVlozeniNavrhu
     * @param $slevaZakladniNova
     * @param $slevaNaStudentaNova
     * @param $slevaNaZdravotniStav
     * @param $logDescription
     * @param bool $jednaSeONavrh
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function zapisOdpocetNavrh($pracovniPomer,
                                      $datumOdOdpocetPuvodni,
                                      $datumOdOdpocetNavrh,
                                      $datumVlozeniNavrhu,
                                      $slevaZakladniNova,
                                      $slevaNaStudentaNova,
                                      $slevaNaZdravotniStav,
                                      $ztpp,
                                      $logDescription,
                                      $jednaSeONavrh=true)
    {
        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database, 'OdpocetNavrh');
        $rec->logDescription = $logDescription;

        $rec->addField('DatumVlozeniNavrhu', 'timestamp', false, 0, time());
        $rec->addField('RC', '', true, 15, $pracovniPomer->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pracovniPomer->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer->datumNastupu);
        $rec->addField('DatumOd', 'timestamp', true, 0, $datumOdOdpocetNavrh);

        $rec->addField('ZakladniOdpocet', 'int', false, 0, $slevaZakladniNova);
        $rec->addField('OdpocetNaStudenta', 'int', false, 0, $slevaNaStudentaNova);

        $rec->addField('NaZdravotniStav', 'int', false, 0, $slevaNaZdravotniStav);
        $rec->addField('Stav', 'int', false, 0, ($jednaSeONavrh ? 0 : 2));
        $rec->addField('ElektronickyPodpis', 'memo', false, 0, $this->signaturaUzivatele());
        $rec->addField('ElektronickyPodpisUcetni', 'memo', false, 0, "");
        $rec->addField('ZTPP', 'int', false, 0, isset($ztpp) && $ztpp);

        $params = $this->database->createParameters();
        $sqlWhere =
            " DatumVlozeniNavrhu = " . $params->addParameter($this->database->timestampToParam((int)$datumVlozeniNavrhu)) .
            ' AND RC = ' . $params->addParameter($pracovniPomer->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pracovniPomer->datumNastupu)) .
            " AND DatumOd = " . $params->addParameter($this->database->timestampToParam($datumOdOdpocetPuvodni));

        $rec->setWhere($sqlWhere, $params);

        try {
            $rec->updateOrInsert();
        } catch (Exception $exception) {
            throw new Exception(ronText("chyba_vkladani_aktualizaci") . $exception->getMessage());
        }
        return true;
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param $hodnoty
     * @param $zmenaPPIDRVD
     * @throws DBException
     */
    public function ukliditPoZmenePracovnihoPomeruVRocniVyuctovaniDane($pracovniPomer, $hodnoty, $zmenaPPIDRVD)
    {
        //Vycisteni pred zapisem pokud doslo ke zmenen pp na RVD nebo uzivatel odvolava X
        //Uklidit, pokud doslo ke zmene IDPP v RVD nebo uzivatel prohlaseni k dani necini (mel v prohlaseni X, ale zrusil ho a dal podepsat - tedy necini)
        if (!$hodnoty['prohlasenikdanicinim'] || $zmenaPPIDRVD) {
            //pokud nebude nakonec podepisovat prohlaseni a ma jiz navolene zmeny, pak tyto zmeny najit a smazat
            //uklidit po uzivateli a jeho pripadnych zmenach, ktere puvodne pozadoval nebo meni PPID v RVD pred schvalenim mzdovym zamestnancem

            $this->ukliditNavrhyRocnihoVyuctovaniDane($pracovniPomer, $hodnoty["datumobdobi"], 'Odstraneni navrhu odpoctu/odpoctu na dite - po zmene pracovniho pomeru  Portal');
        }
    }

    public function zapisProhlaseniNerezidenta()
    {
        $rok = DateUtils::dateStringToTimestamp("1.1.{$_POST['obdobi']}");
        $osoba = $_POST['osoba'];

        if (!$this->pravo_prohlaseni->lzeZobrazitPP($osoba))
            RonPortalUtils::neniPravoException();

        require_once "adsapp/ppomer.php";
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Zapis prohlaseni nerezidenta - portal';

//        TODO: zapsat podpis i u tlacitka jineprijmy?
//        if (isset($_POST['souhlas'])) {
            $podpis = $this->signaturaUzivatele() . "\n" . ronText('Prohlaseni_nerezident_cestne_prohlaseni') . ":\n" . RonPortalSettings::getMultilineValue('dane_nerezident_text_souhlasu');
            $rec->addField('ElektronickyPodpisCestneP', 'memo', false, 0, $podpis);
//        }
        if (isset($_POST['jineprijmy'])) {
            $rec->addField('MaPrijemVZahranici', 'int', false, 0, 1);
        }

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($rok));

        $rec->setWhere($sqlWhere, $params);
        $rec->loadRecord();

        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_prohlaseni"));
        }

        return new Redirect(null, "?menuid=dane&action=formularprohlaseni&osoba=$osoba&obdobi=$rok");
    }

    /**
     * @return bool|Redirect
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function zapisProhlaseni()
    {
        $hodnoty = $this->nactiPOSTProhlaseni();

        $rok = (mktime(0,0,0,1,1,$hodnoty["datumobdobi"]));

        require_once "adsapp/ppomer.php";
        /** @var PracovniPomer $pracovniPomer */
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $hodnoty["osoba"]);
        $this->pracovniPomer = $pracovniPomer;

        if (!($this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))){
            RonPortalUtils::neniPravoException();
        }

        //Kontrola zdali prohlášení nebylo schváleno mzdovým pracovníkem v průběhu editace formuláře prohlášení
        $this->kontrolaJizSchvalenehoProhlaseni();

        //kontrola "ProhlaseniElektronicky"
        $this->kontrolaZdaLzePodepsatElektronicky($rok, true);

        try {
            $this->zkontrolujPovinneDokumentyProhlaseni($rok, $hodnoty);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return new Redirect(null, "?menuid=dane&action=formularprohlaseni&osoba={$pracovniPomer->idPP}&obdobi=" . $rok . (isset($_GET['zmenovacast']) ? "&zmenovacast=1" : ""));
        }

        //Pokud je ma RVD v polich ProhlaseniPortalRC, ProhlaseniPortalOsc, ProhlaseniPortalDatNastup jiny IDPP nez je vybran ve
        //..filtru Osoby, pak nastavit $zmenaPPIDRVD na true a odstranit pozadovane navolene zmeny slev na dani
        $zmenaPPIDRVD = ((int)$hodnoty["prohlaseniPPID"] != (int)$hodnoty["osoba"] ? true: false);


        $this->ukliditPoZmenePracovnihoPomeruVRocniVyuctovaniDane($pracovniPomer, $hodnoty, $zmenaPPIDRVD);


        //RocniVyuctovaniDane ---------------------------------------------------
        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Zapis prohlaseni poplatnika dane (podepsani) - portal';

        $rec->addField('DatumProhlaseniPortal', 'timestamp', false, 0, time());
        //Uzivatel muze prohlaseni "nepodepsat"
        $rec->addField('ProhlaseniPortalStav', 'int', false, 0, 0);
        $rec->addField('ProhlaseniPortal', 'int', false, 0, $hodnoty['prohlasenikdanicinim']);
        $rec->addField('ElektronickyPodpis', 'memo', false, 0, $this->signaturaUzivatele());

        $rec->addField('VyzivujeJinyPoplatnik', 'int', false, 0, (int)$hodnoty['vyzivujeJinyPoplatnik']);
        $rec->addField('JinyPoplatnikUplatnujeZv', 'int', false, 0, (int)$hodnoty['jinyPoplatnikUplatnujeZv']);
        $rec->addField('JinyPoplatnikZamestnani', 'int', false, 0, (int)$hodnoty['jinyPoplatnikZamestnani']);
        $rec->addField('JinyPoplatnikJmeno', '', false, 30, $hodnoty['jinyPoplatnikJmeno']);
        $rec->addField('JinyPoplatnikPrijmeni', '', false, 40, $hodnoty['jinyPoplatnikPrijmeni']);
        $rec->addField('JinyPoplatnikRC', '', false, 15, $hodnoty['jinyPoplatnikRC']);

        $rec->addField('JinyPoplatnikObec', '', false, 80, $hodnoty['jinyPoplatnikObec']);
        $rec->addField('JinyPoplatnikPSC', '', false, 10, $hodnoty['jinyPoplatnikPSC']);
        $rec->addField('JinyPoplatnikUlice', '', false, 80, $hodnoty['jinyPoplatnikUlice']);

        $rec->addField('JinyPoplatnikNazevPlatce', '', false, 120, $hodnoty['jinyPoplatnikNazevPlatce']);
        $rec->addField('JinyPoplatnikObecPlatce', '', false, 80, $hodnoty['jinyPoplatnikObecPlatce']);
        $rec->addField('JinyPoplatnikUlicePlatce', '', false, 80, $hodnoty['jinyPoplatnikUlicePlatce']);
        $rec->addField('JinyPoplatnikPSCPlatce', '', false, 10, $hodnoty['jinyPoplatnikPSCPlatce']);

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($rok));

        $rec->setWhere($sqlWhere, $params);
        $rec->loadRecord();


        //Nastavi se az pri podepsani a pouze tehdy kdyz NENI Prohlaseni schvaleno mzdovym pracovnikem
        //Pole muze menit dokud nebude schvaleno mzdovym pracovnikem
        if ($rec->findFieldByName('ProhlaseniPortalStav')->value != 1){
            $rec->addField('ProhlaseniPortalRC', '', false, 15, $pracovniPomer->rc);
            $rec->addField('ProhlaseniPortalOsc', '', false, 15, $pracovniPomer->osobniCislo);
            $rec->addField('ProhlaseniPortalDatNastup', 'timestamp', false, 0, $pracovniPomer->datumNastupu);
        }


        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_prohlaseni"));
        }
        //RocniVyuctovaniDane ---------------------------------------------------



        //OdpocetNavrh - pokud došlo ke změně požadovaných slev -----------------
        //datum od - bude datum hlášené změny a poté již bude použit tento datum (při opětovném otevření bude hledán datum v rámci 1.1. - 31.12.)

        if ($hodnoty['prohlasenikdanicinim'])
        {
            //Pokud ano, pak zapsat zmeny
            if ($hodnoty["pozaduji_upravu_slev"]) {
                $datumOdOdpocetNavrh = $hodnoty["datumOdOdpocetNavrh"] != "" ? $hodnoty["datumOdOdpocetNavrh"] : DateUtils::today();
                //Zde OdpocetNavrh a OdpocetPuvodni je stejny, rozdilne mohou byt pouze u zmen, kde lze zadat jiny mesic nez ktery je aktualni
                $datumOdOdpocetPuvodni = $datumOdOdpocetNavrh;
                $datumVlozeniNavrhu = $hodnoty["datumVlozeniNavrhu"] != "" ? $hodnoty["datumVlozeniNavrhu"] : time();
                $slevaNaZdravotniStav = ($hodnoty["sleva_na_zdravotni_stav2"] == 1 ? 2 : ($hodnoty["sleva_na_zdravotni_stav1"] == 1 ? 1 : 0));

                //OdpocetNavrh ------------------------------------------------------
                $this->zapisOdpocetNavrh($pracovniPomer,
                                         $datumOdOdpocetPuvodni,
                                         $datumOdOdpocetNavrh,
                                         $datumVlozeniNavrhu,
                                         $hodnoty["sleva_zakladni_nova"],
                                         $hodnoty["sleva_na_studenta_nova"],
                                         $slevaNaZdravotniStav,
                                         $hodnoty['sleva_ztpp'],
                                         'Prohlaseni poplatnika dane a navrh odpoctu - portal');
            }


            //OdpocetNaDiteNavrh - pokud došlo ke změně poradi slev u ditete --------
            if ($hodnoty["pocetdetipoplatnika"] > 0) {
                for ($i = 0; $i < $hodnoty["pocetdetipoplatnika"]; $i++) {
                    $rcDitete = $hodnoty["dite_" . $i];
                    $datumOd = $hodnoty["old_datumOdNavrh_na_dite_" . $i] != "" ? $hodnoty["old_datumOdNavrh_na_dite_" . $i] : DateUtils::today();
                    $puvodniPoradi = $hodnoty["poradi_ditete_old_" . $i];
                    $novePoradi = $hodnoty["poradi_ditete_navrh_" . $i];

                    $datumVlozeniNavrhu = (int)$hodnoty["old_datumVlozeniNavrhu_na_dite_$i"] != "" ? $hodnoty["old_datumVlozeniNavrhu_na_dite_$i"] : time();

                    if ($puvodniPoradi != $novePoradi && $novePoradi != "") {
                        $this->zapisOdpocetNaDiteNavrh($pracovniPomer, $rcDitete, $datumOd, $datumOd, $datumVlozeniNavrhu, $novePoradi, 'Prohlaseni poplatnika dane a zmena odpoctu na deti - portal');
                    }
                }
            }
        }



        //Pokud doslo ke zmene osoby v RVD pak tuto osobu nacist do url
        $osobaDoURL = $zmenaPPIDRVD ? $hodnoty["osoba"] : $hodnoty["prohlaseniPPID"];

        return new Redirect(null, "?menuid=dane&osoba=$osobaDoURL");
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param string $rcDitete
     * @param int $datumOd Vzdy k prvnimu dni v mesici - Napr. 1.5.
     * @param int $kalendarniMesicZmenyPuvodni
     * @param int $datumVlozeniNavrhu Aktualizuje se pokud neni ve stavu 1 nebo 3
     * @param int $novePoradi
     * @param string $logDescription
     * @param bool $jednaSeONavrh //Jednali se o navrh(návrh na změnu na začátku roku) pak stav je 0
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function zapisOdpocetNaDiteNavrh($pracovniPomer, $rcDitete, $datumOd, $kalendarniMesicZmenyPuvodni, $datumVlozeniNavrhu, $novePoradi, $logDescription, $jednaSeONavrh=true)
    {
        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database, 'OdpocetNaDiteNavrh');
        $rec->logDescription = $logDescription;
        $rec->addField('DatumVlozeniNavrhu', 'timestamp', true, 0, time());
        $rec->addField('RC', '', true, 15, $pracovniPomer->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pracovniPomer->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer->datumNastupu);
        $rec->addField('RC_Ditete', '', true, 15, $rcDitete);
        //TODO overit spravnost DatumOd
        $rec->addField('DatumOd', 'timestamp', false, 0, $kalendarniMesicZmenyPuvodni);
        $rec->addField('PoradiDitete', 'int', false, 0, $novePoradi);
        $rec->addField('Stav', 'int', false, 0, ($jednaSeONavrh ? 0 : 2));
        $rec->addField('ElektronickyPodpis', 'memo', false, 0, $this->signaturaUzivatele());
        $rec->addField('ElektronickyPodpisUcetni', 'memo', false, 0, "");

        $params = $this->database->createParameters();
        $sqlWhere =
            " DatumVlozeniNavrhu = " . $params->addParameter($this->database->timestampToParam((int)$datumVlozeniNavrhu)) .
            ' AND RC = ' . $params->addParameter($pracovniPomer->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pracovniPomer->datumNastupu)) .
            ' AND RC_Ditete = ' . $params->addParameter($rcDitete) .
            " AND DatumOd = " . $params->addParameter($this->database->timestampToParam($datumOd));

        $rec->setWhere($sqlWhere, $params);

        try {
            $rec->updateOrInsert();
        } catch (Exception $exception) {
            //Pokud nepujde zapsat ma vyskocit upozorneni nebo skoncit ??
            throw new Exception(ronText("chyba_vkladani_aktualizaci_navrhu_odpoctu_ditete") . "\n" . $exception->getMessage());
        }

        return true;
    }



    /**
     * @throws Exception
     * @deprecated
     */
    public function zapisZadosti()
    {
        $rok = $_POST['datumobdobi'];
        //Kontrola zda nedošlo k submitu po termínu
        if (!static::lzeEditovatVyuctovani($rok)) {
            RonPortalUtils::neniPravoException();
        }

        //Kontrola zdali žádost nebyla schválena mzdovým pracovníkem v průběhu editace formuláře
        $this->kontrolaJizSchvaleneZadosti();

        $hodnoty = $this->nactiPOSTZadosti();
        $rok = $hodnoty["datumobdobi"];


        require_once "adsapp/ppomer.php";
        /** @var PracovniPomer $pracovniPomer */
        $pracovniPomer = new PracovniPomer();
        $pracovniPomer->nactiPPPodleId($this->database, $hodnoty["osoba"]);

        //Pokud ještě není žádost schválená pak spočítat školkovné - a zapsat sumu
        $sumaSkolkovnehoProVyuctovaniDane = 0;
        require_once("adsapp/ronportaldbrecord.php");
        if ($hodnoty['pocetdetipoplatnika'] > 0)
        {
            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {

                $rcDitete = $hodnoty['ditepoplatnika_' . $i];
                //Pokud pres kontrolu klienta projde skolkovne vetsi nez max. pak nastavit na max
                $skolkovneNaDite = $hodnoty['skolkovnenadite_' . $rcDitete] > $hodnoty['skolkovne_max'] ? $hodnoty['skolkovne_max'] : $hodnoty['skolkovnenadite_' . $rcDitete];
                $skolkovneNaDitePuvodni = $hodnoty['skolkovnenaditeold_' . $rcDitete];
                if (is_numeric($skolkovneNaDite)) $sumaSkolkovnehoProVyuctovaniDane += $skolkovneNaDite;

                if ($skolkovneNaDite != $skolkovneNaDitePuvodni) {
                    $params = $this->database->createParameters();

                    $rec = new RonPortalDBRecord($this->database, 'SkolkovneNaDite');
                    $rec->logDescription = 'Skolkovne na dite poplatnika - portal';

                    $rec->addField('Rok', 'timestamp', false, '', $rok);
                    $rec->addField('RC', '', false, '15', User::getRC());
                    $rec->addField('RC_RodinnehoPrislusnika', '', false, '15', $rcDitete);
                    $rec->addField('Skolkovne', 'float', false, '0', (int)$skolkovneNaDite);

                    $rec->setWhere(
                        'Rok = ' . $params->addParameter($this->database->timestampToParam($rok)) .
                        ' AND RC = ' . $params->addParameter(User::getRC()) .
                        ' AND RC_RodinnehoPrislusnika = ' . $params->addParameter($rcDitete), $params);

                    $rec->loadRecord();
//                    if (!$rec->updateOrInsert()) {
//                        throw new Exception(StringUtils::fromUTF8('Chyba při zápisu/aktualizaci školkovného'));
//                    }

                    try {
                        $rec->updateOrInsert();
                    }
                    catch (Exception $exception) {
                        throw new Exception(StringUtils::fromUTF8('Chyba při zápisu/aktualizaci školkovného: ' . $exception->getMessage()));
                    }
                }
            }
        }


        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane poplatnika - portal';


        $termin = mktime(0,0,0,2,15, (int)date("Y", time()));
        $aktualniDatum = time();
        $datumPortal = ($termin < $aktualniDatum) ? $termin : $aktualniDatum;

        $rec->addField('ProvestRocniVyuctovani', 'int', false, 0, 1);
        $rec->addField('ZadostPortalStav', 'int', false, 0, 0);
        $rec->addField('AdresaBytovePotrebyUveru', 'memo', false, 0, $hodnoty['zadostbytovepotrebyuveruadresa']);
        $rec->addField('OpravaHodnotDaru', 'float', false, 0, $hodnoty['zadostdaryvyse'] != "" && $hodnoty['zadostdaryvyse'] > 0 ? $hodnoty['zadostdaryvyse'] : 0);
        $rec->addField('OpravaUrokuZeStavebSpor', 'float', false, 0, $hodnoty['zadosturokyvyse'] != "" && $hodnoty['zadosturokyvyse'] > 0 ? $hodnoty['zadosturokyvyse'] : 0);
        $rec->addField('OpravaPenzijnihoPrip', 'float', false, 0, $hodnoty['zadostpenzijnivyse'] != "" && $hodnoty['zadostpenzijnivyse'] > 0 ? $hodnoty['zadostpenzijnivyse'] : 0);
        $rec->addField('ZivotniPojisteni', 'float', false, 0, $hodnoty['zadostpojistnevyse'] != "" && $hodnoty['zadostpojistnevyse'] > 0 ? $hodnoty['zadostpojistnevyse'] : 0);
        $rec->addField('OpravaClenskychPrispevku', 'float', false, 0, $hodnoty['zadostclenskevyse'] != "" && $hodnoty['zadostclenskevyse'] > 0 ? $hodnoty['zadostclenskevyse'] : 0);
        $rec->addField('OpravaUhradZaZkousky', 'float', false, 0, $hodnoty['zadostuhradazazkouskyvyse'] != "" && $hodnoty['zadostuhradazazkouskyvyse'] > 0 ? $hodnoty['zadostuhradazazkouskyvyse'] : 0);
        $rec->addField('OsobyUplatnujiciUver', 'memo', false, 0, $this->sestaveniOsobUplatnujicichUverZHodnot($hodnoty));
        $rec->addField('PredchoziPrijmyKdyOdKoho', 'memo', false, 0, $this->sestaveniPredchozichPrijmuKdyOdKohoZHodnot($hodnoty));
        $rec->addField('Skolkovne', 'float', false, 0, (float)$sumaSkolkovnehoProVyuctovaniDane);

        //Zápis ve formě mm-mm;mm-mm
        if ($this->kontrolaRozsahuMesicu($hodnoty['slevanapartnera']) != "" ) {
            $slevaNaPartneratext = $this->kontrolaRozsahuMesicu($hodnoty['slevanapartnera']) . ";" . $this->kontrolaRozsahuMesicu($hodnoty['slevanapartneraproztp']);
        } else {
            $slevaNaPartneratext = "";
        }
        $rec->addField('SlevaPartnerObdobi', '', false, 15, $slevaNaPartneratext);

        //Zápis ve formě mm-mm;mm-mm
        if ($this->kontrolaRozsahuMesicu($hodnoty['slevanapartneraproztp'])!= "" ) {
            $partnerZtpMesicuOd = (int)(explode("-", $hodnoty['slevanapartneraproztp'])[0]);
            $partnerZtpMesicuDo = (int)(explode("-", $hodnoty['slevanapartneraproztp'])[1]);
            $partnerZtpMesicu = $partnerZtpMesicuDo - $partnerZtpMesicuOd;
        } else {
            $partnerZtpMesicu = "";
        }
        $rec->addField('PartnerZTPP', 'int', false, 0, (int)$partnerZtpMesicu);


        if ($hodnoty['datumProhlaseniPortal'] == "" ) {
            $this->zapisProhlaseniRocnihoVyuctovani($datumPortal, $pracovniPomer);
        }

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            ' AND Rok = ' . $params->addParameter($this->database->timestampToParam($rok));

        $rec->setWhere($sqlWhere, $params);

        $rec->loadRecord();
        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_rocniho_vyuctovani"));
        }

        return new Redirect(null, "?menuid=dane");
    }



    /**
     * Pokud není prohlaseni dotud podepsano a uzivatel žádá o zúčtování dane, pak se doplní i datum prohlaseni
     * @param $datumPortal
     * @param PracovniPomer $pracovniPomer
     * @throws DBException
     * @throws Exception
     */
    public function zapisProhlaseniRocnihoVyuctovani($datumPortal, $pracovniPomer)
    {
        require_once ("adsapp/ronportaldbrecord.php");
        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane poplatnika - zapis soucasne s zadosti portal';

        $rec->addField('DatumZadostiPortal', 'timestamp', false, 0, $datumPortal);
        $rec->addField('DatumProhlaseniPortal', 'timestamp', false, 0, $datumPortal);
        $rec->addField('ProhlaseniPortal', 'int', false, 0, 1);

        //TODO doplnit elektronicky podpis

        $rok = DateUtils::firstDayOfYear($datumPortal);
        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($rok));

        $rec->setWhere($sqlWhere, $params);

        $rec->loadRecord();
        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_prohlaseni"));
        }
    }


    /**
     * @param $hodnoty
     * @return string
     */
    public function sestaveniOsobUplatnujicichUverZHodnot($hodnoty)
    {
        $result = "";

        for ($i = 0; $i < $hodnoty['pocetjinychosobuplatnujicichodpocet']; $i++)
        {
            if ($hodnoty['jinaosobauplatnujeodpocetjmeno_' . $i] != ""){
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetjmeno_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetprijmeni_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetrc_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetadresa_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."\n";
            }
        }

        return $result;
    }


    /**
     * @param $hodnoty
     * @return string
     */
    public function sestaveniPredchozichPrijmuKdyOdKohoZHodnot($hodnoty)
    {
        $result = "";

        for ($i = 0; $i < $hodnoty['pocetpredchozichplatcudane']; $i++)
        {
            if ($hodnoty['predchoziplatcedanemesice_' . $i] != ""){

                $datum = explode("-", filter_var($hodnoty['predchoziplatcedanemesice_' . $i], FILTER_SANITIZE_SPECIAL_CHARS));
                $datumOd = str_pad($datum[0], 2, "0", STR_PAD_LEFT);
                $datumDo = str_pad($datum[1], 2, "0", STR_PAD_LEFT);
                $pattern = "/^(0?[1-9]|1[012])$/";

                if (preg_match($pattern, $datum[0]) && preg_match($pattern, $datum[1]) && $datumDo >= $datumOd) {
                    $result .= $datumOd . "-" . $datumDo . "$-$";
                    $result .= filter_var($hodnoty['predchoziplatcedaneadresa_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) . "\n";
                }
            }
        }

        return $result;
    }


    /** Kontrola zdali je string ve tvaru mm-mm
     * @param $rozsah
     * @return string
     */
    public function kontrolaRozsahuMesicu($rozsah)
    {
        $odDo = "";
        $datum = explode("-", filter_var($rozsah, FILTER_SANITIZE_SPECIAL_CHARS));
        $datumOd = str_pad($datum[0], 2, "0", STR_PAD_LEFT);
        $datumDo = str_pad($datum[1], 2, "0", STR_PAD_LEFT);
        $pattern = "/^(0?[1-9]|1[012])$/";

        if (preg_match($pattern, $datum[0]) && preg_match($pattern, $datum[1]) && $datumDo >= $datumOd) {
            $odDo .= $datumOd . "-" . $datumDo;
        }

        return $odDo;
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    public function mojeProhlaseniDashboard($output)
    {
        require_once 'ron/httputils.php';

        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=dane">' .
            '<tr>' .
//            '<th>' . \Ron\ronText('zadost') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('rok') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('Mzdy_stav_prohlaseni') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('Mzdy_stav_zadosti') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
//            '</th><th>' . \Ron\ronText('Mzdy_pozaduji_rocni_vyuctovani') . '</th>' .
            '</tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumProhlaseniPortal, ProhlaseniPortalStav, S_AnoNe.popis, Rok, ProvestRocniVyuctovani, ZadostPortalStav, DatumZadostiPortal, ' .  // 0..6
            '{fn_CASTWIDEMEMO(RVD.PoznamkaProZamestnance)}, {fn_CASTWIDEMEMO(RVD.PoznamkaProZamZadost)}' . // 7 - 8

            ' FROM RocniVyuctovaniDane RVD LEFT JOIN S_AnoNe ON (RVD.ProvestRocniVyuctovani = S_AnoNe.Kod)' .
            ' Where RC = ' . $params ->addParameter(User::getRC()) .
            '  AND {fn_YEAR(RVD.Rok)} >= ' . $params->addParameter(RonPortalSettings::getIntValue('dane_pocatecni_rok', 2020));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $rok = $this->database->stringToTimestamp($row[3]);

            $stavProhlaseni = '';
            switch ($row[1]){
                case '0':
                    $stavProhlaseni = ronText('Mzdy_navrh');
                    break;

                case '1':
                    $stavProhlaseni = ronText('Mzdy_schvaleno');
                    break;

                case '2':
                    $stavProhlaseni = ronText('Mzdy_vraceno');
                    break;

                case '3':
                    $stavProhlaseni = ronText('vice_pp_pro_zdaneni');
                    break;
            }


            $stavZadosti = '';
            switch ($row[5]){
                case '0':
                    $stavZadosti = ronText('Mzdy_navrh');
                    break;

                case '1':
                    $stavZadosti = ronText('Mzdy_schvaleno');
                    break;

                case '2':
                    $stavZadosti = ronText('Mzdy_vraceno');
                    break;

                case '3':
                    $stavZadosti = ronText('vice_pp_pro_zdaneni');
                    break;
            }

            $seznamPP = \RonPortal\PracovniPomer::nactiPPOsoby($this->database, User::getRC(), $rok, DateUtils::incDay(DateUtils::incMonth($rok, 12), -1));
            $pracPomer = $seznamPP[0];

            $html .=
                '<tr>' .

//                    "<td><a href='?menuid=dane&action=formularzadosti&obdobi=$rokZadosti'>" .
//                            sprintf(\Ron\ronText('zadost_za_rok'), date("Y", $rokZadosti)) . '</a></td>' .

                    "<td>" . date("Y", $rok) . '</td>' .


                    "<td><a href='?menuid=dane&action=formularprohlaseni&osoba=" . $pracPomer->idPP . "&obdobi=" . $rok . "'>" . $stavProhlaseni . "</a>&nbsp;" .
                        ($row[0] != "" ? sprintf(ronText('zadost_podepsano'), StringUtils::format('date', $cursor ->stringToTimestamp($row[0]))) : ronText('nepodepsano')) . '</td>' .

                    "<td><a href='?menuid=dane&action=formularzadosti&osoba=" . $pracPomer->idPP . "&obdobi=" . $rok . "'>" . $stavZadosti . "</a>&nbsp;" .
                        ($row[6] != "" ? sprintf(ronText('zadost_podepsano'), StringUtils::format('date', $cursor ->stringToTimestamp($row[6]))) : ronText('nepodepsano')) . '</td>' .

//                    '<td>' . (($row[4] != "") ? \Ron\ronText('zadost_zadam') : \Ron\ronText('zadost_nezadam')) . '</td>' .
                '</tr>';

            //je zadana poznamka pro zamestnance k prohlaseni nebo zadosti
            if ($row[7] || $row[8]) {
                $html .=
                    '<tr>' .

                    '<td>&nbsp;</td>' .


                    '<td colspan="2">' . ($row[8] !== '' ? $row[8] : $row[7]) . '</td>' .

                    '</tr>';
            }
        }
        $cursor->close();

        $html .= '</table>';

        $output->addText('html', $html);
    }


    /**
     * @return TranslationsOutputObject
     */
    function translations()
    {
        return new TranslationsOutputObject();
    }


    /** Slouží ke změně přes odkaz v browseru Žádám / Nežádám
     * @param $obdobi
     * @param $zadam
     * @return Redirect
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function zadostZuctovaniZapisPresOdkazBrowseru($obdobi, $zadam, $idPP)
    {
        //14.2.2022 zrusena kontrola na pracovni pomer se zdanenim
        /*$ppSeZdanenim = $this->kontrolaNaVicePracovnichPomeruSeZdanenim($this->database, $obdobi, $idPP);
        if (empty($ppSeZdanenim)) {
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }*/

        //Načte PP
        require_once ("adsapp/ppomer.php");
        $this->pracovniPomer = new PracovniPomer();
        $this->pracovniPomer->nactiPPPodleId($this->database, $idPP);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if (!($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))) {
            RonPortalUtils::neniPravoException();
        }

        //kontrola "ProhlaseniElektronicky" - lze na Portale vyplnit?
        $this->kontrolaZdaLzePodepsatElektronicky($obdobi, true);

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane zadam/nezadam pres odkaz - portal';

        $rec->addField('Rok', 'timestamp', true);
        $rec->addField('RC', '', true);
        $rec->addField('ProhlaseniPortalStav', 'int', true);
        $rec->addField('DatumProhlaseniPortal', 'timestamp', true);

        $rec->addField('ProvestRocniVyuctovani', 'int', false, 0, $zadam);
        $rec->addField('ZadostPortalStav', 'int', false, 0, 1);

        $rec->addField('ElektronickyPodpisZadost', 'memo', false, 0, $this->signaturaUzivatele());
        $rec->addField('DatumZadostiPortal', 'timestamp', false, 0, time());

        $params = $this->database->createParameters();

        $rec ->setWhere(
            'Rok = ' . $params ->addParameter($this->database->timestampToParam($obdobi)) .
            ' AND RC = ' . $params ->addParameter($this->pracovniPomer->rc), $params);

        $rec->loadRecord();

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception(StringUtils::fromUTF8('Chyba při zápisu rocniho zuctovani dane.'));
        }

        return new Redirect(null,'?menuid=dane&');
    }


    /**
     * @param PracovniPomer $pp
     * @return array
     * @throws DBException
     */
    public function nactiAktualniStavOdpoctu($pp)
    {
        $rc = $pp->rc;
        $osobniCislo = $pp->osobniCislo;
        $datumNastupu = $pp->datumNastupu;

        $params = $this->database->createParameters();

        $sql = "SELECT DatumOd, DatumDo, ZakladniOdpocet, OdpocetNaStudenta, NaZdravotniStav 
                FROM Odpocet 
                WHERE RC = " . $params->addParameter($rc) .
                " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($datumNastupu)) .
                " AND DatumDo IS NULL ";

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        $hodnotyDB = [];
        $hodnotyDB['datumOd'] = $this->database->stringToTimestamp($row[0]);
        $hodnotyDB['datumDo'] = $this->database->stringToTimestamp($row[1]);
        $hodnotyDB['zakladniOdpocet'] = $row[2];
        $hodnotyDB['odpocetNaStudenta'] = $row[3];
        $hodnotyDB['naZdravotniStav'] = $row[4];

        $cursor->close();

//        $zmenyOdpoctu = new stdClass();
//        foreach ($hodnotyDB as $key => $value) {
//            $zmenyOdpoctu->$key = $value;
//        }

        return $hodnotyDB;
    }


    /**
     * @param $osoba
     * @return Redirect
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function zmenovaCastProhlaseni($osoba)
    {
        require_once "adsapp/ppomer.php";
        /** @var PracovniPomer $pracovniPomer */
        $pracovniPomer = new PracovniPomer();
        $pracovniPomer->nactiPPPodleId($this->database, $osoba);

        if ( !($this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))){
            RonPortalUtils::neniPravoException();
        }

        $this->pracovniPomer = $pracovniPomer;

        //Kontrola zda lze podepsat elektronicky
        $this->kontrolaZdaLzePodepsatElektronicky($_POST['obdobi_timestamp']);

        $aktualniStavOdpoctuOsoby = $this->nactiAktualniStavOdpoctu($pracovniPomer);

        //zmeny porovnat s aktu. hodnotami
        $hodnotyPOSTu = $this->nactiPOSTZmenoveCastiAVyhledejZmenu($aktualniStavOdpoctuOsoby);
        $zmenyOdpoctuNavrh = $this->nactiOdpocetNavrh($pracovniPomer, $hodnotyPOSTu->obdobi_timestamp, false);

        $aktualizovatRVDSlevy = false;
        $aktualizovatRVDDeti = false;
        if ($hodnotyPOSTu->nastalaZmenaOdpoctu && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev == 1) {

            //upravit na zvoleny mesic
            //existuje old mesic?
            if ($hodnotyPOSTu->zmenova_cast_datumOd_old != ""){
                $mesicNavrhovanyUzivatelem = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarni_mesice_nastale_zmeny, 1, (int)date("y", time()));
                $datumOdOdpocetNavrh = $mesicNavrhovanyUzivatelem != "" ? $mesicNavrhovanyUzivatelem : DateUtils::firstDayOfMonth(time());
                $datumOdPuvodni = $hodnotyPOSTu->zmenova_cast_datumOd_old;

            } else {
                $mesicNavrhovanyUzivatelem = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarni_mesice_nastale_zmeny, 1, (int)date("y", time()));
                $datumOdOdpocetNavrh = $mesicNavrhovanyUzivatelem != "" ? $mesicNavrhovanyUzivatelem : DateUtils::firstDayOfMonth(time());
                $datumOdPuvodni = DateUtils::firstDayOfMonth(time());
            }

            $aktualizovatRVDSlevy = $this->zapisOdpocetNavrh($pracovniPomer,
                                    $datumOdPuvodni, $datumOdOdpocetNavrh,
                                    $hodnotyPOSTu->zmenova_cast_datumVlozeniNavrhu,
                                    $hodnotyPOSTu->zmenova_cast_zakladniOdpocet,
                                    $hodnotyPOSTu->zmenova_cast_odpocetNaStudenta,
                                    $hodnotyPOSTu->zmenova_cast_naZdravotniStav,
                                    $hodnotyPOSTu->zmenova_cast_ztpp,'Rocni vyuctovani dane poplatnika, zmenova cast - portal', false);

        }


        if (($hodnotyPOSTu->zmenova_cast_pozadujiUpravuPoradiDeti) && (int)$hodnotyPOSTu->pocetDetiZmenyPoradi > 0 )
        {
            foreach ($hodnotyPOSTu->zmenova_cast_poradiDanovehoZvyhodneniNaDeti as $rcDitete => $novePoradi) {

                $kalendarniMesicZmeny = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti, 1, (int)date("y", time()));
                if ($hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti_old != "") {
                    $kalendarniMesicZmenyPuvodni = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti_old, 1, (int)date("y", time()));
                } else {
                    $kalendarniMesicZmenyPuvodni = $kalendarniMesicZmeny;
                }

                //TODO doplnit kontrolu na zadani obsazeneho poradi ???? Nebo jak si ma program domyslet, že u tohoto chce uzivatel 1, a pokud 2, 3 nezada u ktereho chce co, pokud budou v konfliktu - jelikož 1 a 2 mohou byt pouze jednou!
                //TODO Domnivam se uzivatel by mel byt donucen pri zmene nastavit vsechny deti, tak aby poradi nebylo v konfliktu a bylo jednoznacne.
                $lzeZapsatNavrh = true;
//                $hodnotyPOSTu;
//                die("Testovani..");
//                $puvodniPoradiDeti = $hodnotyPOSTu->zmenova_cast_poradiDanovehoZvyhodneniNaDeti_old;
//                $novePoradiDeti = $hodnotyPOSTu->zmenova_cast_poradiDanovehoZvyhodneniNaDeti;
//
//                foreach ($puvodniPoradiDeti as $rcDiteteSAktualnimPoradim => $aktualniPoradiPorovnavanehoDitete){
//                    foreach ($novePoradiDeti as $rcDiteteSNovymPoradim => $novePoradiPorovnavanehoDitete){
//                        if ($aktualniPoradiPorovnavanehoDitete == 1 || $aktualniPoradiPorovnavanehoDitete == 2){
//                            if ($aktualniPoradiPorovnavanehoDitete == 1){
//                                //TODO pokud zadam poradi 1 jinemu diteti nez ktere jej ma a tomu ktere jej ma nezmenim poradi tak hlasku a nezapsat
//                                if ($rcDiteteSAktualnimPoradim != $rcDiteteSNovymPoradim){
//                                    $lzeZapsatNavrh = false;
//                                    MessageManager::addErrorMessage(sprintf(ronText('Prohlaseni_poradi_prirazene_jinemu_diteti'), 1));
//                                }
//                            }
//
//                             if ($aktualniPoradiPorovnavanehoDitete == 2){
//                                //TODO pokud zadam poradi 1 jinemu diteti nez ktere jej ma a tomu ktere jej ma nezmenim poradi tak hlasku a nezapsat
//                                if ($rcDiteteSAktualnimPoradim != $rcDiteteSNovymPoradim){
//                                    $lzeZapsatNavrh = false;
//                                    MessageManager::addErrorMessage(sprintf(ronText('Prohlaseni_poradi_prirazene_jinemu_diteti'), 2));
//                                }
//                            }
//                        }
//                    }
//                }



                if ($lzeZapsatNavrh) {
                    //TODO pokud se bude upravovat poradi 1 u ditete s tim, ze jiz bude nastaveno u jineho ktere ale nebude upravovat preskocit a vypsat hlasku nebo nejak vyresit automaticky za uzivatele ??
                    if ($novePoradi != "") {
                        $datumVlozeniNavrhu = (int)$hodnotyPOSTu->zmenova_cast_DatumVlozeniNavrhuUpravyPoradiDeti[$rcDitete] != "" ? $hodnotyPOSTu->zmenova_cast_DatumVlozeniNavrhuUpravyPoradiDeti[$rcDitete] : time();
                        $logDescription = 'Rocni vyuctovani dane, zmenova cast odpoctu na dite - portal';
                        $aktualizovatRVDDeti = $this->zapisOdpocetNaDiteNavrh($pracovniPomer,
                            $rcDitete,
                            $kalendarniMesicZmenyPuvodni,
                            $kalendarniMesicZmeny,
                            $datumVlozeniNavrhu,
                            $novePoradi,
                            $logDescription, false);
                    }
                }
            }
        }

        //aktualizuje pokud doslo k zapisu zmeny
        if ($aktualizovatRVDDeti || $aktualizovatRVDSlevy){
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"
//            require_once 'mzdy/prohlasenipoplatnikadane.php';
//            $hodnotyPOSTuProhlaseni = ProhlaseniPoplatnikaDane::nactiProhlaseni($this->database, $pracovniPomer, $hodnotyPOSTu->obdobi_timestamp);
            $this->aktualizaceRocniVyuctovaniDanePoZmene($pracovniPomer, $hodnotyPOSTu->obdobi_timestamp);

            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_zapsana'));
        }


        if (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev)  && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && $zmenyOdpoctuNavrh->stav == 2 ){
            //Pokud uzivatel odvola zmenu jeste pred schvalenim, pak zmenu smazat

            $this->ukliditNavrhyRocnihoVyuctovaniDane($pracovniPomer, (int)$hodnotyPOSTu->zmenova_cast_datumOd_old,'Odstraneni navrhu odpoctu/odpoctu na dite - uzivatel neuplatnuje slevu  Portal', true);
            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_odstranena'));
        } elseif (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev) && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && !isset($zmenyOdpoctuNavrh->stav) ){

            //Pokud v prubehu otevreni formulare mzdovy pracovnik zmenu jiz schvali a uzivatel chce zmenu odvolat
            MessageManager::addErrorMessage(ronText('Prohlaseni_zmenova_cast_jiz_schvalena'));
        }


        return new Redirect(null,"?menuid=dane&osoba=$osoba");
    }


    /**
     * @param $pracovniPomer
     * @param $obdobi
     * @throws DBException
     * @throws Exception
     * @deprecated
     */
    public function aktualizaceRocniVyuctovaniDanePoZmene($pracovniPomer, $obdobi)
    {
        require_once ("adsapp/ronportaldbrecord.php");

        $rec = new RonPortalDBRecord($this->database,'RocniVyuctovaniDane');
        $rec->logDescription = 'Rocni vyuctovani dane poplatnika - aktualizace zmenove casti, portal';
        $rec->addField('ProhlaseniPortalZmenaStav', 'int', false, 0, 0);

        $params = $this->database->createParameters();
        $sqlWhere =
            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
            " AND Rok = " . $params->addParameter($this->database->timestampToParam($obdobi));

        $rec->setWhere($sqlWhere, $params);
        $rec->loadRecord();
        if (!$rec->loadAndUpdateRecord()) {
            throw new Exception(ronText("Prohlaseni_chyba_aktualizace_po_zmene"));
        }
    }



    /**
     * @param $aktualniStavOdpoctuOsoby
     * @return stdClass
     */
    public function nactiPOSTZmenoveCastiAVyhledejZmenu($aktualniStavOdpoctuOsoby)
    {
        $hodnoty = [];
        foreach ($_POST as $key => $value) {
            $hodnoty[$key] = $value;
        }
        $hodnoty["zmenova_cast_naZdravotniStav"] = 0;
        $hodnoty['nastalaZmenaOdpoctu'] = false;

        if (is_array($hodnoty['zmenova_cast_naZdravotniStavVolba'])){
            $slevaNaInvaliditu = max($hodnoty['zmenova_cast_naZdravotniStavVolba']);
            $hodnoty["zmenova_cast_naZdravotniStav"] = $slevaNaInvaliditu;
        } else {
            $slevaNaInvaliditu = 0;
            $hodnoty["zmenova_cast_naZdravotniStav"] = $slevaNaInvaliditu;
        }


        //Test zda nastala zmena
        foreach ($aktualniStavOdpoctuOsoby as $index => $item)
        {
            if ($hodnoty["nastalaZmenaOdpoctu"] == false) {
                switch ($index){
                    case 'datumOd':
                        //Uprava mesice zmeny
                        if ($hodnoty['zmenova_cast_kalendarni_mesice_nastale_zmeny_old'] != $hodnoty['zmenova_cast_kalendarni_mesice_nastale_zmeny']){
                            $hodnoty["nastalaZmenaOdpoctu"] = true;
                        }
                        break;

                    case 'naZdravotniStav':
                        //Dva checkboxy obsluhuji 1 pole v db, proto zjisti max z POSTu a ten porovna s aktualni hodnotou nactenou z DB
                        //Nacte hodnotu rozsah (0-2), pokud je aktu. 0 a uzivatel zmenil na "Rozsirenou slevu na invaliditu" coz je 2,
                        //tak porovna hodnoty a zjisti rozdil, pak to je duvodem pro zapis do "OdpocetNavrh"
                        if ( (int)$slevaNaInvaliditu != (int)$aktualniStavOdpoctuOsoby['naZdravotniStav'] ){
                            $hodnoty["nastalaZmenaOdpoctu"] = true;
                        }
                        break;

                    case 'zakladniOdpocet':
                    case 'odpocetNaStudenta':
                        //Pokud se obe hodnoty nerovnaji, doslo ke zmene
                        $hodnoty["nastalaZmenaOdpoctu"] = ((int)$aktualniStavOdpoctuOsoby[$index] != (int)$hodnoty["zmenova_cast_$index"]);
                        break;
                }
            } else {
                //Nastala zmena ukoncit dalsi testovani
                break;
            }
        }

        ksort($hodnoty);
        //Pro snadny pristup k hodnotam
        $hodnotyPOST = new stdClass();
        foreach ($hodnoty as $key => $value) {
            $hodnotyPOST->$key = $value;
        }

        return $hodnotyPOST;
    }

    /**
     * @return void
     * @throws DBException
     * @deprecated Nahravani predelano z javascriptu na pole ve formulari
     */
    function dokumentUpload($castProhlaseni, $rok, $osoba, $prava)
    {
        require_once 'adsapp/dokumentyutils.php';


        $datumOd = DateUtils::dateStringToTimestamp("1.1.$rok");
        $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        if (!$this->pravo_prohlaseni->lzeZobrazitPP($osoba))
            RonPortalUtils::neniPravoException();

        require_once ("adsapp/ppomer.php");
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        $typDokumentu = static::vratTypDokumentuProSekci($castProhlaseni);

        // vlozi do databaze obsah dokumentu
        $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $typDokumentu, StringUtils::fromUTF8($_POST['popis']), $this);

        require_once 'adsapp/ronportaldbrecord.php';
        $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby', $this);
        $rec->addField('Id', 'int', false, 0, (int)$id);
        $rec->addField('RC', '', false, 15, $pracovniPomer->rc);
        $rec->addField('VyuctovaniDane_Rok', 'timestamp', false, 0, $datumOd);
        $rec->addField('VyuctovaniDane_RC', '', false, 15, $pracovniPomer->rc);

        $rec->insertRecord();
    }

    public function overPravoNaDokument($idDokumentu, $pracovniPomer, $rok)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentyOsoby.Id ' .
            '  FROM DokumentyOsoby ' .
            ' WHERE DokumentyOsoby.Id = ' . $params->addParameter($idDokumentu) .
            '   AND DokumentyOsoby.RC =  ' . $params->addParameter($pracovniPomer->rc) .
            '   AND DokumentyOsoby.VyuctovaniDane_Rok = ' . $params->addParameter($this->database->dateToParam($rok)) .
            '   AND DokumentyOsoby.VyuctovaniDane_RC = ' . $params->addParameter($pracovniPomer->rc);

        if (!$cursor = $this->database->select($sql, $params)) {
            throw new DBException($this->database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            throw new Exception('Pozadovany dokument vyuctovani dane nebyl nalezen!');
        }
    }

    public function downloadDokument($idDokumentu, $osoba, $rok)
    {
        $datumOd = DateUtils::dateStringToTimestamp("1.1.$rok");
        $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        require_once ("adsapp/ppomer.php");
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))
            RonPortalUtils::neniPravoException();

        $this->overPravoNaDokument($idDokumentu, $pracovniPomer, $datumOd);

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this->database, $idDokumentu);
    }

    public function smazDokument($idDokumentu, $osoba, $rok)
    {
        $datumOd = DateUtils::dateStringToTimestamp("1.1.$rok");
        $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        require_once ("adsapp/ppomer.php");
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))
            RonPortalUtils::neniPravoException();

        $this->overPravoNaDokument($idDokumentu, $pracovniPomer, $datumOd);

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'DokumentyOsoby');

        $rec ->addField('Id', 'int');
        $rec ->addField('RC', '');

        $params = $this->database->createParameters();
        $rec ->deleteRecord(
            'Id = ' . $params ->addParameter((int)$idDokumentu) .
            ' AND RC = ' . $params ->addParameter($pracovniPomer->rc),
            $params);

        require_once('adsapp/dokumentyutils.php');
        try {
            DokumentyUtils::smazDokument($this->database, (int)$idDokumentu, $this);
        } catch (Exception $e) {
            // chybu ignoruje - zobrazi ji jen v debug rezimu - dokument totiz muze byt pripojen jeste v jine agende
            if (App::getDebugMode()) {
                MessageManager::addErrorMessage($e);
            }
        }

        $action = $_GET['backurl'] ?? 'formularprohlaseni';

        return new Redirect(null, "?menuid=dane&action=$action&osoba=$osoba&obdobi=$datumOd");
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'mojeprohlaseni':
                $this->mojeProhlaseniDashboard($output);
                break;
            case 'dokumentupload' :
                $this ->dokumentUpload($_POST['dokumentprohlasenityp'], $_POST['rok'], $_POST['idosoby']);
                break;

        }
    }

    /**
     * @param string $action
     * @return bool|BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\TranslationsOutputObject|\Ron\MVC\ViewObject|BrowserProhlaseniPoplatnikaDane|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!($this->pravo_prohlaseni->jsouNejakePrava() || $this->pravo_vyuctovani->jsouNejakePrava())){
            RonPortalUtils::neniPravoException();
        }

        HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('MI_Dane'));

        //Pokud by byli prava na dane prirazeny ale Mzdy nejsou...
        if (!RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY')){
            MessageManager::addErrorMessage(ronText('Mzdy_nejsou_implementovany'));
            RonPortalUtils::neniPravoException();
        }

        $daneaction = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($daneaction) {

            case 'dokumentupload' :
            case 'mojeprohlaseni':
                return parent::handleAction($action);

            // dokumenty
            case 'dokumentdelete' :
                return $this ->smazDokument((int)$_GET['record'], $_GET['osoba'], $_GET['rok']);

            case 'dokumentdownload' : // download dokumentu
                return $this->downloadDokument((int)$_GET['record'], $_GET['osoba'], $_GET['rok']);

            case 'formularprohlaseni':
                return $this->zobrazitFormularProhlaseni($_GET['osoba'], $_GET['obdobi'], isset($_GET['zmenovacast']));

            case 'ulozitprohlaseni':
                return $this->zapisProhlaseni();

            case 'zmenovacast':
                return $this->zmenovaCastProhlaseni($_POST['osoba']);

            case 'prohlaseninerezidenta':
                return $this->zapisProhlaseniNerezidenta();

            case 'rocnizuctovani':
                //Odkazem
                return $this->zadostZuctovaniZapisPresOdkazBrowseru($_GET['obdobi'], $_GET['zadamzuctovani'], $_GET['osoba']);


            case 'zadost':
                return $this->browserZadostiORocniZuctovaniZaloh();

            case 'ulozitzadost':
                return $this->zapisZadosti();


            case 'translations':
                return $this->translations();


            default:
                return $this->browserProhlaseniPoplatnikaDane();
        }
    }
}