<?php

namespace RonPortal\Mzdy;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\ProhlaseniPoplatnikaDane;
use Ron\RonPortal\ZadostOZuctovaniZaloh;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\User;
use function Ron\ronText;

require_once ('mzdy/danecontroller.php');
require_once ('mzdy/zadostozuctovanizaloh.php');
require_once ('mzdy/prohlasenipoplatnikadane.php');

class DaneController extends \Ron\RonPortal\DaneController
{


    /** @var Database */
    public $database;

    /** @var PracovniPomer $pracovniPomer */
    public $pracovniPomer;


    /** @var PravaNaOsoby $pravo_vyplnit */
    public $pravo_prohlaseni;

    /** @var PravaNaOsoby $pravo_vyuctovani */
    public $pravo_vyuctovani;

    /**
     * DaneController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
//        $this->database = $database;
//
//        $this->pravo_prohlaseni = Prava::vratPrava('dane_prohlaseni');
//        $this->pravo_vyuctovani = Prava::vratPrava('dane_vyuctovani');
    }

    /**
     * @param int $rok Rok vyuctovani dane
     * @return bool
     * @throws Exception
     */
    public static function lzeEditovatVyuctovani($rok)
    {
        if (RonPortalSettings::getIntValue('zadostizuctovanidanepoterminu')) {
            return true;
        }

        //editace vyuctovani za zvoleny rok je mozna od 1.1. nasledujiciho roku
        $editovatOd = DateUtils::incMonth($rok, 12);
        //do zvoleneho terminu v danem (nasledujicim) roce (napr. 28.2.) - pokud neni zadano bere se 31.3.
        $editovatDo = DateUtils::dateStringToTimestamp(RonPortalSettings::getValue('daneterminvyplneni', '31.3.') . date('Y', $editovatOd));

        if ((DateUtils::today() >= $editovatOd) && (DateUtils::today() <= $editovatDo)) {
            return true;
        }

        return false;
    }

    /** Vrati typ dokumentu pro zvolenou sekci
     * @param $sekce
     * @return int|mixed
     */
    public static function vratTypDokumentuProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case ZadostOZuctovaniZaloh::DOKUMENT_ZAMESTNAVATEL :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_typ', -5);
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_PARTNER :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ', -6);
            case ZadostOZuctovaniZaloh::DOKUMENT_SKOLKOVNE :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ', -7);
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ', -8);
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_EXEKUCE :
                return RonPortalSettings::getIntValue('dane_dokumenty_exekuce_typ', -11);
            //prohlaseni
            case ProhlaseniPoplatnikaDane::DOKUMENT_DETI :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_typ', -2);
            case ProhlaseniPoplatnikaDane::DOKUMENT_PARTNER_NEUPLATNUJE :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_typ', -4);
            case ProhlaseniPoplatnikaDane::DOKUMENT_OBECNY :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ', -3);
        }
    }

    /** Vrati true pokud je prilozeni dokumentu pro danou sekci povinne
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return int|mixed
     */
    public static function povinnyDokumentProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case ZadostOZuctovaniZaloh::DOKUMENT_ZAMESTNAVATEL :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne');
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_PARTNER :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne');
            case ZadostOZuctovaniZaloh::DOKUMENT_SKOLKOVNE :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne');
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_povinne');
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_EXEKUCE :
                return RonPortalSettings::getIntValue('dane_dokumenty_exekuce_povinne');
            //prohlaseni
            case ProhlaseniPoplatnikaDane::DOKUMENT_DETI :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne');
            case ProhlaseniPoplatnikaDane::DOKUMENT_PARTNER_NEUPLATNUJE :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne');
            case ProhlaseniPoplatnikaDane::DOKUMENT_OBECNY :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne');
        }
    }

    /** Vrati typ formulare (akci pro url) dle zvolene sekce.
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return string
     */
    public static function vratTypFormulareDleSekce($sekce)
    {
        switch ($sekce) {
            case ZadostOZuctovaniZaloh::DOKUMENT_ZAMESTNAVATEL :
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_PARTNER :
            case ZadostOZuctovaniZaloh::DOKUMENT_SKOLKOVNE :
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET :
            case ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_EXEKUCE :
                return 'formularzadosti';
            case ProhlaseniPoplatnikaDane::DOKUMENT_DETI :
            case ProhlaseniPoplatnikaDane::DOKUMENT_PARTNER_NEUPLATNUJE :
            case ProhlaseniPoplatnikaDane::DOKUMENT_OBECNY :
            default :
                return 'formularprohlaseni';
        }
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param int $rok
     * @param int $typDokumentu
     * @param string $popisDokumentu
     * @return int
     * @throws DBException
     */
    public static function vratNovePoradoveCisloDokumentu($database, $pracovniPomer, $rok, $typDokumentu, $popisDokumentu = null)
    {
        $params = $database->createParameters();
        if (!is_null($popisDokumentu)) {
            $sql =
                "SELECT Popis FROM Dokument JOIN DokumentyOsoby ON Dokument.id = DokumentyOsoby.id 
                WHERE DokumentyOsoby.VyuctovaniDane_RC = " . $params->addParameter($pracovniPomer->rc) .
                "  AND DokumentyOsoby.VyuctovaniDane_Rok = " . $params->addDateParameter($rok) .
                "  AND Dokument.typ = " . $params->addParameter($typDokumentu) .
                "  AND Dokument.Popis LIKE " . $params->addParameter("%{$popisDokumentu}%") .
                " ORDER BY Dokument.id DESC";
            $posledniPopis = DBUtils::getSqlValue($database, $sql, $params);

            $posledniCislo = substr($posledniPopis, -1);
        } else {
            $sql =
                "SELECT COUNT(Dokument.id) FROM Dokument JOIN DokumentyOsoby ON Dokument.id = DokumentyOsoby.id 
                WHERE DokumentyOsoby.VyuctovaniDane_RC = " . $params->addParameter($pracovniPomer->rc) .
                "  AND DokumentyOsoby.VyuctovaniDane_Rok = " . $params->addDateParameter($rok) .
                "  AND Dokument.typ = " . $params->addParameter($typDokumentu);
            $posledniCislo = DBUtils::getSqlValue($database, $sql, $params);
        }

        return (int)$posledniCislo + 1;
    }

    /** Zobrazi seznam pripojenych dokumentu k sekci s moznosti vlozeni noveho.
     * @param Database $database
     * @param $rok
     * @param $pracovniPomer
     * @param string $sekceFormulare ke ktere casti jsou dokumenty mozne hodnoty: deti, partner, odecet
     * @param $povolitEditaci
     * @param $kontrolovatPovinnost
     * @throws DBException
     */
    public static function zobrazDokumenty($database, $rok, $pracovniPomer, $sekceFormulare, $povolitEditaci, $kontrolovatPovinnost = true, $castiSekce = null)
    {
        echo "<A name='dokumenty'></A>\n";

        $typDokumentu = self::vratTypDokumentuProSekci($sekceFormulare);
        $povinny = false;
        if ($kontrolovatPovinnost) {
            $povinny = self::povinnyDokumentProSekci($sekceFormulare);
        }

        if ($typDokumentu) {
            // Seznam dokumentu
            $b = new DokumentyVyuctovaniBrowser($database, $rok, $pracovniPomer, $typDokumentu, $povolitEditaci, $povinny, self::vratTypFormulareDleSekce($sekceFormulare));
            $b->load();
            $b->render();

            if ($povolitEditaci) {
                echo "<br/>";

                if (is_array($castiSekce)) {
                    foreach ($castiSekce as $cast => $popis) {
                        echo "<label for='{$sekceFormulare}_{$cast}'>$popis</label>
                            <span id='{$sekceFormulare}_{$cast}'><input type='file' name='{$sekceFormulare}_{$cast}'></span><br/>";
                    }
                } else {
                    echo "<label for='{$sekceFormulare}'>" . ronText('Vlozit_novy_dokument') . "</label><br/>
                            <span id='{$sekceFormulare}'><input type='hidden' name='{$sekceFormulare}_pocet' value='1'><input type='file'  name='{$sekceFormulare}'></span>";

                    echo "<br/> <a href='#' title='" . TextManager::_('Vlozit_novy_dokument') . "'>" .
                        "<i class='inline-glyph fa glyph-upload add-dokument-input' data-input-name='{$sekceFormulare}' ></i> " .
                        "</a>";

                    echo "<br/>";
                }

                echo "<div class='font-italic padding-bottom-2'>" . sprintf(TextManager::_('dokument_max_velikost_soboru'), ini_get('upload_max_filesize'), ini_get('post_max_size')) . "</div>";
            }
        }
    }

    /** Zobrazi odkaz na pripojeni dokumentu k radku nezdanitelnych polozek (slev na dani). Umoznuje cislovani popisu dokumentu.
     * @param Database $database
     * @param int $rok
     * @param \RonPortal\PracovniPomer $pracovniPomer
     * @param string $castProhlaseni
     * @param string $popisDokumentu
     * @throws DBException
     */
    public static function zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, $castProhlaseni, $popisDokumentu)
    {
        echo
            "<a href='#' title='" . TextManager::_('Vlozit_novy_dokument') . "'>" .
            "<i class='inline-glyph fa glyph-upload show-dokument-input-dialog' data-input-name='{$castProhlaseni}_{$popisDokumentu}' ></i> " .
            "</a>";
    }

//    /**
//     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
//     * @param array $historieOdpoctuNaDeti
//     * @return bool
//     */
//    public function jePovinnyDokument($hodnotyProhlaseni, $historieOdpoctuNaDeti)
//    {
//        $povinnyDokumentDeti = false;
//        if (self::povinnyDokumentProSekci('deti')) {
//            $povinnyDokumentDeti = true;
//
//            $odpocetNaDetiZmeny = $historieOdpoctuNaDeti['zmeny'];
//
//            if (empty($odpocetNaDetiZmeny) || count($odpocetNaDetiZmeny) == 0)
//                $povinnyDokumentDeti = false;
//
//            $uplatnujeOdpocetNaDite = false;
//            foreach ($odpocetNaDetiZmeny as $dite) {
//                if (($dite['datumOdNavrh'] != '' && $dite['poradiDitete_navrh'] != '') || ($dite['datumOd'] != '' && $dite['poradiDitete'] != '')) {
//                    $uplatnujeOdpocetNaDite = true;
//                    break;
//                }
//            }
//            if (!$uplatnujeOdpocetNaDite)
//                $povinnyDokumentDeti = false;
//        }
//
//        if ($povinnyDokumentDeti
//            || self::povinnyDokumentProSekci('partner')
//            || (self::povinnyDokumentProSekci('odecet') && (($hodnotyProhlaseni->naZdravotniStav != $hodnotyProhlaseni->naZdravotniStavNavrh)
//                    || ($hodnotyProhlaseni->odpocetNaStudenta != $hodnotyProhlaseni->naZdravotniStavNavrh))))
//            return true;
//
//        return false;
//    }
//
//    public function zkontrolujPovinneDokumentyProhlaseni($rok, $hodnoty)
//    {
//        $typ = RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
//        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne') && ($hodnoty['pocetdetipoplatnika'] > 0)) {
//            $diteStr = isset($hodnoty['pozaduji_upravu_poradi_ditete']) ? 'poradi_ditete_navrh_' : 'poradi_ditete_old_';
//            $uplatnujeOdecet = false;
//            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {
//                if ($hodnoty["$diteStr{$i}"] > 0)
//                    $uplatnujeOdecet = true;
//            }
//            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
//            if ($uplatnujeOdecet) {
//                $this->jePripojenyDokument($typ, $rok);
//            }
//        }
//
//        //TODO: kontrolovat povinnost jen pokud je zadana zmena na invaliditu nebo studenta
//        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
//        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne') && $this->jeZmenaSlev($hodnoty))
//            $this->jePripojenyDokument($typ, $rok);
//
//        $typ = RonPortalSettings::getIntValue('dane_dokumenty_partner_typ');
//        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne'))
//            $this->jePripojenyDokument($typ, $rok);
//    }
//

    public static function zkontrolujNastaveni()
    {
        $chybaZamestnavatele = false;
        $chybaPartner = false;

        //pokud je vyzadovan dokument o predchozim zamestnavateli a nejde zadat zadneho, vypise chybu
        if (RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne') && RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3) == 0) {
            $chybaZamestnavatele = true;
        }
$pocet = RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3);
        //pokud se vyzaduje potvrzeni, ze druhy poplatnik neuplatnuje slevu na dite a udaje druheho poplatnika neni mozne zadavat pak vypise chybu
        if (RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne') && RonPortalSettings::getIntValue('dane_skryt_dalsiho_poplatnika') == 1) {
            $chybaPartner = true;
        }

        if ($chybaZamestnavatele || $chybaPartner) {
            MessageManager::addWarning(TextManager::_('Mzdy_dane_chyba_nastaveni'));
        }

        if (Prava::vratPrava('admin')->jsouNejakePrava()) {
            if ($chybaZamestnavatele) {
                MessageManager::addErrorMessage(TextManager::_('Mzdy_dane_chyba_nastaveni_zamestnavatel'));
            }
            if ($chybaPartner) {
                MessageManager::addErrorMessage(TextManager::_('Mzdy_dane_chyba_nastaveni_partner'));
            }
        }
    }

    public function zkontrolujPovinneDokumentyVyuctovani($rok, $hodnoty)
    {
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ', -7);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne') && ($hodnoty['pocetdetipoplatnika'] > 0)) {
            $uplatnujeSkolkovne = false;
            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {
                $rcDitete = $hodnoty['ditepoplatnika_' . $i];
                $skolkovneNaDite = $hodnoty['skolkovnenadite_' . $rcDitete];
                if ($skolkovneNaDite > 0)
                    $uplatnujeSkolkovne = true;
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeSkolkovne) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ', -8);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_povinne')) {
            if (($hodnoty['zadostdaryvyse'] != "" && $hodnoty['zadostdaryvyse'] > 0) ||
                ($hodnoty['zadosturokyvyse'] != "" && $hodnoty['zadosturokyvyse'] > 0) ||
                ($hodnoty['zadostpenzijnivyse'] != "" && $hodnoty['zadostpenzijnivyse'] > 0) ||
                ($hodnoty['zadostpojistnevyse'] != "" && $hodnoty['zadostpojistnevyse'] > 0) ||
                ($hodnoty['zadostclenskevyse'] != "" && $hodnoty['zadostclenskevyse'] > 0) ||
                ($hodnoty['zadostuhradazazkouskyvyse'] != "" && $hodnoty['zadostuhradazazkouskyvyse'] > 0) )
            $this->jePripojenyDokument($typ, $rok);
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ', -6);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne')) {
            if (isset($hodnoty['slevanapartnera']) && ($hodnoty['slevanapartnera'] !== '')) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_typ', -5);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne')) {
            if (RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3) && (($hodnoty['predchoziplatcedaneadresa_0'] !== '') || ($hodnoty['predchoziplatcedaneadresa_1'] !== '') || ($hodnoty['predchoziplatcedaneadresa_2'] !== ''))) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_exekuce_typ', -11);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_exekuce_povinne')) {
            if (isset($hodnoty['nahrada_exekuce']) && ((float)$hodnoty['nahrada_exekuce'] > 0)) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }
    }

    /** Zkontroluje zda je k vyctovani zvoleneho roku pripojen dokument daneho typu.
     * @param $typDokumentu
     * @param $rok
     * @return void
     * @throws DBException
     */
    public function jePripojenyDokument($typDokumentu, $rok)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentyOsoby.Id ' .
            '  FROM DokumentyOsoby JOIN Dokument ON (Dokument.Id = DokumentyOsoby.Id) ' .
            ' WHERE DokumentyOsoby.RC =  ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND DokumentyOsoby.VyuctovaniDane_Rok = ' . $params->addParameter($this->database->dateToParam($rok)) .
            '   AND DokumentyOsoby.VyuctovaniDane_RC = ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND Dokument.typ = ' . $params->addParameter($typDokumentu);

        if (!$cursor = $this->database->select($sql, $params)) {
            throw new DBException($this->database, $sql);
        }

        $row = $cursor ->fetch();
        if (!$row) {
            $params = $this->database->createParameters();
            $typDokumentuPopis = DBUtils::getSqlValue($this->database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            throw new Exception(sprintf(ronText('Prohlaseni_neexistuje_povinny_dokument'), $typDokumentuPopis));
        }
    }

//    public function jeZmenaSlev($hodnoty)
//    {
//        if (isset($hodnoty['pozaduji_upravu_slev']) && (($hodnoty['sleva_na_zdravotni_stav1'] != $hodnoty['sleva_na_zdravotni_stav1_old'])
//                || ($hodnoty['sleva_na_zdravotni_stav2'] != $hodnoty['sleva_na_zdravotni_stav2_old']) || ($hodnoty['sleva_na_studenta_nova'] != $hodnoty['odpocetNaStudenta_old'])))
//            return true;
//
//        return false;
//    }

    /**
     * @return Redirect|RocniVyuctovaniDaneBrowser
     * @throws DBException
     */
    public function browserProhlaseniPoplatnikaDane()
    {
        $osoba = isset($_GET['osoba']) ? $_GET['osoba'] : '';
        //Pokud se jedna o dny kdy jeste nebylo vytvoreno obdobi pak nemusi jeste existovat zaznam o rocnim vyuctovani dane, proto zkontrolovat
        $obdobi = DateUtils::firstDayOfYear(DateUtils::today());         //1.1.aktu.roku

        //14.2.2022 zrusena kontrola na PP se zdanenim po dohode s Radovanem Kulou, nove se nacitaji PP platne v minulem a aktualnim roce
        //Kontrola na PP se zdanenim, musí existovat alespon jeden
        /*$ppSeZdanenim = $this->kontrolaNaVicePracovnichPomeruSeZdanenim($this->database, $obdobi);
        if (empty($ppSeZdanenim)){
            //Pokud existuje pak neni jiste zda je tento PP prirazen k RVD, tedy pokusi se najit PP v RVD, pokud tam nebude pak se pravdepodobne jedna o 1. pristup
            //a nutno vyzvat uzivatele k vyberu PP, tedy v pripade ze se nejedná o zamestnance s prave jednim PP
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }*/

        //Pokud je v URL osoba pak jiz nekontrolovat
        if ($osoba != "") {
            //Kontroly zda je vytvoreno, zda ma nastaveno IDPP, zda existuje vice PP a zaroven neni nastaveno IDPP v RVD
            //Pred zobrazenim browseru overi zda existuje zaznam na aktualni rok, pokud neni pak ho doplni a pouzije hodnoty z lonska pokud jsou k dispozici
            $this->kontrolaZaznamuRocnihoVyuctovaniDane($osoba, $obdobi);
        } else {
            $osoba = $this->nactiPracovniPomerZRocnihoVyuctovaniDane($obdobi);
            if (!is_null($osoba)) {
                $pp = PracovniPomer::nactiPP($this->database, $osoba);
                $pp->vyberNactenyPP();
            } else {
                $seznamPPosoby = PracovniPomer::nactiPPOsoby($this->database, User::getRC());
                //pokud ma uzivatel jen jeden PP, pak vytvorit zaznam vyuctovani z kopie predchoziho
                if (count($seznamPPosoby) == 1) {
                    $osoba = $seznamPPosoby[0];
                    $this->kontrolaZaznamuRocnihoVyuctovaniDane($osoba->idPP, $obdobi);
                }
            }
        }

        //Pokud je vic PP a neni v RVD urcen konkretni pak se vypise hlaska a uzivatel se vyzve aby jeden zvolil.
        require_once ('mzdy/b_prohlasenipoplatnikadane.php');
//        $b = new BrowserProhlaseniPoplatnikaDane($this->database, null, $obdobi);
        $b = new RocniVyuctovaniDaneBrowser($this->database, $obdobi);
        $b->load();

        return $b;
    }

    /**
     * @param $obdobi
     * @param $idPP
     * @return BaseViewObject
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zobrazitFormularZadostiOZuctovaniDane($obdobi, $idPP)
    {
        //Načte PP
        $this->pracovniPomer = new PracovniPomer();
        $this->pracovniPomer->nactiPPPodleId($this->database, $idPP);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if (!($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))) {
            throw new NeniPravoException();
        }

        require_once 'mzdy/zadostozuctovanizaloh.php';
        $hodnotyZadosti = ZadostOZuctovaniZaloh::nactiZadost($this->database, $this->pracovniPomer, $obdobi);

        $chybaPP = false;
        if ($hodnotyZadosti->zadostPortalStav == 3)
        {
            //Formulář nelze vyplnit, dokud nebude opraveno mzdovým pracovníkem
            $chybaPP = true;
        }

        Model::addData('database', $this->database);

        //TODO datumObdobiRok a datumObdobi_timestamp sjednotit !
        Model::addData('datumObdobiRok', $obdobi);
        Model::addData('datumObdobi_timestamp', $obdobi);
        Model::addData('chybaPP', $chybaPP);
        Model::addData('hodnotyZadosti', $hodnotyZadosti);
        Model::addData('volby_comboboxu', $this->volbyComboBoxuZadostiAProhlaseni());
        Model::addData('adresa', $this->aktualniAdresaPoplatnika($obdobi));
        Model::addData('pracovniPomer', $this->pracovniPomer);
        Model::addData('partnerPoplatnika', $this->nactiPartneraPoplatnika($this->database, $this->pracovniPomer, $obdobi, true));

        require_once 'mzdy/prohlasenipoplatnikadane.php';
        $hodnotyProhlaseni = ProhlaseniPoplatnikaDane::nactiProhlaseni($this->database, $this->pracovniPomer, $obdobi);
        Model::addData('detiPoplatnika', $this->nactiOdpocetNaDeti($hodnotyProhlaseni, $this->pracovniPomer));
        Model::addData('nastaveniOdpoctu', NastaveniOdpoctu::nactiNastaveni($this->database, $obdobi));

        Model::addData('reportlist', Report::loadFormReports($this->database, 'DaneVyuctovani'));

        switch (date('Y', $obdobi)) {
            case 2022:
            case 2023:
                return new BaseViewObject("ronportal/mzdy/view_zadostozuctovanizaloh_vz3.php");
            case 2021:
                return new BaseViewObject("ronportal/mzdy/view_zadostozuctovanizaloh_vz2.php");
            case 2020:
            default:
                return new BaseViewObject("mzdy/view_zadostozuctovanizaloh.php");
        }

    }

    /**
     * @return \Ron\MVC\BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zapisZadosti()
    {
        $rok = $_POST['datumobdobi'];
        //Kontrola zda nedošlo k submitu po termínu
        if (!static::lzeEditovatVyuctovani($rok)) {
            throw new NeniPravoException();
        }

        $hodnoty = $this->nactiPOSTZadosti();

        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, (int)$hodnoty["osoba"]);

        $recVyuctovani = new RocniVyuctovaniDaneRecord($this->database,'Rocni vyuctovani dane poplatnika - portal');

        $recVyuctovani->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        //Kontrola zdali žádost nebyla schválena mzdovým pracovníkem v průběhu editace formuláře
        $recVyuctovani->jeZadostSchvalena();

        try {
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_ZAMESTNAVATEL, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_PARTNER, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SKOLKOVNE, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_dary');
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_uroky');
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_penzijni');
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_zivotni');
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_clenske_prispevky');
            $this->dokumentUpload(ZadostOZuctovaniZaloh::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_zkousky');

            $this->zkontrolujPovinneDokumentyVyuctovani($rok, $hodnoty);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularZadostiOZuctovaniDane($rok, $this->pracovniPomer->idPP);
        }

        //Pokud ještě není žádost schválená pak spočítat školkovné - a zapsat sumu
        $sumaSkolkovnehoProVyuctovaniDane = 0;

        if ($hodnoty['pocetdetipoplatnika'] > 0) {
            for ($i = 0; $i < $hodnoty['pocetdetipoplatnika']; $i++) {

                $rcDitete = $hodnoty['ditepoplatnika_' . $i];
                //Pokud pres kontrolu klienta projde skolkovne vetsi nez max. pak nastavit na max
                $skolkovneNaDite = min($hodnoty['skolkovnenadite_' . $rcDitete], $hodnoty['skolkovne_max']);
                $skolkovneNaDitePuvodni = $hodnoty['skolkovnenaditeold_' . $rcDitete];

                if (is_numeric($skolkovneNaDite)) {
                    $sumaSkolkovnehoProVyuctovaniDane += $skolkovneNaDite;
                }

                if ($skolkovneNaDite !== $skolkovneNaDitePuvodni) {
                    $params = $this->database->createParameters();

                    $rec = new RonPortalDBRecord($this->database, 'SkolkovneNaDite');
                    $rec->logDescription = 'Skolkovne na dite poplatnika - portal';

                    $rec->addField('Rok', 'timestamp', true, 0, $rok);
                    $rec->addField('RC', '', true, 15, $this->pracovniPomer->rc);
                    $rec->addField('RC_RodinnehoPrislusnika', '', true, 15, $rcDitete);
                    $rec->addField('Skolkovne', 'float', false, 0, (int)$skolkovneNaDite);

                    $rec->setWhere(
                        'Rok = ' . $params->addParameter($this->database->timestampToParam($rok)) .
                        ' AND RC = ' . $params->addParameter($this->pracovniPomer->rc) .
                        ' AND RC_RodinnehoPrislusnika = ' . $params->addParameter($rcDitete), $params);

                    try {
                        $rec->updateOrInsert();
                    }
                    catch (Exception $exception) {
                        throw new Exception(StringUtils::fromUTF8('Chyba při zápisu/aktualizaci školkovného: ' . $exception->getMessage()));
                    }
                }
            }
        }

        $termin = mktime(0,0,0,2,15, (int)date("Y", time()));
        $aktualniDatum = time();
        $datumPortal = ($termin < $aktualniDatum) ? $termin : $aktualniDatum;
        $stav = isset($hodnoty['podepsatzadost']) ? 1 : 0;

        $partnerZtpMesicu = 0;
        $partnerMesicu = 0;
        $slevaNaPartneratext = '';

        //Zápis ve formě mm-mm;mm-mm
        if (isset($hodnoty['slevanapartnera']) && ($hodnoty['slevanapartnera'] !== "" )) {
            $slevaNaPartneratext = $this->kontrolaRozsahuMesicu($hodnoty['slevanapartnera']) . ";" . (isset($hodnoty['slevanapartneraproztp']) ? $this->kontrolaRozsahuMesicu($hodnoty['slevanapartneraproztp']) : '');

            $partnerMesiceOd = (int)(explode("-", $hodnoty['slevanapartnera'])[0]);
            $partnerMesiceDo = (int)(explode("-", $hodnoty['slevanapartnera'])[1]);

            $partnerMesicu = $partnerMesiceDo - $partnerMesiceOd;
            $partnerMesicu += 1;
        }

        //Zápis ve formě mm-mm;mm-mm
        if (isset($hodnoty['slevanapartneraproztp']) && ($hodnoty['slevanapartneraproztp'] !== "")) {
            $partnerZtpMesiceOd = (int)(explode("-", $hodnoty['slevanapartneraproztp'])[0]);
            $partnerZtpMesiceDo = (int)(explode("-", $hodnoty['slevanapartneraproztp'])[1]);

            $partnerZtpMesicu = $partnerZtpMesiceDo - $partnerZtpMesiceOd;
            $partnerZtpMesicu += 1;
        }

        if ($partnerZtpMesicu > $partnerMesicu) {
            throw new Exception(ronText('zadost_chyba_pocet_mesicu_ztp_partnera'));
        }

        $recVyuctovani->nastavProvedeniVyuctovani(1);

        $recVyuctovani->findFieldByName('AdresaBytovePotrebyUveru')->setNewValue($hodnoty['zadostbytovepotrebyuveruadresa']);
        $recVyuctovani->findFieldByName('OpravaHodnotDaru')->setNewValue($hodnoty['zadostdaryvyse'] != "" && $hodnoty['zadostdaryvyse'] > 0 ? $hodnoty['zadostdaryvyse'] : 0);

        //do vyuctovani 2020 se zadavala jedna castka uveru, od roku 2021 je ve formulari zatrzitko na vyber zda uver byl pred rokem 2021 nebo az po
        if (date('Y', $rok) <= 2020) {
            $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue($hodnoty['zadosturokyvyse'] != "" && $hodnoty['zadosturokyvyse'] > 0 ? $hodnoty['zadosturokyvyse'] : 0);
        } elseif ((int)$hodnoty['zadosturokyvyse'] == 0) {
            $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue(0);
            $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue(0);
        } elseif ($hodnoty['zadosturokyvyse'] != "") {
                //13.1.2023 Teams RaKu: takže pokud zaškrtnou obě varianty, zapiš do každé položky max. 150. Nejprve použij tu nižší (do 150 000) a zbytek dej do té druhé.
            if (!empty($hodnoty['zadostbytovepotrebypred2021']) and !empty($hodnoty['zadostbytovepotrebypo2021'])) {
                $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue(min((float)$hodnoty['zadosturokyvyse'], RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021));
                if ((float)$hodnoty['zadosturokyvyse'] > RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021) {
                    $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue((float)$hodnoty['zadosturokyvyse'] - RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021);
                } else {
                    $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue(0);
                }
            } elseif (!empty($hodnoty['zadostbytovepotrebypred2021'])) {
                $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue((float)$hodnoty['zadosturokyvyse']);
            } elseif (!empty($hodnoty['zadostbytovepotrebypo2021'])) {
                $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue((float)$hodnoty['zadosturokyvyse']);
            }
        }

        $recVyuctovani->findFieldByName('OpravaPenzijnihoPrip')->setNewValue($hodnoty['zadostpenzijnivyse'] != "" && $hodnoty['zadostpenzijnivyse'] > 0 ? $hodnoty['zadostpenzijnivyse'] : 0);
        $recVyuctovani->findFieldByName('ZivotniPojisteni')->setNewValue($hodnoty['zadostpojistnevyse'] != "" && $hodnoty['zadostpojistnevyse'] > 0 ? $hodnoty['zadostpojistnevyse'] : 0);
        $recVyuctovani->findFieldByName('OpravaClenskychPrispevku')->setNewValue($hodnoty['zadostclenskevyse'] != "" && $hodnoty['zadostclenskevyse'] > 0 ? $hodnoty['zadostclenskevyse'] : 0);
        $recVyuctovani->findFieldByName('OpravaUhradZaZkousky')->setNewValue($hodnoty['zadostuhradazazkouskyvyse'] != "" && $hodnoty['zadostuhradazazkouskyvyse'] > 0 ? $hodnoty['zadostuhradazazkouskyvyse'] : 0);
        $recVyuctovani->findFieldByName('OsobyUplatnujiciUver')->setNewValue($this->sestaveniOsobUplatnujicichUverZHodnot($hodnoty));
        $recVyuctovani->findFieldByName('PredchoziPrijmyKdyOdKoho')->setNewValue($this->sestaveniPredchozichPrijmuKdyOdKohoZHodnot($hodnoty));
        $recVyuctovani->findFieldByName('Skolkovne')->setNewValue((float)$sumaSkolkovnehoProVyuctovaniDane);

//        $recVyuctovani->addField('DatumZadostiPortal', 'timestamp', false, 0, $datumPortal); TODO overit

        //pri kliknuti na schvalit se zapise i elektronicky podpis
        if ($stav == 1) {
            $recVyuctovani->podepsatZadost();
        }

        $recVyuctovani->findFieldByName('UplatnujeOdpocetPartnera')->setNewValue(isset($hodnoty['slevanapartnera']) && ($hodnoty['slevanapartnera'] !== ""));

        $recVyuctovani->findFieldByName('SlevaPartnerObdobi')->setNewValue($slevaNaPartneratext);
        $recVyuctovani->findFieldByName('PocetMesicuOdpoctu')->setNewValue($partnerMesicu);
        $recVyuctovani->findFieldByName('PartnerZTPP')->setNewValue($partnerZtpMesicu);

        $recVyuctovani->findFieldByName('SlevaZaZastavenouExekuci')->setNewValue((float)$hodnoty['nahrada_exekuce']);

        $recVyuctovani->aktualizaceVypoctuVyuctovani();

        if (!$recVyuctovani->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_rocniho_vyuctovani"));
        }

        //jen uložení, znovu zobrazí formulář
        if ($stav == 0) {
            return new Redirect(null, "?menuid=dane&action=formularzadosti&obdobi=$rok&osoba={$this->pracovniPomer->idPP}");
        }

        return new Redirect(null, "?menuid=dane");
    }

    public function zapisProhlaseni()
    {
        $hodnoty = $this->nactiPOSTProhlaseni();

        $rok = (mktime(0,0,0,1,1,$hodnoty["datumobdobi"]));

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $hodnoty["osoba"]);
        $this->pracovniPomer = $pracovniPomer;

        if (!($this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))){
            throw new NeniPravoException();
        }

        $rec = new RocniVyuctovaniDaneRecord($this->database, 'Zapis prohlaseni poplatnika dane (podepsani) - portal');

        $rec->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        //Kontrola zdali prohlášení nebylo schváleno mzdovým pracovníkem v průběhu editace formuláře prohlášení
        $rec->jeProhlaseniSchvaleno();
        //kontrola "ProhlaseniElektronicky"
        $rec->lzePodepsatProhlaseniElektronicky();

        try {
            $this->dokumentUpload(ProhlaseniPoplatnikaDane::DOKUMENT_OBECNY, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(ProhlaseniPoplatnikaDane::DOKUMENT_DETI, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(ProhlaseniPoplatnikaDane::DOKUMENT_PARTNER_NEUPLATNUJE, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);

            $this->zkontrolujPovinneDokumentyProhlaseni($rok, $hodnoty);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularProhlaseni($this->pracovniPomer->idPP, $rok, isset($_GET['zmenovacast']));
        }

        //Pokud je ma RVD v polich ProhlaseniPortalRC, ProhlaseniPortalOsc, ProhlaseniPortalDatNastup jiny IDPP nez je vybran ve
        //..filtru Osoby, pak nastavit $zmenaPPIDRVD na true a odstranit pozadovane navolene zmeny slev na dani
        $zmenaPPIDRVD = ((int)$hodnoty["prohlaseniPPID"] != (int)$hodnoty["osoba"] ? true: false);


        $this->ukliditPoZmenePracovnihoPomeruVRocniVyuctovaniDane($pracovniPomer, $hodnoty, $zmenaPPIDRVD);

        $stav = isset($hodnoty['podepsatprohlaseni']) ? 1 : 0;

        //RocniVyuctovaniDane ---------------------------------------------------

        //zapise se jen pri podepsani, ne pri ulozeni
        if ($stav == 1) {
            $rec->podepsatProhlaseni($hodnoty['prohlasenikdanicinim']);
//            $rec->findFieldByName('DatumProhlaseniPortal')->setNewValue(time());
        }
        //Uzivatel muze prohlaseni "nepodepsat"
//        $rec->addField('ProhlaseniPortalStav', 'int', false, 0, 0);
//        $rec->addField('ProhlaseniPortal', 'int', false, 0, $hodnoty['prohlasenikdanicinim']);
//        $rec->addField('ElektronickyPodpis', 'memo', false, 0, $this->signaturaUzivatele());

        $rec->findFieldByName('VyzivujeJinyPoplatnik')->setNewValue((int)$hodnoty['vyzivujeJinyPoplatnik']);
        $rec->findFieldByName('JinyPoplatnikUplatnujeZv')->setNewValue((int)$hodnoty['jinyPoplatnikUplatnujeZv']);
        $rec->findFieldByName('JinyPoplatnikZamestnani')->setNewValue((int)$hodnoty['jinyPoplatnikZamestnani']);
        $rec->findFieldByName('JinyPoplatnikJmeno')->setNewValue($hodnoty['jinyPoplatnikJmeno']);
        $rec->findFieldByName('JinyPoplatnikPrijmeni')->setNewValue($hodnoty['jinyPoplatnikPrijmeni']);
        $rec->findFieldByName('JinyPoplatnikRC')->setNewValue($hodnoty['jinyPoplatnikRC']);

        $rec->findFieldByName('JinyPoplatnikObec')->setNewValue($hodnoty['jinyPoplatnikObec']);
        $rec->findFieldByName('JinyPoplatnikPSC')->setNewValue($hodnoty['jinyPoplatnikPSC']);
        $rec->findFieldByName('JinyPoplatnikUlice')->setNewValue($hodnoty['jinyPoplatnikUlice']);

        $rec->findFieldByName('JinyPoplatnikNazevPlatce')->setNewValue($hodnoty['jinyPoplatnikNazevPlatce']);
        $rec->findFieldByName('JinyPoplatnikObecPlatce')->setNewValue($hodnoty['jinyPoplatnikObecPlatce']);
        $rec->findFieldByName('JinyPoplatnikUlicePlatce')->setNewValue($hodnoty['jinyPoplatnikUlicePlatce']);
        $rec->findFieldByName('JinyPoplatnikPSCPlatce')->setNewValue($hodnoty['jinyPoplatnikPSCPlatce']);

        //Nastavi se az pri podepsani a pouze tehdy kdyz NENI Prohlaseni schvaleno mzdovym pracovnikem
        //Pole muze menit dokud nebude schvaleno mzdovym pracovnikem
        if ($rec->findFieldByName('ProhlaseniPortalStav')->value != 1) {
            $rec->nastavPracovniPomer($pracovniPomer);
        }

        $rec->doplnHistoriiProhlaseni($stav == 1 ? ronText('prohlaseni_podepsani') : ronText('prohlaseni_ulozeni'));

        if (!$rec->updateRecord()) {
            throw new Exception(TextManager::_("chyba_aktualizace_prohlaseni"));
        }
        //RocniVyuctovaniDane ---------------------------------------------------



        //OdpocetNavrh - pokud došlo ke změně požadovaných slev -----------------
        //datum od - bude datum hlášené změny a poté již bude použit tento datum (při opětovném otevření bude hledán datum v rámci 1.1. - 31.12.)

        if ($hodnoty['prohlasenikdanicinim'])
        {
            //Pokud ano, pak zapsat zmeny
            if ($hodnoty["pozaduji_upravu_slev"]) {
                $datumOdOdpocetNavrh = $hodnoty["datumOdOdpocetNavrh"] != "" ? $hodnoty["datumOdOdpocetNavrh"] : DateUtils::today();
                //Zde OdpocetNavrh a OdpocetPuvodni je stejny, rozdilne mohou byt pouze u zmen, kde lze zadat jiny mesic nez ktery je aktualni
                $datumOdOdpocetPuvodni = $datumOdOdpocetNavrh;
                $datumVlozeniNavrhu = $hodnoty["datumVlozeniNavrhu"] != "" ? $hodnoty["datumVlozeniNavrhu"] : time();
                $slevaNaZdravotniStav = ($hodnoty["sleva_na_zdravotni_stav2"] == 1 ? 2 : ($hodnoty["sleva_na_zdravotni_stav1"] == 1 ? 1 : 0));

                //OdpocetNavrh ------------------------------------------------------
                $this->zapisOdpocetNavrh($pracovniPomer,
                    $datumOdOdpocetPuvodni,
                    $datumOdOdpocetNavrh,
                    $datumVlozeniNavrhu,
                    $hodnoty["sleva_zakladni_nova"],
                    $hodnoty["sleva_na_studenta_nova"],
                    $slevaNaZdravotniStav,
                    $hodnoty['sleva_ztpp'],
                    'Prohlaseni poplatnika dane a navrh odpoctu - portal');
            }


            //OdpocetNaDiteNavrh - pokud došlo ke změně poradi slev u ditete --------
            if ($hodnoty["pocetdetipoplatnika"] > 0) {
                for ($i = 0; $i < $hodnoty["pocetdetipoplatnika"]; $i++) {
                    $rcDitete = $hodnoty["dite_" . $i];
                    $datumOd = $hodnoty["old_datumOdNavrh_na_dite_" . $i] != "" ? $hodnoty["old_datumOdNavrh_na_dite_" . $i] : DateUtils::today();
                    $puvodniPoradi = $hodnoty["poradi_ditete_old_" . $i];
                    $novePoradi = $hodnoty["poradi_ditete_navrh_" . $i];

                    $datumVlozeniNavrhu = (int)$hodnoty["old_datumVlozeniNavrhu_na_dite_$i"] != "" ? $hodnoty["old_datumVlozeniNavrhu_na_dite_$i"] : time();

                    if ($puvodniPoradi != $novePoradi && $novePoradi != "") {
                        $this->zapisOdpocetNaDiteNavrh($pracovniPomer, $rcDitete, $datumOd, $datumOd, $datumVlozeniNavrhu, $novePoradi, 'Prohlaseni poplatnika dane a zmena odpoctu na deti - portal');
                    }
                }
            }
        }

        //Pokud doslo ke zmene osoby v RVD pak tuto osobu nacist do url
        $osobaDoURL = $zmenaPPIDRVD ? $hodnoty["osoba"] : $hodnoty["prohlaseniPPID"];

        //jen uložení, znovu zobrazí formulář
        if ($stav == 0) {
            return new Redirect(null, "?menuid=dane&action=formularprohlaseni&osoba=$osobaDoURL&obdobi=$rok");
        }

        return new Redirect(null, "?menuid=dane&osoba=$osobaDoURL");
    }

    /** Zapis zmenove casti, prevzaty z puvodniho kodu, castecne upraven.
     * @param $idPP
     * @param $rok
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zapisZmenyProhlaseni($idPP, $rok)
    {
        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, $idPP);

        $rok = mktime(0,0,0,1,1, $rok);

        $datumOd = $rok;
        $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($this->pracovniPomer->rc)) {
            throw new NeniPravoException();
        }

        $rec = new RocniVyuctovaniDaneRecord($this->database, 'Rocni vyuctovani dane poplatnika - aktualizace zmenove casti, portal');

        $rec->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        $rec->lzePodepsatProhlaseniElektronicky();

        $aktualniStavOdpoctuOsoby = $this->nactiAktualniStavOdpoctu($this->pracovniPomer);

        //zmeny porovnat s aktu. hodnotami
        $hodnotyPOSTu = $this->nactiPOSTZmenoveCastiAVyhledejZmenu($aktualniStavOdpoctuOsoby);
        $zmenyOdpoctuNavrh = $this->nactiOdpocetNavrh($this->pracovniPomer, $hodnotyPOSTu->obdobi_timestamp, false);

        try {
            $this->dokumentUpload(ProhlaseniPoplatnikaDane::DOKUMENT_OBECNY, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(ProhlaseniPoplatnikaDane::DOKUMENT_DETI, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);

            $this->zkontrolujPovinneDokumentyZmenyProhlaseni($rok, $hodnotyPOSTu);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularProhlaseni($this->pracovniPomer->idPP, $rok, true);
        }

        $aktualizovatRVDSlevy = false;
        $aktualizovatRVDDeti = false;
        if ($hodnotyPOSTu->nastalaZmenaOdpoctu && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev == 1) {

            //upravit na zvoleny mesic
            //existuje old mesic?
            if ($hodnotyPOSTu->zmenova_cast_datumOd_old != ""){
                $mesicNavrhovanyUzivatelem = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarni_mesice_nastale_zmeny, 1, (int)date("y", time()));
                $datumOdOdpocetNavrh = $mesicNavrhovanyUzivatelem != "" ? $mesicNavrhovanyUzivatelem : DateUtils::firstDayOfMonth(time());
                $datumOdPuvodni = $hodnotyPOSTu->zmenova_cast_datumOd_old;

            } else {
                $mesicNavrhovanyUzivatelem = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarni_mesice_nastale_zmeny, 1, (int)date("y", time()));
                $datumOdOdpocetNavrh = $mesicNavrhovanyUzivatelem != "" ? $mesicNavrhovanyUzivatelem : DateUtils::firstDayOfMonth(time());
                $datumOdPuvodni = DateUtils::firstDayOfMonth(time());
            }

            $aktualizovatRVDSlevy = $this->zapisOdpocetNavrh($this->pracovniPomer,
                $datumOdPuvodni, $datumOdOdpocetNavrh,
                $hodnotyPOSTu->zmenova_cast_datumVlozeniNavrhu,
                $hodnotyPOSTu->zmenova_cast_zakladniOdpocet,
                $hodnotyPOSTu->zmenova_cast_odpocetNaStudenta,
                $hodnotyPOSTu->zmenova_cast_naZdravotniStav,
                $hodnotyPOSTu->zmenova_cast_ztpp,'Rocni vyuctovani dane poplatnika, zmenova cast - portal', false);

        }


        if (($hodnotyPOSTu->zmenova_cast_pozadujiUpravuPoradiDeti) && (int)$hodnotyPOSTu->pocetDetiZmenyPoradi > 0 )
        {
            foreach ($hodnotyPOSTu->zmenova_cast_poradiDanovehoZvyhodneniNaDeti as $rcDitete => $novePoradi) {

                $kalendarniMesicZmeny = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti, 1, (int)date("y", time()));
                if ($hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti_old != "") {
                    $kalendarniMesicZmenyPuvodni = mktime(0, 0, 0, (int)$hodnotyPOSTu->zmenova_cast_kalendarniMesic_zmenyPoradiDeti_old, 1, (int)date("y", time()));
                } else {
                    $kalendarniMesicZmenyPuvodni = $kalendarniMesicZmeny;
                }

                //TODO doplnit kontrolu na zadani obsazeneho poradi ???? Nebo jak si ma program domyslet, že u tohoto chce uzivatel 1, a pokud 2, 3 nezada u ktereho chce co, pokud budou v konfliktu - jelikož 1 a 2 mohou byt pouze jednou!
                //TODO Domnivam se uzivatel by mel byt donucen pri zmene nastavit vsechny deti, tak aby poradi nebylo v konfliktu a bylo jednoznacne.
                $lzeZapsatNavrh = true;

                if ($lzeZapsatNavrh) {
                    //TODO pokud se bude upravovat poradi 1 u ditete s tim, ze jiz bude nastaveno u jineho ktere ale nebude upravovat preskocit a vypsat hlasku nebo nejak vyresit automaticky za uzivatele ??
                    if ($novePoradi != "") {
                        $datumVlozeniNavrhu = (int)$hodnotyPOSTu->zmenova_cast_DatumVlozeniNavrhuUpravyPoradiDeti[$rcDitete] != "" ? $hodnotyPOSTu->zmenova_cast_DatumVlozeniNavrhuUpravyPoradiDeti[$rcDitete] : time();
                        $logDescription = 'Rocni vyuctovani dane, zmenova cast odpoctu na dite - portal';
                        $aktualizovatRVDDeti = $this->zapisOdpocetNaDiteNavrh($this->pracovniPomer,
                            $rcDitete,
                            $kalendarniMesicZmenyPuvodni,
                            $kalendarniMesicZmeny,
                            $datumVlozeniNavrhu,
                            $novePoradi,
                            $logDescription, false);
                    }
                }
            }
        }

        //aktualizuje pokud doslo k zapisu zmeny
        if ($aktualizovatRVDDeti || $aktualizovatRVDSlevy){
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"

            $rec->zmenaProhlaseni();

            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_zapsana'));
        }


        if (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev)  && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && $zmenyOdpoctuNavrh->stav == 2 ){
            //Pokud uzivatel odvola zmenu jeste pred schvalenim, pak zmenu smazat

            $this->ukliditNavrhyRocnihoVyuctovaniDane($this->pracovniPomer, (int)$hodnotyPOSTu->zmenova_cast_datumOd_old,'Odstraneni navrhu odpoctu/odpoctu na dite - uzivatel neuplatnuje slevu  Portal', true);
            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_odstranena'));
        } elseif (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev) && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && !isset($zmenyOdpoctuNavrh->stav) ){

            //Pokud v prubehu otevreni formulare mzdovy pracovnik zmenu jiz schvali a uzivatel chce zmenu odvolat
            MessageManager::addErrorMessage(ronText('Prohlaseni_zmenova_cast_jiz_schvalena'));
        }

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception('Chyba pri aktualizaci prohlaseni ' . $exception->getMessage());
        }

        return new Redirect(null,"?menuid=dane&osoba=$idPP");
    }

    function dokumentUpload($castProhlaseni, $rok, $pracovniPomer, $prava, $suffix = '')
    {
        switch ($_FILES[$castProhlaseni . $suffix]['error']) {
            case UPLOAD_ERR_OK :
            case UPLOAD_ERR_NO_FILE: //soubor nemusi byt ve formulari pripojen, kontrola se provadi az po nacteni
                break;

            case UPLOAD_ERR_INI_SIZE :
            case UPLOAD_ERR_FORM_SIZE :
                throw new \Exception( TextManager::_('Err_velky_soubor') );

            case UPLOAD_ERR_PARTIAL :
                throw new \Exception( TextManager::_('Err_nekompletni_soubor') );

            case UPLOAD_ERR_NO_TMP_DIR :
                throw new \Exception( TextManager::_('Err_chybi_slozka') );

            case UPLOAD_ERR_CANT_WRITE :
                throw new \Exception( TextManager::_('Err_chyba_zapisu') );

            case UPLOAD_ERR_EXTENSION :
            default :
                throw new \Exception( TextManager::_(sprintf('Err_chyba_nahrani_souboru', $_FILES[$castProhlaseni . $suffix]['error'])));
        }

        $pocetDokumentu = $_POST[$castProhlaseni . $suffix . "_pocet"] ?? 1;

        for ($i = 1; $i <= $pocetDokumentu; $i++) {
            $index = $i > 1 ? "-$i" : "";

            if (($_FILES[$castProhlaseni . $suffix . $index]['error'] == UPLOAD_ERR_INI_SIZE) || ($_FILES[$castProhlaseni . $suffix . $index]['error'] == UPLOAD_ERR_FORM_SIZE)) {
                throw new Exception(TextManager::_('Err_velky_soubor') . ' (' . $_FILES[$castProhlaseni . $suffix . $index]['name'] . ')');
            }

            if ($_FILES[$castProhlaseni . $suffix . $index]['error'] !== UPLOAD_ERR_OK) {
                return;
            }

            $datumOd = $rok;
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
            Prava::nastavObdobiProPrava($datumOd, $datumDo);

            if (!$this->pracovniPomer->lzeZobrazitPP($prava)) {
                throw new NeniPravoException();
            }

            $typDokumentu = static::vratTypDokumentuProSekci($castProhlaseni);
            $params = $this->database->createParameters();
            $popisTypu = DBUtils::getSqlValue($this->database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            $popisTypu = ronText($popisTypu);
            $popisDokumentu = null;

            $popis = RonPortalSettings::getValue('dane_dokumenty_maska', '%TYPDOKUMENTU% %PORADOVECISLO%');

            if ($suffix !== '') {
                $popisDokumentu = substr($suffix, 1);

                switch ($popisDokumentu) {
                    case 'dary':
                        $popisDokumentu = ronText('zadost_dary_pripojeni_dokumentu');
                        break;
                    case 'uroky':
                        $popisDokumentu = ronText('zadost_uroky_pripojeni_dokumentu');
                        break;
                    case 'penzijni':
                        $popisDokumentu = ronText('zadost_penzijni_pripojeni_dokumentu');
                        break;
                    case 'zivotni':
                        $popisDokumentu = ronText('zadost_zivotni_pripojeni_dokumentu');
                        break;
                    case 'clenske_prispevky':
                        $popisDokumentu = ronText('zadost_clenske_pripojeni_dokumentu');
                        break;
                    case 'zkousky':
                        $popisDokumentu = ronText('zadost_zkousky_pripojeni_dokumentu');
                        break;
                }

                $popisTypu = "{$popisTypu} - {$popisDokumentu}";
            }

            $noveCislo = static::vratNovePoradoveCisloDokumentu($this->database, $pracovniPomer, $rok, $typDokumentu, $popisDokumentu);

            $popis = str_replace(
                ['%JMENO%', '%PRIJMENI%', '%OSOBNICISLO%', '%IDOSOBY%', '%IDPP%', '%DATUM%', '%TYPDOKUMENTU%', '%PORADOVECISLO%'],
                [$pracovniPomer->jmeno, $pracovniPomer->prijmeni, $pracovniPomer->osobniCislo, $pracovniPomer->rc, $pracovniPomer->idPP, date('j.n.Y', time()), $popisTypu, $noveCislo],
                $popis);

            // vlozi do databaze obsah dokumentu
            $id = DokumentyUtils::vlozDokument($this->database, $castProhlaseni . $suffix . $index, $typDokumentu, $popis, $this);

            $rec = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this);
            $rec->addField('Id', 'int', false, 0, $id);
            $rec->addField('RC', '', false, 15, $pracovniPomer->rc);
            $rec->addField('VyuctovaniDane_Rok', 'timestamp', false, 0, $datumOd);
            $rec->addField('VyuctovaniDane_RC', '', false, 15, $pracovniPomer->rc);

            $rec->insertRecord();
        }
    }

    /**
     * Nastavi pole provest rocni vyuctovani na zadam a zobrazi formular, nebo nastavi nezadam a zadost podepise. Provadi se odkazem z browseru.
     * @param $obdobi
     * @param int $zadam
     * @param $idPP
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function nastavProvestRocniVyuctovaniDane($obdobi, $zadam, $idPP)
    {
        Csrf::validateCSRFGetParam();

        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, $idPP);

        if (!($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))) {
            throw new NeniPravoException();
        }

        $rec = new RocniVyuctovaniDaneRecord($this->database, 'Rocni vyuctovani dane zadam/nezadam pres odkaz - portal');

        $rec->nactiVyuctovani($obdobi, $this->pracovniPomer->rc);
        $rec->lzePodepsatZadostElektronicky();

        $rec->nastavProvedeniVyuctovani($zadam);

        //po kliknuti na nezadam se zmeni stav ProvestRocniVyuctovani a podepise
        if ($zadam == 0) {
            $rec->podepsatZadost();
        }

        $rec->doplnHistoriiZadosti($zadam ? ronText('zadost_stav_zadam') : ronText('zadost_stav_nezadam'));

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception(StringUtils::fromUTF8('Chyba při zápisu rocniho zuctovani dane.'));
        }

        //pri stisku zadam se pokracuje primo na formular zadosti
        if ($zadam == 1) {
            return $this->zobrazitFormularZadostiOZuctovaniDane($obdobi, $idPP);
        }

        return new Redirect(null,'?menuid=dane&');
    }

    public function report()
    {
        if (!$_GET['idpp'] || !$_GET['rok']) {
            throw new Exception('Neni zadano rocni vyuctovani pro tisk!');
        }

        Prava::nastavObdobiProPrava(time(), time());

        $idPP = (int)$_GET['idpp'];
        $rok  = DateUtils::dateStringToTimestamp('1.1.' . $_GET['rok']);
        $konecRoku = DateUtils::incDay(DateUtils::incMonth($rok, 12), -1);

        if (($_GET['typ'] == 'prohlaseni')) {
            $reportName = 'DaneProhlaseni';
            if (!$this->pravo_prohlaseni->lzeZobrazitPP($idPP)) {
                throw new NeniPravoException();
            }
        }
        if ($_GET['typ'] == 'vyuctovani') {
            $reportName = 'DaneVyuctovani';
            if (!$this->pravo_vyuctovani->lzeZobrazitPP($idPP)) {
                throw new NeniPravoException();
            }
        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, $reportName, false);
        $result->nastavPP(PracovniPomer::nactiPP($this->database, $_GET['idpp']));
        $result->nastavObdobi(3, $rok, $konecRoku);

        $result->dateParams['rok'] = date('j.n.Y', $rok);

        return $result;
    }

    public function handleAction($action)
    {
        if (!($this->pravo_prohlaseni->jsouNejakePrava() || $this->pravo_vyuctovani->jsouNejakePrava())){
            throw new NeniPravoException();
        }

        TextManager::getInstance()->loadStrings('mzdy.ini');
        TextManager::getInstance()->loadStrings('person.ini');
        HtmlHeaders::getInstance()->addCss('css/mzdy.css');
        HtmlHeaders::getInstance()->addScript('script/mzdy.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=dane&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);

        static::zkontrolujNastaveni();

        $daneaction = isset($_GET['action']) ? $_GET['action'] : '';
        switch ($daneaction) {
            case 'formularzadosti':
                return $this->zobrazitFormularZadostiOZuctovaniDane($_GET['obdobi'], $_GET['osoba']);
            case 'ulozitprohlaseni':
                return $this->zapisProhlaseni();
            case 'zmenovacast':
                return $this->zapisZmenyProhlaseni($_POST['osoba'], $_POST['obdobi']);
            case 'ulozitzadost':
                return $this->zapisZadosti();
            case 'rocnizuctovani':
                return $this->nastavProvestRocniVyuctovaniDane($_GET['obdobi'], $_GET['zadamzuctovani'], $_GET['osoba']);
            case 'report':
                return $this->report();
            default:
                return parent::handleAction($action);
        }
    }

}
