<?php
/**
 * pavel.rudol, 28.12.2022 10:56
 */

namespace RonPortal\Mzdy;

use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;

class RocniVyuctovaniDaneRecord extends RonPortalDBRecord
{
    const PROHLASENI_NAVRH = 0;
    const PROHLASENI_SCHVALENO = 1;
    const PROHLASENI_VRACENO = 2;
    const PROHLASENI_VICE_PP = 3;
    
    const ZADOST_NAVRH = 0;
    const ZADOST_SCHVALENO = 1;
    const ZADOST_VRACENO = 2;
    const ZADOST_VICE_PP = 3;
    const ZADOST_SCHVALENO_UCETNI = 4;

    const LIMIT_UVERU_BYDLENI_PO_2021 = 150000;

    public $rok;
    public $rc;
    public $historieProhlaseniField;
    public $historieZadostiField;

    public function __construct($database, $logDescription)
    {
        parent::__construct($database, 'RocniVyuctovaniDane', $logDescription);

        $this->rok = $this->addTimestampField('Rok', true);
        $this->rc = $this->addStringField('RC', true, 15);

        $this->addStringField('ProhlaseniPortalRC', true, 15);
        $this->addStringField('ProhlaseniPortalOsc', true, 15);
        $this->addTimestampField('ProhlaseniPortalDatNastup', true);

        $this->addIntField('ProhlaseniElektronicky', true);
        $this->addIntField('ZadostElektronicky', true);
        $this->addIntField('Schvaleno', true);

        $this->addTimestampField('DatumZmenyZadosti', true);

        $this->historieProhlaseniField = $this->addMemoField('HistorieProhlaseni', true)->setUpdateLog(false);
        $this->historieZadostiField = $this->addMemoField('HistorieZadost', true)->setUpdateLog(false);

        $this->addIntField('ProhlaseniPortal', true);
        $this->addIntField('ProhlaseniPortalStav', true);
        $this->addMemoField('ElektronickyPodpis', true);
        $this->addTimestampField('DatumProhlaseniPortal', true);

        $this->addIntField('ProvestRocniVyuctovani', true);
        $this->addIntField('ZadostPortalStav', true);
        $this->addMemoField('ElektronickyPodpisZadost', true);
        $this->addTimestampField('DatumZadostiPortal', true);

        //udaje pro prohlaseni
        $this->addIntField('VyzivujeJinyPoplatnik', true);
        $this->addIntField('JinyPoplatnikUplatnujeZv', true);
        $this->addIntField('JinyPoplatnikZamestnani', true);
        $this->addStringField('JinyPoplatnikJmeno', true, 30);
        $this->addStringField('JinyPoplatnikPrijmeni', true, 40);
        $this->addStringField('JinyPoplatnikRC', true, 15);

        $this->addStringField('JinyPoplatnikObec', true, 80);
        $this->addStringField('JinyPoplatnikPSC', true, 10);
        $this->addStringField('JinyPoplatnikUlice', true, 80);

        $this->addStringField('JinyPoplatnikNazevPlatce', true, 120);
        $this->addStringField('JinyPoplatnikObecPlatce', true, 80);
        $this->addStringField('JinyPoplatnikUlicePlatce', true, 80);
        $this->addStringField('JinyPoplatnikPSCPlatce', true, 10);

        //zmenova cast prohlaseni
        $this->addIntField('ProhlaseniPortalZmenaStav', true);

        //udaje pro rocni vyuctovani
        $this->addMemoField('AdresaBytovePotrebyUveru', true);
        $this->addFloatField('OpravaHodnotDaru', true);
        $this->addFloatField('OpravaUrokuZeStavebSpor', true);
        $this->addFloatField('OpravaPenzijnihoPrip', true);
        $this->addFloatField('ZivotniPojisteni', true);
        $this->addFloatField('OpravaClenskychPrispevku', true);
        $this->addFloatField('OpravaUhradZaZkousky', true);
        $this->addMemoField('OsobyUplatnujiciUver', true);
        $this->addMemoField('PredchoziPrijmyKdyOdKoho', true);
        $this->addFloatField('Skolkovne', true);

        $this->addIntField('UplatnujeOdpocetPartnera', true);

        $this->addStringField('SlevaPartnerObdobi', true, 15);
        $this->addIntField('PocetMesicuOdpoctu', true);
        $this->addIntField('PartnerZTPP', true);

        $this->addIntField('AktualizovatVypocet', true);

        $this->addFloatField('OpravaUrokuZeStavebSpor', true);
        $this->addFloatField('UrokZeStavebSporPo2021', true);
        $this->addFloatField('SlevaZaZastavenouExekuci', true);
    }

    /**
     * @return void
     * @throws Exception
     */
    public function lzePodepsatProhlaseniElektronicky()
    {
        if ($this->findFieldByName('ProhlaseniElektronicky')->value == 0) {
            throw new Exception(TextManager::_('Mzdy_elektronicky_podpis_zrusen'));
        }
    }

    /**
     * @return void
     * @throws Exception
     */
    public function jeProhlaseniSchvaleno()
    {
        if ($this->findFieldByName('ProhlaseniPortalStav')->value == self::PROHLASENI_SCHVALENO){
            throw new Exception(TextManager::_('prohlaseni_schvaleno'));
        }
    }

    /**
     * @return void
     * @throws Exception
     */
    public function lzePodepsatZadostElektronicky()
    {
        if ($this->findFieldByName('ZadostElektronicky')->value == 0 || $this->findFieldByName('Schvaleno')->value == 1) {
            throw new Exception(TextManager::_('Mzdy_elektronicky_podpis_zrusen'));
        }
    }

    /**
     * @return void
     * @throws Exception
     */
    public function jeZadostSchvalena()
    {
        if ($this->findFieldByName('ZadostPortalStav')->value == self::ZADOST_SCHVALENO || $this->findFieldByName('ZadostPortalStav')->value == self::ZADOST_SCHVALENO_UCETNI ) {
            throw new Exception(TextManager::_('zadost_schvalena'));
        }
    }

    /**
     * @param int $rok
     * @param string $rc
     * @return bool
     * @throws DBException
     */
    public function nactiVyuctovani(int $rok, string $rc): bool
    {
        $params = $this->database->createParameters();

        $this->setWhere(
            'Rok = ' . $params ->addParameter($this->database->timestampToParam($rok)) .
            ' AND RC = ' . $params ->addParameter($rc), $params);

        return parent::loadRecord();
    }

    /**
     * @return string
     */
    private function podpisUzivatele(): string
    {
        $ip = $_SERVER['REMOTE_ADDR'];
        $ipAdresa = "IP: " . $ip;
        $login = " Login: " . User::getLoginName() . "(" . User::getUserName() . ")";
        $datumCas = "Datum: " . StringUtils::format("datetime", time());

        return $datumCas . ", " . $ipAdresa. ", " . $login;
    }

    /**
     * @param PracovniPomer $pracovniPomer
     * @return void
     */
    public function nastavPracovniPomer(PracovniPomer $pracovniPomer)
    {
        $this->findFieldByName('ProhlaseniPortalRC')->setNewValue($pracovniPomer->rc);
        $this->findFieldByName('ProhlaseniPortalOsc')->setNewValue($pracovniPomer->osobniCislo);
        $this->findFieldByName('ProhlaseniPortalDatNastup')->setNewValue($pracovniPomer->datumNastupu);
    }
    
    /**
     * Nastavi pole ProvestRocniVyctovani na hodnotu Zadam/Nezadam
     * @param int $zadam
     * @return void
     */
    public function nastavProvedeniVyuctovani(int $zadam)
    {
        $this->findFieldByName('ProvestRocniVyuctovani')->setNewValue($zadam);
        $this->findFieldByName('DatumZmenyZadosti')->setNewValue(time());
    }

    /**
     * Podepise prohlaseni dane, nastavi pole ZadostPortalStav, ElektronickyPodpisZadost, DatumZadostiPortal
     * @param int $prohlaseniCini 0/1 - prohlaseni k dani ne/ano
     * @return void
     */
    public function podepsatProhlaseni(int $prohlaseniCini)
    {
        $this->findFieldByName('ProhlaseniPortal')->setNewValue($prohlaseniCini);
        $this->findFieldByName('ProhlaseniPortalStav')->setNewValue(self::PROHLASENI_NAVRH);

        $this->findFieldByName('DatumProhlaseniPortal')->setNewValue(time());

        //23.2.2023 pri opakovanem podpisu po vraceni mzdovou ucetni, zustane elektronicky podpis na puvodni hodnote
        if (empty($this->findFieldByName('ElektronickyPodpis')->originalValue)) {
            $this->findFieldByName('ElektronickyPodpis')->setNewValue($this->podpisUzivatele());
        }
    }

    /**
     * Podepise zadost o rocni vyuctovani dane, nastavi pole ZadostPortalStav, ElektronickyPodpisZadost, DatumZadostiPortal
     * @return void
     */
    public function podepsatZadost()
    {
        $this->findFieldByName('ZadostPortalStav')->setNewValue(self::ZADOST_SCHVALENO);
        $this->findFieldByName('ElektronickyPodpisZadost')->setNewValue($this->podpisUzivatele());
        $this->findFieldByName('DatumZadostiPortal')->setNewValue(time()); //TODO: v puvodnim kodu bylo omezeni na 15.2. aktualniho roku, ma se zachovat, nebo upravit?
    }

    /**
     * Doplni text aktualne provadene cinnosti do pole s historii zadosti o vyuctovani
     * @param string $udalost
     * @return void
     */
    public function doplnHistoriiZadosti(string $udalost)
    {
        $this->historieZadostiField->value =  "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . $udalost
            . "\n" . $this->historieZadostiField->originalValue;
    }

    /**
     * Doplni text aktualne provadene cinnosti do pole s historii prohlaseni
     * @param string $udalost
     * @return void
     */
    public function doplnHistoriiProhlaseni(string $udalost)
    {
        $this->historieProhlaseniField->value =  "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . $udalost
            . "\n" . $this->historieProhlaseniField->originalValue;
    }

    /**
     * @return void
     */
    public function aktualizaceVypoctuVyuctovani()
    {
        if ($this->findFieldByName('OpravaHodnotDaru')->isChanged() ||
            $this->findFieldByName('OpravaUrokuZeStavebSpor')->isChanged() ||
            $this->findFieldByName('OpravaPenzijnihoPrip')->isChanged() ||
            $this->findFieldByName('ZivotniPojisteni')->isChanged() ||
            $this->findFieldByName('OpravaClenskychPrispevku')->isChanged() ||
            $this->findFieldByName('OpravaUhradZaZkousky')->isChanged() ||
            $this->findFieldByName('PocetMesicuOdpoctu')->isChanged() ||
            $this->findFieldByName('Skolkovne')->isChanged()
        ) {
            $this->findFieldByName('AktualizovatVypocet')->setNewValue(1);
        }
    }

    public function zmenaProhlaseni()
    {
        $this->findFieldByName('ProhlaseniPortalZmenaStav')->setNewValue(self::PROHLASENI_NAVRH);
    }
}