<?php
/**
 * Roman.Szablatura RON Software 17.07.2019
 */

namespace RonPortal\Skoleni;


use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Class SchvalovaniPlanuKurzuOsobyFilter
 * @package Ron\RonPortal
 */
class SchvalovaniPlanuKurzuOsobyFilter extends SkoleniFilterForm
{
    /**
     * SchvalovaniPlanuKurzuOsobyFilter constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni', true);

        $this->f_classname = 'filtercontainer';

        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');

        $this ->nastavOsoby(true,
            [$pravoPrihlasit, $pravoSchvalit],
            time(), time()); // prava podle aktualniho casu

        Prava::nastavObdobiProPrava(time(),time());

        $this->nastavDatumOdDo(true, false);

        if ($pravoPrihlasit->jsouNejakePrava() || $pravoSchvalit->jsouNejakePrava()) {
            $this->addFilter((new FilterFormFilter('filtrprihlaseninaplanovaneskoleni'))
                ->addItem('planypodleosoby', TextManager::_('plany_podle_osoby'))
                ->addItem('planypodlekurzu', TextManager::_('plany_podle_kurzu'))
                ->setSessionName('filtrprihlaseninaplanovaneskoleni'),
                true // top - na prvnim radku
            );
        }

        $this ->addDruhSkoleniFilter();

        $this ->addFilter((new FilterFormFilter('action', FilterFormFilter::TYPE_HIDDEN))
            ->setValue('prihlaseninaplanovaneskoleni')
        );

        $this ->addFilter((new FilterFormFilter('aktualniplanykurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_aktualni_plany'))
        );

        $this ->addFilter((new FilterFormFilter('neschvaleneplanykurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_neschvalene'))
            ->setSqlFilter('PlanovaneSkoleniOsob.Potvrzeni = 0')
        );

        $this ->addFilter((new FilterFormFilter('pouzenaplanukurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_na_planu_kurzu'))
            ->setSqlFilter('PlanovaneSkoleniOsob.RC IS NOT NULL')   // TODO: nemelo by tu byt AND PlanovaneSkoleniOsob.Potvrzeni < 2 ? (aby nezobrazoval zamitnute)
        );

    }


//    public function renderPodMenuPrihlaseniNaPlanovaneSkoleni()
//    {
//        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
//            LinksList::renderLinks(
//                [
//                    'planypodleosoby' => \Ron\ronText('plany_podle_osoby'),
//                    'planypodlekurzu' => \Ron\ronText('plany_podle_kurzu')
//                ],
//                'planypodleosoby',
//                '?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=');
//        }
//    }



//    public function renderPouzeNeschvalenePlanyKurzu()
//    {
//        // TODO: predelat priznaky  ze session, pripadne zobrazit list misto checkboxu
//        echo "<div class='filterrow'>";
//        echo '<span class="filtercaption">';
//            GuiUtils::renderCheckbox('aktualniplanykurzu', \Ron\ronText('filtr_pouze_aktualni_plany'),
//                isset($_SESSION['aktualniplanykurzu']) ? $_SESSION['aktualniplanykurzu'] : 0,
//                '', 'clickAutoSubmit');
//        echo "</span>";
//
//        echo '<span class="">';
//            GuiUtils::renderCheckbox('neschvaleneplanykurzu', \Ron\ronText('filtr_pouze_neschvalene'),
//                isset($_SESSION['neschvaleneplanykurzu']) ? $_SESSION['neschvaleneplanykurzu'] : 0,
//                '', 'clickAutoSubmit');
//        echo "</span>";
//
//        echo '<span class="">';
//            GuiUtils::renderCheckbox('pouzenaplanukurzu', \Ron\ronText('filtr_pouze_na_planu_kurzu'),
//                isset($_SESSION['pouzenaplanukurzu']) ? $_SESSION['pouzenaplanukurzu'] : 0,
//                '', 'clickAutoSubmit');
//        echo "</span>";
//        echo "</div>";
//    }


//    private function loadPodMenuPrihlaseniNaPlanovaneSkoleni()
//    {
//        //Moje kurzy odstraneno (na zadost Martina) jelikoz funkcnost nahrazuje "pro osobu"
//        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
//            if (isset($_GET['filtrprihlaseninaplanovaneskoleni'])) {
//                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = $_GET['filtrprihlaseninaplanovaneskoleni'];
//            }
//            //Pri prvnim zobrazeni stranky se implicitne nastavi Pro osobu
//            if (!isset($_SESSION['filtrprihlaseninaplanovaneskoleni'])) {
//                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = 'planypodleosoby';
//            }
//        }
//    }


//    /**
//     * @throws Exception
//     */
//    function loadData()
//    {
//        parent::loadData();
//        $this ->loadDruhSkoleni();
//        $this->loadPodMenuPrihlaseniNaPlanovaneSkoleni();
//
//        if (isset($_GET['adsfilterform'])) {
//            $_SESSION['neschvaleneplanykurzu'] = isset($_GET['neschvaleneplanykurzu']);
//            $_SESSION['aktualniplanykurzu'] = isset($_GET['aktualniplanykurzu']);
//            $_SESSION['pouzenaplanukurzu'] = isset($_GET['pouzenaplanukurzu']);
//        }
//    }

//    /**
//     * @throws DBException
//     */
//    function renderFilters()
//    {
//        $this->renderPodMenuPrihlaseniNaPlanovaneSkoleni();
//
//        parent::renderFilters();
//        $this->renderDruhSkoleni();
//
//        $this->renderPouzeNeschvalenePlanyKurzu();
//
//        echo
//            '<input type="hidden" name="action" value="prihlaseninaplanovaneskoleni"/>' .
//            '<input type="hidden" name="adsfilterform" value="1"/>' .
//            "<input type='hidden' name='filtrprihlaseninaplanovaneskoleni' value='planypodleosoby'/>";
//    }
}


/**
 * Class EditacePlanovanehoSkoleniField
 */
class EditacePlanovanehoSkoleniField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // Prava na editaci planovanych skoleni se urci podle pole "Zadatel" (tj. kdo pozadal o kurz)
        if (Prava::vratPrava('skoleni_plan') ->lzeZobrazitRC($rowData['col-zadatel'], time())) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}



/**
 * Class StavNaKurzuField
 * @package Ron\RonPortal
 */
class StavPrihlaseniNaKurzField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData['col-potvrzeni']){
            case 0:
                return isset($rowData['col-prihlaseni-rc'])
                    ? \Ron\ronText('osoba_plan_kurzu_prihlasen_neschvaleno')
                    : \Ron\ronText('-');
                break;

            case 1:
                return \Ron\ronText('osoba_plan_kurzu_prihlasen_schvaleno');
                break;

            case 2:
                return \Ron\ronText('osoba_plan_kurzu_zamitnuto');
                break;
        }

        return '';
    }
}


/**
 * Class PocetOsobNaKurzuField
 * @package Ron\RonPortal
 */
class PocetOsobNaKurzuField extends BrowserField
{

    /**
     * PocetOsobNaKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this
            ->setHeaderText('kapacita_obsazenost_volne')
            //->setSqlFieldName('Kapacita')
            ->setAlign('center')
            ->setOrdering(true);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $vycerpano = ((int)$rowData['col-pocetosob']-(int)$rowData['col-pocet-prihlasenych']) <= 0 // pocetosob = kapacita
            ? 'minus'
            : '';

        $kapacita = (int)$rowData['col-pocetosob'];
        $volno = ((int)$rowData['col-pocetosob']-(int)$rowData['col-pocet-prihlasenych']);

        return
            "<span>" . $kapacita . "</span>/<span class='$vycerpano'>" . $volno . "</span>";
    }
}


/**
 * Class SchvaleniPrihlaseniNaKurzuField
 * @package Ron\RonPortal
 */
class SchvaleniPrihlaseniNaKurzuField extends BrowserField
{
    /** @var int    Id PP vybrane osoby */
    private $idPP;

    /** @var PravaNaOsoby $pravaPrihlasit */
    public $jePravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $jePravoSchvalit;

    /**
     * SchvaleniPrihlaseniNaKurzuField constructor.
     * @param Database $database
     * @param int $idPP
     * @param bool $jePravoPrihlasit
     * @param bool $jePravoSchvalit
     */
    function __construct($database, $idPP, $jePravoPrihlasit, $jePravoSchvalit)
    {
        parent::__construct($database);

//        $this
//            ->setSqlFieldName('PS.Potvrzeni')
//            ->setOrdering(1);

        $this ->idPP = $idPP;
        $this->jePravoPrihlasit = $jePravoPrihlasit;
        $this->jePravoSchvalit = $jePravoSchvalit;
    }


    /**
     * @param array $rowData
     * @param string $akce  schvalit nebo zamitnout
     * @param string $textOznameniPoznamky
     * @return string
     */
    private function schvalitPrihlaseniLink($rowData, $akce, $textOznameniPoznamky)
    {
        return
            "<span class='pripojit-poznamku-schvaleni js'" .
                " data-ron-akce='$akce'" .
                " data-ron-osoba='{$this ->idPP}'" .
                " data-ron-cislokurzu='" . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . "'" .
                " data-ron-poznamkaschvaleni='" . StringUtils::htmlTextConvert($rowData['col-poznamkaschvalovani']) . "'" .
                ">$textOznameniPoznamky</span>";
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datumPocatkuKurzu = $this->database->stringToTimestamp($rowData['col-datumpocatku']);

        $url = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&cisloskoleni={$rowData['col-cisloskoleni']}&osoba={$this ->idPP}&hromadne=0";

        //Pokud ma pravo schvalit osobu
        $urlSchvaleni = $url . "&plankurzuzamestnance=schvalit";
        //Odhlasit - zrusit (odstrani se zaznam)
        $urlOdhlaseni = $url . "&plankurzuzamestnance=zrusit";
        $urlZamitnuti = $url . "&plankurzuzamestnance=zamitnout";

        $jePrihlasen = $rowData['col-potvrzeni'] == 0 && $rowData['col-prihlaseni-rc'] == 1;
        $jeSchvalen = $rowData['col-potvrzeni'] == 1;
        $jeZamitnut = $rowData['col-potvrzeni'] == 2;

        if ($datumPocatkuKurzu < DateUtils::today()) {
            // kurz uz probiha nebo probehl
            return \Ron\ronText('expirace_kurzu');
        }

        if (!$jeSchvalen  &&  !$jePrihlasen  &&  (int)$rowData['col-pocetosob'] <= (int)$rowData['col-pocet-prihlasenych']) {
            // neni prihlasen ani schvalen a kapacita kurzu je naplnena
            return \Ron\ronText('kapacita_planu_naplnena');
        }

//        if (!$muzeSchvalit  &&  (!$muzePrihlasit || $jeSchvalen)) {
//            // bud nemam zadne pravo, nebo je plan schvalen a nemam pravo (od)schvalovat => jen zobrazit
//            if ($jeSchvalen) {
//                return \Ron\ronText('osoba_je_na_planu');
//            }
//
//            return '';
//        }


        //Pokud ma pravo na schvaleni ale neni zapsan na planu pak nabidnout prihlaseni a schvaleni jinak pouze prihlaseni
        //Pokud je prihlasen a nema pravo schvalit pak zobrazit pouze text uzivatel je prihlasen
        if ($this ->jePravoPrihlasit || !$this ->jePravoSchvalit) {
            if ($jeSchvalen) {
                //Je schvalen -> je i prihlasen
                if ($this ->jePravoSchvalit) {
                    $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                    return "<a href='$urlOdhlaseni'>$textOznameni</a>";
                }

                return \Ron\ronText('osoba_je_na_planu');
            }

            //Neni schvalen
            if ($jePrihlasen) {
                //Je prihlasen, ale neschvalen ->  moznost chvalit nebo ignorovat
                if ($this ->jePravoSchvalit) {
                    $textOznameni = \Ron\ronText('schvalit_plan_skoleni_uzivatel_ma_zajem');
                    $textOznameniPoznamky = \Ron\ronText('schvalit_plan_kurzu_poznamka');

                    $result =
                        "<div>" .
                        "<span class='padding-right-2'><a href='$urlSchvaleni'>$textOznameni</a></span>" .
                        $this->schvalitPrihlaseniLink($rowData, 'schvalit', $textOznameniPoznamky) .
                        "</div>";

                    if (!$jeZamitnut) {
                        $textOznameni = \Ron\ronText('zamitnout');
                        $textOznameniPoznamky = \Ron\ronText('zamitnout_plan_kurzu_poznamka');

                        $result .=
                            "<div>" .
                            "<span class='margin-right-2'><a href='$urlZamitnuti'>$textOznameni</a></span>" .
                            $this->schvalitPrihlaseniLink($rowData, 'zamitnout', $textOznameniPoznamky) .
                            "</div>";
                    }

                    return $result;
                }

                if ($this ->jePravoPrihlasit) {
                    $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                    return "<a href='$urlOdhlaseni'>$textOznameni</a>";
                }

                return \Ron\ronText('osoba_je_prihlasena_na_planu');

            }

            //Neni prihlasen
            if ($this ->jePravoSchvalit) {
                $textOznameni = \Ron\ronText('prihlasit_schvalit_plan_skoleni_uzivatel_neprojevil_zajem');
                $textOznameniPoznamky = \Ron\ronText('prihlasit_schvalit_plan_kurzu_poznamka');
                return
                    "<span class='margin-right-2'><a href='$urlSchvaleni'>$textOznameni</a></span>" .
                    $this->schvalitPrihlaseniLink($rowData, 'schvalit', $textOznameniPoznamky);
            }

            if ($this ->jePravoPrihlasit && !$this ->jePravoSchvalit) {
                $textOznameni = \Ron\ronText('prihlasit_plan_skoleni_uzivatel_neprojevil_zajem');
                $textOznameniPoznamky = \Ron\ronText('prihlasit_plan_kurzu_poznamka');

                return
                    "<span class='margin-right-2'><a title=''  href='$urlSchvaleni'>$textOznameni</a></span>" .
                    $this->schvalitPrihlaseniLink($rowData, 'schvalit', $textOznameniPoznamky);
            }
        }

        return '';
    }
}


/**
 * Class SchvalovaniPlanuOsobyBrowser
 */
class SchvalovaniPlanuOsobyBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;
    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /** @var PravaNaOsoby $pravoPlan */
    public $pravoPlan;



    /**
     * SchvalovaniPlanuKurzuPodrizenehoBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni');

        $this->code = 'browserschvalovaniplanukurzuosoby';
        $this->tableId = 'schvalovaniplanukurzuosoby';
        $this->browserUrl = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=planypodleosoby";
        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        $this->pravoPlan = Prava::vratPrava('skoleni_plan');

        Prava::nastavObdobiProPrava(time(), time());

        $this->filterForm = new SchvalovaniPlanuKurzuOsobyFilter($database);
        $this->filterForm->loadData();

        $fromParameters = $this ->database ->createParameters();
        $this->sqlFromPart = new SqlStatement(
            'PlanovaneSkoleni' .
            ' LEFT OUTER JOIN PlanovaneSkoleniOsob' .
            '  ON PlanovaneSkoleniOsob.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni' .
            '  AND PlanovaneSkoleniOsob.RC = ' . $fromParameters ->addParameter($this ->filterForm ->vybranaOsobaRC),
            $fromParameters
        );

        $commentColSpan = 1;

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleni.CisloSkoleni')
            ->setCode('col-cisloskoleni');
        $this ->primaryField = 0;

        if (Prava::jsouNejakePrava('skoleni_plan')) {
            $commentColSpan++;

            // odkaz na editaci planovaneho skoleni
            $this->addField(new EditacePlanovanehoSkoleniField($this ->database))
                ->setType(BrowserField::FIELD_TYPE_TEXT)
                ->setText('<i class="inline-glyph fa glyph-edit" title="' . \Ron\ronText('editace_planu_kurzu') . '"></i>')
                ->setLink('?menuid=skoleni&action=planovaneskolenikurzy&formaction=edit&record=')
                ->setLinkAttributes(['target' => '_blank'])
                ->setCode('col-detail')
                ->setAlign('center');
        }

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DruhSkoleni') ->setHeaderText('druh_skoleni') ->setOrdering(true)
            ->setCode('col-druhskoleni');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.Popis') ->setHeaderText('popis_planu_kurzu') ->setClassName('font-weight-bold') ->setOrdering(true)
            ->setCode('col-popis');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumPocatkuSkoleni') ->setHeaderText('datum_pocatku_skoleni')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumpocatku');

        $this ->defaultSort = '-col-datumpocatku';

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumUkonceniSkoleni') ->setHeaderText('datum_ukonceni_skoleni')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumukonceni');

        $this->addField(new PocetOsobNaKurzuField($database));

        $this->addField(new StavPrihlaseniNaKurzField($database))
            ->setHeaderText('stav') ->setAlign('center') ->setOrdering(true)
            ->setCode('col-stav-prihlaseni')
            ->setSortSQL('PlanovaneSkoleniOsob.Potvrzeni');

        $muzePrihlasit = $this->pravoPrihlasit->lzeZobrazitRC($this ->filterForm ->vybranaOsobaRC);
        $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC($this ->filterForm ->vybranaOsobaRC);

        $this->addField(new SchvaleniPrihlaseniNaKurzuField($database, $this ->filterForm ->vybranaOsoba, $muzePrihlasit, $muzeSchvalit))
            ->setCode('col-schvaleni');

        // hidden pole - pouzivaji se v kodu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleniOsob.RC')
            ->setCode('col-prihlaseni-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleniOsob.Potvrzeni')
            ->setCode('col-potvrzeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN) // pro kontrolu prav na editaci planovaneho skoleni
            ->setSqlFieldName('PlanovaneSkoleni.Zadatel')
            ->setCode('col-zadatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleni.PocetOsob AS Kapacita')
            ->setCode('col-pocetosob');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO' .
                ' WHERE PSO.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni AND PSO.Potvrzeni < 2 ) AS CelkemPrihlasenych')
            ->setCode('col-pocet-prihlasenych');

        // poznamky (zobrazeny na dalsich radcich)
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleniOsob.PoznamkaSchvalovani)}')
            ->setCode('col-poznamkaschvalovani')
            ->setHeader(TextManager::_('poznamka_schvaleni') . ': ', -$commentColSpan)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');


        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleni.Poznamka)}')
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('poznamka_kurzu') . ': ', -$commentColSpan)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');
            //->setFooterColSpan(1);
    }


    function setFilterForm()
    {
        // uz bylo nastaveno v konstruktoru
    }


    /**
     * @return SqlStatement
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'PlanovaneSkoleni.Stav < 2';

        if ($this ->filterForm ->filtrovatDatum) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter($this ->filterForm ->datumOd) .
                ' AND PlanovaneSkoleni.DatumPocatkuSkoleni <= ' . $params->addDateParameter($this ->filterForm ->datumDo);
        }

        if ($this ->filterForm ->getFilterValue('druhskoleni')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DruhSkoleni = ' . $params ->addParameter($this ->filterForm ->getFilterValue('druhskoleni'));
        }

//        if ($this ->filterForm ->getFilterValue('neschvaleneplanykurzu')) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PlanovaneSkoleniOsob.Potvrzeni = 0';
//        }

        if ($this ->filterForm ->getFilterValue('aktualniplanykurzu')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter(time());
        }

//        if ($this ->filterForm ->getFilterValue('pouzenaplanukurzu')) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PlanovaneSkoleniOsob.RC IS NOT NULL';
//        }


        return new SqlStatement($result, $params);
    }


    public function render()
    {
        parent::render();

        // dialog pro zadani poznamky
        require 'ronportal/skoleni/view_poznamka_schvaleni_planu.php';
    }
}
