<?php
/*
 * UTF8
 * Stanislav Elbl, 08-2014
 */

namespace Ron\RonPortal\XML;

use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;


/**
 * Class XMLFormElement
 * @package Ron\RonPortal\XML
 * @deprecated
 */
class XMLFormElement extends \RonPortal\XML\XMLFormElement
{
}


/**
 * Obecny predek pro prikazy (obvykle elementy, ktere negeneruji zadny obsah, ale provadi nejakou cinnost)
 * Class XMLFormElementCommand
 * @package Ron\RonPortal\XML
 * @deprecated
 */
abstract class XMLFormElementCommand extends \RonPortal\XML\XMLFormElementCommand
{
}



class XMLFormElementIgnore extends XMLFormElement
{
    function getHTMLOutput()
    {
    }

    function event()
    {
    }

    function process()
    {
    }
}


class XMLFormElementText extends XMLFormElement
{
    // tag text
    var $text;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);
        $this ->text = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );

    }


    function getHTMLOutput() {
        return $this -> nahradPromenne($this -> text);
    }
}




// TODO: ... Udelat parametry ... nebude to tak jednoduche, protoze se musi poskladat sql text dotazu
//class XMLFormElementSQLParam extends XMLFormElementCommand
//{
//
//    function doCommand()
//    {
//        $sqlElement = $this ->findParentByName('sqlaction');
//        if (!$sqlElement) return;
//
//        $valueText = $this ->getAttributeValue('value');
//        if (!$valueText) $valueText = $this ->getTextContent(false);
//
//        throw new \Exception('XMLFormElementSQLParam::doCommand neni implementovano!');
//
//    }
//}


class XMLFormElementTry extends XMLFormElement
{
    /** @var XMLFormElementCatch[] */
    var $catchElements = [];

    function getHTMLOutput()
    {
        try {
            return parent::getHTMLOutput();
        } catch (\Exception $e) {
            $result = '';

            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExceptionVariables($e);
                    $result .= $catchElement->getElementsHTMLOutput();
                }
            }

            return $result;
        }
    }

    function event()
    {
        try {
            parent::elementsEvent();
        } catch (\Exception $e) {
            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExceptionVariables($e);
                    $catchElement->elementsEvent();
                }
            }
        }
    }

    function process()
    {
        try {
            parent::processElements();
        } catch (\Exception $e) {
            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExceptionVariables($e);
                    $catchElement->processElements();
                }
            }
        }
    }
}


/**
 * Zpracovava se jen pri zachyceni prislusne podminky - proto je to potomek XMLFormElementIgnore
 * Class XMLFormElementCatch
 * @package Ron\RonPortal\XML
 */
class XMLFormElementCatch extends XMLFormElementIgnore
{
    function __construct($xmlForm, $parent)
    {
        parent::__construct($xmlForm, $parent);
        if ($parent instanceof XMLFormElementTry) {
            $parent ->catchElements[] = $this;
        }
    }


    /**
     * @param \Exception $e
     * @return bool
     */
    function handlesException($e)
    {
        // TODO: umoznit zachytit jen konkretni vyjimky!
        return true;
    }

    /**
     * @param \Exception $e
     */
    function addExceptionVariables($e)
    {
        $this ->setVariableValue('XML_EXCEPTION', $e);
        $this ->setVariableValue('XML_EXCEPTIONMESSAGE', $e ->getMessage());

        if ($e instanceof DBException) $type = 'dbexception';
        else $type = 'exception';

        $this ->setVariableValue('XML_EXCEPTIONTYPE', $type);
    }
}

